/**
 * \file gre/lrbingmaps.h
 * \brief GRE LAYER_BINGMAPS definitions
 *
 * \if NODOC
 * $Id: lrbingmaps.h_v 1.13 2011/09/06 22:27:30 bcolombe Exp $
 *
 * $Log: lrbingmaps.h_v $
 * Revision 1.13  2011/09/06 22:27:30  bcolombe
 * Add ctor that takes mapstyle
 *
 * Revision 1.12  2011/03/07 17:40:10  bcolombe
 * Add DataTipReverseGeocode
 *
 * Revision 1.11  2011/02/24 16:46:25  bcolombe
 * Remove excludeextents member from dispparm
 *
 * Revision 1.10  2011/02/23 18:42:23  bcolombe
 * Add GRECB_Layer
 *
 * Revision 1.9  2011/02/07 23:27:43  bcolombe
 * Add v_DataTipGetString
 * Add v_DataTipGetObjectNamesString
 *
 * Revision 1.8  2011/02/04 18:29:23  bcolombe
 * *** empty log message ***
 *
 * Revision 1.7  2011/02/02 22:46:13  bcolombe
 * Fix typo in ifndef statement
 *
 * Revision 1.6  2011/02/02 21:09:37  bcolombe
 * Make DlgShowTermsOfUse static
 *
 * Revision 1.5  2011/02/01 18:08:54  bcolombe
 * Add ShowTermsOfUse
 *
 * Revision 1.4  2011/01/31 23:03:31  bcolombe
 * Added v_CanRender
 *
 * Revision 1.3  2011/01/31 19:11:57  bcolombe
 * Add dispparm values
 *
 * Revision 1.2  2011/01/28 23:20:50  bcolombe
 * Add layer menu
 *
 * Revision 1.1  2011/01/24 18:09:22  bcolombe
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRBINGMAPS_H
#define	INC_GRE_LRBINGMAPS_H

#ifndef	INC_GRE_LRIMAGE_H
#include <gre/lrimage.h>
#endif

#ifndef INC_SPATMOD_IMAGESOURCEBINGMAPS_H
#include <spatmod/imagesourcebingmaps.h>
#endif

#ifndef INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#include <spatmod/imageresamplemethod.h>
#endif

namespace GRE {
//===================================================================================================================

//! Tileset Layer class
class LAYER_BINGMAPS : public GRE::LAYER_IMAGE {
	public:

		// LAYER_TILESET::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:
				DISPPARM ();

				DISPPARM (
					const DISPPARM& rhs
					);

				virtual ~DISPPARM ();

				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Copy(rhs); } return (*this); }

				const SPATMOD::IMAGE::SOURCE_BINGMAPS::MAPSTYLE& GetMapStyle (
					) const { return (m_MapStyle); }

				SPATMOD::IMAGE::RESAMPLEMETHOD GetResampleMethod (
					) const { return (m_ResampleMethod); }

				bool GetShowDatatipTileInfo (
					) const { return (m_bShowDatatipTileInfo); }

				bool GetShowDatatipImageValue (
					) const { return (m_bShowDatatipImageValue); }

				bool GetShowDatatipDate (
					) const { return (m_bShowDatatipDate); }

				bool GetShowLabels (
					) const { return (m_bShowLabels); }

				//! Determine if has been validated.
				bool IsValidated (
					) const { return (m_IsValidated); }

				void SetMapStyle (
					const SPATMOD::IMAGE::SOURCE_BINGMAPS::MAPSTYLE& MapStyle
					) { m_MapStyle = MapStyle; }

				void SetResampleMethod (
					SPATMOD::IMAGE::RESAMPLEMETHOD ResampleMethod
					) { m_ResampleMethod = ResampleMethod; }

				void SetShowDatatipTileInfo (
					bool ShowDatatipTileInfo
					) { m_bShowDatatipTileInfo = ShowDatatipTileInfo; }

				void SetShowDatatipImageValue (
					bool ShowDatatipImageValue
					) { m_bShowDatatipImageValue = ShowDatatipImageValue; }

				void SetShowDatatipDate (
					bool ShowDatatipDate
					) { m_bShowDatatipDate = ShowDatatipDate; }

				void SetShowLabels (
					bool ShowLabels
					) { m_bShowLabels = ShowLabels; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				bool m_IsValidated;
				bool m_bShowDatatipTileInfo;
				bool m_bShowDatatipImageValue;
				bool m_bShowDatatipDate;
				bool m_bShowLabels;
				SPATMOD::IMAGE::SOURCE_BINGMAPS::MAPSTYLE m_MapStyle;
				SPATMOD::IMAGE::RESAMPLEMETHOD m_ResampleMethod;

				void Copy (const DISPPARM&);

				// SERIALIZABLE overrides
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

				// LAYER::DISPPARM overrides
				virtual ERRVALUE v_Validate ();

				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_TILESET::DISPPARM

		#ifdef USE_MGUI
		//! Show bing maps terms of use agreement.  Returns 0 if agreed, EUserCancel if not
		static ERRVALUE DlgShowTermsOfUse (
			MDLGPARENT dlgparent,
			bool bForceToShow = false	//!< True to ignore if terms have been agreed to before and show dialog
			);

		//! Get string with reverse geocoded location information
		static ERRVALUE DataTipReverseGeocode (
			MISTRING& string,			//!< String to return
			const DPOINT2D& point,		//!< DataTip point in disp coords
			GRE::VIEW *view				//!< View
			);
		#endif

		//! Register this layer type
		static void RegisterType (
			);

		//! Construct layer and insert into list
		explicit LAYER_BINGMAPS (
			GRE::GROUP *group,							//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,	//!< Position in list
			LAYER *reflayer = 0							//!< Reference layer for insertion
			);

		//! Construct layer and insert into list
		explicit LAYER_BINGMAPS (
			GRE::GROUP *group,							//!< Group to contain layer
			SPATMOD::IMAGE::SOURCE_BINGMAPS::MAPSTYLE MapStyle, //!< Map STyle
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,	//!< Position in list
			LAYER *reflayer = 0							//!< Reference layer for insertion
			);

		virtual ~LAYER_BINGMAPS ();

		//! Retrieve layer-specific display parameters.
		//! @return current parameters
		const DISPPARM& GetDispParm (
			) const {
			return (m_dispparm);
			}

		//! Set layer-specific display parameters.
		void SetDispParm (
			const DISPPARM& dispparm,					//!< Parameters to set
			bool bUpdate = true							//!< Redraw view and set flags
			);

	protected:
		DISPPARM m_dispparm;								//!< Layer-specific display parameters
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef LAYER_IMAGE BASECLASS;
		
		class DLG;
		#ifdef USE_MGUI
			DLG *m_pDlgControlPanel;
		#endif

		SPATMOD::IMAGE::SOURCE_BINGMAPS* m_source;
		SIMPLE_ARRAY<INT32> m_ZoomLevelArray;
		bool m_HasError;

		ERRVALUE Draw2D (LAYERDC& layerdc);
		static ERRVALUE DoRequest (const DPOINT2D& pt, MISTRINGLIST& strings);

		#ifdef USE_MGUI
		void OnMenuTool (MGUI::ID, bool);
		#endif

		static void GRECB_Layer (const GRE::MSG *msg, void *vpCommonParm);

		// Overrides from LAYER.
		#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
		virtual void v_MenuAddLayerItems (MGUI::MENU& menu);
		#endif
		virtual bool v_CanRender () const;
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		bool v_DataTipHasEnabled () const;
		virtual void v_DataTipGetObjectNamesString (const DPOINT2D& point, MISTRING& string, int indent, GRE::VIEW *view) const;
		int v_DataTipGetString (const DPOINT2D& point, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images, GRE::VIEW *view);
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual ERRVALUE v_DrawOverlay (LAYERDC& layerdc);
		virtual double v_GetScaleMaxDetail (const DPOINT2D& point, const SPATREF::COORDREFSYS& crs, double DisplayPixelSizeMM) const;
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual ERRVALUE v_UpdateExtents (bool forceupdate);
		
		LAYER_BINGMAPS (const LAYER_BINGMAPS&);
		LAYER_BINGMAPS& operator= (const LAYER_BINGMAPS&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER_BINGMAPS


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRBINGMAPS_H
