/**
 * \file gre/lrimage.h
 * \brief GRE LAYER_IMAGE definitions
 *
 * \if NODOC
 * $Id: lrimage.h_v 1.6 2010/07/15 21:59:13 mju Exp $
 *
 * $Log: lrimage.h_v $
 * Revision 1.6  2010/07/15 21:59:13  mju
 * Moved pipeline methods to layerdc.
 *
 * Revision 1.5  2009/07/24 23:07:39  bcolombe
 * Add InsertZoomFilter that will return the zoom that was used
 *
 * Revision 1.4  2007/12/07 22:20:16  bcolombe
 * Rename InsertCropFilters to InsertZoomFilter
 *
 * Revision 1.3  2007/11/30 20:25:30  bcolombe
 * InsertResampleAndCropFilters have been split into two functions so spatial filters can be placed after crop and before resampling
 *
 * Revision 1.2  2007/10/10 20:38:05  mju
 * Add setResampleMethod.
 *
 * Revision 1.1  2006/11/15 22:15:04  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRIMAGE_H
#define	INC_GRE_LRIMAGE_H

#ifndef	INC_GRE_LAYER_H
	#include <gre/layer.h>
#endif

#ifndef	INC_GRE_LEGENDIMAGESAMPLE_H
	#include <gre/legendimagesample.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SPATMOD {
	namespace IMAGE {
		class STAGE;
		class STAGE_AUTOPTR;
		}
	}
namespace MGD {
	class DEVICE;
	}
#endif


namespace GRE {
//===================================================================================================================

//! Image Layer base class.
class LAYER_IMAGE : public GRE::LAYER {
	public:

		//! Construct layer and insert into list
		LAYER_IMAGE (
			GRE::GROUP *group,								//!< Group to contain layer
			TYPE layertype,									//!< Layer type
			DISPPARM& dispparm,								//!< Reference to layer's internal DISPPARM
			CREATEFLAGS createflags,						//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,	//!< Position in list
			LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		virtual ~LAYER_IMAGE ();

		void SetObjRegion (
			const REGION2D& ObjRegion
			) { m_ObjRegion = ObjRegion; }

		//! Set portion of image to draw
		void SetVisibleExtents (
			const LRECT2D& rect								//!< Extents in columns/rows
			);

	protected:

		LEGEND_IMAGESAMPLE m_ImageSampleLegend;		//!< Image sample legend

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Overrides from LAYER.
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual bool v_HasLegend () const;

		LAYER_IMAGE (const LAYER_IMAGE&);
		LAYER_IMAGE& operator= (const LAYER_IMAGE&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER_IMAGE


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRIMAGE_H
