/**
 * \file gre/uiviewtoolsimple.h
 * \brief GRE::UI::VIEWTOOL_SIMPLE definition
 *
 * \if NODOC
 * $Id: uiviewtoolsimple.h_v 1.7 2011/01/27 18:54:31 mju Exp $
 *
 * $Log: uiviewtoolsimple.h_v $
 * Revision 1.7  2011/01/27 18:54:31  mju
 * Add setDelegateOnDeactivate.
 *
 * Revision 1.6  2010/12/02 16:34:18  mju
 * Add hotkey for template too.
 *
 * Revision 1.5  2010/12/02 16:32:00  mju
 * Add hotkey.
 *
 * Revision 1.4  2010/10/25 20:05:23  mju
 * Add autoTrans parm to ctors.
 * Add setDelegateOnResume.
 *
 * Revision 1.3  2010/10/22 14:24:28  mju
 * Add template version.
 *
 * Revision 1.2  2010/10/21 14:08:48  mju
 * Add setDelegateOnActivate.
 *
 * Revision 1.1  2010/10/20 17:02:57  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_UIVIEWTOOLSIMPLE_H
#define	INC_GRE_UIVIEWTOOLSIMPLE_H

#ifndef	INC_GRE_UIVIEWTOOL_H
#include <gre/uiviewtool.h>
#endif

#ifndef	INC_MGUI_TOOLBASE_H
#include <mgui/toolbase.h>
#endif

namespace GRE {
namespace UI {

//===================================================================================================================

//! Base class for simple tool with associated MGUI tool, maintains transformation on MGUI tool.
class VIEWTOOL_SIMPLE : public GRE::UI::VIEWTOOL {
	public:

		//! Construct with associated MGUI tool.
		VIEWTOOL_SIMPLE (
			GRE::UI::VIEW_CANVAS& view,
			MGUI::TOOL_BASE& guitool,
			const MISTRING& name,
			ICONID iconid,
			bool AutoTrans = true,								//!< Automatically maintain view/tool transformation to match "view" coordinates
			VIEWTOOLID toolid = VIEWTOOLID_Unspecified,
			char HotKey = 0
			);

		//! Construct with associated MGUI tool and customization IDs.
		VIEWTOOL_SIMPLE (
			GRE::UI::VIEW_CANVAS& view,
			MGUI::TOOL_BASE& guitool,
			const MISTRING& name,
			ICONID iconid,
			ICID icidmenu,
			ICID icidbutton,
			bool AutoTrans = true,								//!< Automatically maintain view/tool transformation to match "view" coordinates
			VIEWTOOLID toolid = VIEWTOOLID_Unspecified,
			char HotKey = 0
			);

		virtual ~VIEWTOOL_SIMPLE ();

		//! Enable automatic tool/view transformation.
		//! Transformation is enabled by default.
		void EnableAutoTrans (
			bool enable
			) { m_AutoTrans = enable; }

		//! Set delegate to call when tool becomes active.
		//! Delegate is called just before GUI tool is activated to allow its position to be set.
		void SetDelegateOnActivate (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnActivate = delegate; }

		//! Set delegate to call when tool becomes inactive.
		//! Delegate is called just before GUI tool is activated to allow its position to be set.
		void SetDelegateOnDeactivate (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnDeactivate = delegate; }

		//! Set delegate to call when tool resumed after redraw.
		//! Delegate is called just before GUI tool is resumed to allow its position to be set.
		void SetDelegateOnResume (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnResume = delegate; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef VIEWTOOL_SIMPLE THISCLASS;

		MGUI::TOOL_BASE& m_guitool;
		DELEGATE_VOID_NOPARMS m_DelegateOnActivate;
		DELEGATE_VOID_NOPARMS m_DelegateOnDeactivate;
		DELEGATE_VOID_NOPARMS m_DelegateOnResume;
		bool m_AutoTrans;

		void SetupTrans ();

		virtual bool v_Activate ();
		virtual void v_Deactivate ();
		virtual void v_Suspend ();
		virtual void v_Resume ();

		#endif
	};

//===================================================================================================================

//! Template class for simple tool with associated MGUI tool.
template <typename _TOOLTYPE> class VIEWTOOL_SIMPLE_T : public VIEWTOOL_SIMPLE {
	public:

		//! Construct.
		VIEWTOOL_SIMPLE_T (
			GRE::UI::VIEW_CANVAS& view,
			const MISTRING& name,
			ICONID iconid,
			bool AutoTrans = true,								//!< Automatically maintain view/tool transformation to match "view" coordinates
			VIEWTOOLID toolid = VIEWTOOLID_Unspecified,
			char HotKey = 0
			) : VIEWTOOL_SIMPLE(view,m_guitool,name,iconid,AutoTrans,toolid,HotKey), m_guitool(&view.GetCanvas()) { }

		//! Construct with customization IDs.
		VIEWTOOL_SIMPLE_T (
			GRE::UI::VIEW_CANVAS& view,
			const MISTRING& name,
			ICONID iconid,
			ICID icidmenu,
			ICID icidbutton,
			bool AutoTrans = true,								//!< Automatically maintain view/tool transformation to match "view" coordinates
			VIEWTOOLID toolid = VIEWTOOLID_Unspecified,
			char HotKey = 0
			) : VIEWTOOL_SIMPLE(view,m_guitool,name,iconid,icidmenu,icidbutton,AutoTrans,toolid,HotKey), m_guitool(&view.GetCanvas()) { }

		//! Access MGUI tool (const).
		const _TOOLTYPE& GetGuiTool () const
			{ return (m_guitool); }

		//! Access MGUI tool (non-const).
		_TOOLTYPE& GetGuiTool ()
			{ return (m_guitool); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		_TOOLTYPE m_guitool;
		#endif
	};

//===================================================================================================================

} // End namespace UI
} // End namespace GRE


#endif	INC_GRE_UIVIEWTOOLSIMPLE_H
