/**
 * \file gre/viewpos.h
 * \brief GRE VIEWPOS class definitions
 *
 * \if NODOC
 * $Id: viewpos.h_v 1.6 2011/10/05 21:53:14 mju Exp $
 *
 * $Log: viewpos.h_v $
 * Revision 1.6  2011/10/05 21:53:14  mju
 * Use trans2daffine and remove tile members/methods.
 *
 * Revision 1.5  2008/03/31 17:18:49  mju
 * Add access methods for tileorigin and transformation.
 *
 * Revision 1.1  2007/10/05 14:17:53  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_VIEWPOS_H
#define	INC_GRE_VIEWPOS_H

#ifndef INC_GRE_VIEWPLACE_H
	#include <gre/viewplace.h>
#endif

#ifndef INC_MI32_TRANS2DAFFINE_H
	#include <mi32/trans2daffine.h>
#endif

namespace GRE {
//===================================================================================================================

//! Position of "view" relative to "device"
class VIEWPOS {
	public:
	
		VIEWPOS ();

		~VIEWPOS ();

		//! Get center point.
		const DPOINT2D& GetCenterPoint (
			) const { return (m_ViewPlace.GetPoint()); }

		//! Get CRS for center point.
		const SPATREF::COORDREFSYS& GetCenterCRS (
			) const { return (m_ViewPlace.GetCoordRefSys()); }

		//! Get map scale.
		double GetMapScale (
			) const { return (m_ViewPlace.GetSize()); }

		//! Get view "place".
		const VIEWPLACE& GetPlace (
			) const { return (m_ViewPlace); }

		//! Get transformation from "view" to rendering "device".
		const TRANS2D_AFFINE& GetTransViewDevice (
			) const { return (m_TransViewToDevice); }

		//! Get zoom value (1.0 is full view).
		const DPOINT2D& GetZoom (
			) const { return (m_Zoom); }

		//! Apply scale to zoom value.
		void ScaleZoom (
			double scale
			) { m_Zoom *= scale; }

		//! Set center CRS and point.
		void SetCenterPoint (
			const SPATREF::COORDREFSYS& CenterCRS,
			const DPOINT2D& CenterPoint
			) { m_ViewPlace.SetPoint(CenterCRS,CenterPoint); }

		//! Set center point.
		void SetCenterPoint (
			const DPOINT2D& CenterPoint
			) { m_ViewPlace.SetPoint(CenterPoint); }

		//! Set map scale.
		void SetMapScale (
			double MapScale
			) { m_ViewPlace.SetSize(MapScale); }

		//! Set view "place".
		void SetPlace (
			const VIEWPLACE& ViewPlace
			) { m_ViewPlace = ViewPlace; }

		//! Set tranformation from "view" to rendering "device".
		void SetTransViewDevice (
			const TRANS2D_AFFINE& TransViewDevice
			) { m_TransViewToDevice = TransViewDevice; }

		//! Set zoom with equal X/Y values.
		void SetZoom (
			double zoom
			) { m_Zoom.x = m_Zoom.y = zoom; }

		//! Set zoom.
		void SetZoom (
			const DPOINT2D& zoom
			) { m_Zoom = zoom; }

		//! Validate zoom based on whether allowing different X and Y values.
		void ValidateZoom (
			bool AllowDifferXY
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		VIEWPLACE m_ViewPlace;
		TRANS2D_AFFINE m_TransViewToDevice;		// Transformation between view and device
		DPOINT2D m_Zoom;								// Zoom factor, full view = 1.0
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_VIEWPOS_H
