/**
 * \file mgui/dlgviewlog.h
 * \brief MGUI::DLG_VIEWLOG class definitions
 *
 * \if NODOC
 * $Id: dlgviewlog.h_v 1.4 2008/09/22 12:42:20 mju Exp $
 *
 * $Log: dlgviewlog.h_v $
 * Revision 1.4  2008/09/22 12:42:20  mju
 * Use ctrl_grid instead of ctrl_grid_t.
 *
 * Revision 1.2  2007/08/22 22:37:13  honerkamp
 * Highlight the same pids as the one selected
 *
 * Revision 1.1  2007/08/16 22:37:15  honerkamp
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_DLGVIEWLOG_H
#define	INC_MGUI_DLGVIEWLOG_H

#ifndef	INC_MGUI_DLGSHELL_H
	#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_GRID_H
	#include <mgui/grid.h>
#endif

#ifndef	INC_MGUI_FORMBTXT_H
	#include <mgui/formbtxt.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
	#include <mi32/filepath.h>
#endif

#include <vector>

namespace MGUI {

//! Dialog to view session log files in grid type layout.
//! Initially opens with the current days log.
class DLG_VIEWLOG : public MGUI::DLGSHELL {
	public:
		//! default ctor
		DLG_VIEWLOG (
			);  

		//! ctor with title option
		DLG_VIEWLOG (
			const MISTRING &title
			); 

		//! Set the log file to be viewed. Returns an error if unsuccessful, or the logfile is invalid
		ERRVALUE SetLogPath(
			const FILEPATH &logfile
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
	private:
		typedef DLG_VIEWLOG THISCLASS;

		struct LOGRECORD {
			MISTRING date;
			MISTRING runtime;
			MISTRING pid;
			MISTRING process;
			MISTRING description;
			int recnum;
			bool highlighted;
			};
		
		CTRL_GRID m_LogGrid;
		CTRL_PUSHBUTTON m_SysInfoButton;
		FORM_BUTTONTEXT m_FormSelectFile;
			
		std::vector<LOGRECORD> m_records; 
		FILEPATH m_CurrLogPath;
		MISTRING m_SysInfo;
		int m_OrderedColumn;
		bool m_OrderAscending;

		void OnClickItem(int itemnum, int column, KEYSTATE state);
		void OnColumnClick(int row, int column, KEYSTATE keystate);
		bool OnCustomDraw(CTRL_GRID::CUSTOMDRAW &draw);
		int OnItemCompare(UINT32 row1, UINT32 row2, int col, bool Ascending);
		void OnSelectLogFile();
		void OnShowSysInfo();
		ERRVALUE ReadLogFile(MISTRING &longestdesc); //returns the longest row that was read. used for sizing
		ERRVALUE UpdateGrid();
		void UpdateHighlighted(bool force = 0, int itemnumselected=-1);
		ERRVALUE v_CreateContent();
		#endif
	};

}	//end namespace MGUI
#endif
