/** 
 * \file mgui/formeditcontact.h 
 * \brief Definitions for MGUI::FORM_EDIT_CONTACT classes.
 *
 * \if NODOC
 * $Id: formeditcontact.h_v 1.6 2011/11/23 14:00:39 mju Exp $
 *
 * $Log: formeditcontact.h_v $
 * Revision 1.6  2011/11/23 14:00:39  mju
 * Remove redundant hdrs.
 *
 * Revision 1.5  2007/02/16 17:53:17  scowan
 * Fixed phone section format.
 *
 * Revision 1.1  2006/05/12 19:39:56  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMEDITCONTACT_H
#define	INC_MGUI_FORMEDITCONTACT_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MI32_CONTACTINFO_H
#include <mi32/contactinfo.h>
#endif


namespace MGUI {

class FORM_EDIT_CONTACT : public FORM_COMPOSITE {
	public:
		FORM_EDIT_CONTACT (
			);
	
		~FORM_EDIT_CONTACT (
			);
			
		bool CheckRequiredSet (
			CONTACTINFO::REQUIRED RequiredSet
			);
			
		//! Create form
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			bool ReadOnly = false,						//!< Set to 'true' for a view only form
			CONTACTINFO::REQUIRED RequiredSet = CONTACTINFO::REQUIRED_None,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		void GetContactInfo (
			CONTACTINFO& ContactInfo
			);
	
		void SetContactInfo (
			const CONTACTINFO& ContactInfo
			);

		void SetDelegateOnRequiredSet (
			FastDelegate<void(bool)> delegate
			);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MGUI::LAYOUT_PANE m_PersonPane;
		MGUI::FORM_EDIT_STRING m_ContactPerson;
		MGUI::FORM_EDIT_STRING m_ContactPosition;
		MGUI::FORM_EDIT_STRING m_ContactOrg;
		MGUI::FORM_EDIT_STRING m_ContactAddress;
		MGUI::LAYOUT_PANE m_SCPane;
		MGUI::FORM_EDIT_STRING m_ContactCity;
		MGUI::FORM_EDIT_STRING m_ContactState;
		MGUI::LAYOUT_PANE m_PCPane;
		MGUI::FORM_EDIT_STRING m_ContactPostCode;
		MGUI::FORM_EDIT_STRING m_ContactCountry;
		MGUI::CTRL_GROUPBOX m_PhoneBox;
		MGUI::LAYOUT_PANE m_PhonePane;
		MGUI::FORM_EDIT_STRING m_ContactPhone;
		MGUI::FORM_EDIT_STRING m_ContactFax;
		MGUI::FORM_EDIT_STRING m_ContactEmail;
		MGUI::FORM_EDIT_STRING m_ContactURL;
		MGUI::LAYOUT_PANE m_HOSPane;
		MGUI::FORM_EDIT_STRING m_HoursOfService;
		MGUI::CTRL_PUSHBUTTON m_Instructions;
		MGUI::CTRL_LABEL m_RequiredLabel;

		CONTACTINFO m_ContactInfo;
		CONTACTINFO::REQUIRED m_RequiredSet;
		FastDelegate<void(bool)> m_OnRequiredSet;
		bool m_ReadOnly;
		
		void OnCmdInstructions ();
		void OnUserEdit ();
		
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


class DLG_EDIT_CONTACT : public DLGSHELL {
	public:
		DLG_EDIT_CONTACT (
			const MISTRING& Title,
			bool ReadOnly,
			CONTACTINFO::REQUIRED RequiredSet = CONTACTINFO::REQUIRED_None
			);
	
		~DLG_EDIT_CONTACT (
			);
			
		void GetContactInfo (
			CONTACTINFO& ContactInfo
			);
	
		void SetContactInfo (
			const CONTACTINFO& ContactInfo
			);
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FORM_EDIT_CONTACT m_FormEditContact;
		CONTACTINFO::REQUIRED m_RequiredSet;
		bool m_ReadOnly;

		virtual ERRVALUE v_CreateContent ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
}	// End of MGUI namespace

#endif		//	INC_MGUI_FORMEDITCONTACT_H
