/**
 * \file mgui/shell.h
 * \brief MGUI::SHELL definitions.
 *
 * \if NODOC
 * $Id: shell.h_v 1.51 2011/12/02 22:25:04 mju Exp $
 *
 * $Log: shell.h_v $
 * Revision 1.51  2011/12/02 22:25:04  mju
 * Remove redundant hdr.
 *
 * Revision 1.50  2011/11/22 13:45:03  mju
 * Fix dox dependencies.
 *
 * Revision 1.44  2008/09/17 17:52:27  mju
 * Add setAllowTitleProcessID.
 *
 * Revision 1.43  2008/09/12 21:39:12  mju
 * Add "addSeparator" parm to attachMenuBar.
 *
 * Revision 1.42  2008/04/11 16:15:32  mju
 * Add static findShell.
 * Add queueDelegate.
 *
 * Revision 1.40  2008/04/01 17:30:45  mju
 * Add placement::setPositionTopLeft.
 * Add placement::hasNormalSize.
 *
 * Revision 1.39  2008/01/02 16:36:51  mju
 * Add placement.hasNormalPosition.
 *
 * Revision 1.38  2007/11/29 23:57:28  scowan
 * Added hot key set members to handle hot key support.
 *
 * Revision 1.37  2007/11/26 21:24:23  mju
 * Add optional placementKey to setPlacement.
 *
 * Revision 1.36  2007/11/26 21:08:11  mju
 * Make getDftBackgroundColor static.
 * Make attachMenuBar public.
 *
 * Revision 1.35  2007/11/26 16:54:15  mju
 * Add placement::serialGetItemDef.
 *
 * Revision 1.34  2007/11/02 14:49:41  mju
 * Add getDftBackgroundColor.
 *
 * Revision 1.32  2007/04/12 21:36:37  mju
 * Add X stuff for queuing message.
 *
 * Revision 1.31  2007/04/02 20:55:53  mju
 * Add queueMessage and v_onMessage.
 *
 * Revision 1.30  2006/10/13 22:52:35  dwilliss
 * Added AddDelegateOnClose()
 *
 * Revision 1.25  2006/06/23 21:37:22  mju
 * Add support for status pane.
 *
 * Revision 1.23  2006/06/19 14:21:02  mju
 * Add updateTitle so can show size if large.
 *
 * Revision 1.22  2006/05/01 17:25:07  mju
 * Add dlgShowHelp.
 *
 * Revision 1.21  2006/04/17 23:01:11  scowan
 * Added dlgwizard as a friend.
 *
 * Revision 1.19  2006/03/14 18:00:16  mju
 * Add getPlacementKey.
 *
 * Revision 1.18  2006/03/08 17:50:45  scowan
 * Added show method.
 *
 * Revision 1.17  2005/11/09 17:16:27  mju
 * Make detachShell private, add onNotifyXDestroy.
 *
 * Revision 1.16  2005/10/06 14:10:34  dwilliss
 * added an optional parameter to ForceOnScreen
 *
 * Revision 1.15  2005/10/06 13:51:25  dwilliss
 * Added ForceOnScreen
 *
 * Revision 1.14  2005/09/23 13:15:15  mju
 * Add detachShell.
 *
 * Revision 1.13  2005/09/19 16:56:03  mju
 * Add setDftDlgParent().
 * In docs for setBold indicate must have valid status.
 *
 * Revision 1.12  2005/09/19 14:07:38  mju
 * Add getDftDlgParent.
 *
 * Revision 1.9  2005/09/12 20:20:57  mju
 * Use pane_shell instead of pane_main.
 * Add attachMenuBar.
 *
 * Revision 1.1  2003/05/28 16:08:35  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_SHELL_H
#define	INC_MGUI_SHELL_H

#ifndef	INC_MI32_ICONID_H
#include <mi32/iconid.h>
#endif

#ifndef	INC_MGUI_LAYOUT_H
#include <mgui/layout.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_CRITSEC_H
#include <mi32/critsec.h>
#endif

#if !defined(INC_MI32_XBASE_H) && defined(X_NATIVE)
#include <mi32/xbase.h>
#endif

#if !defined(X_H) && defined(X_NATIVE)
#include <X11/X.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class SERIALIZERITEM;
namespace MGUI {
	class CTRL_SEPARATOR;
	class CTRL_STATUSBAR;
	class DLGSHELL;
	class DLGWIZARD;
	class TOPSHELL;
	class MENU;
	}
#endif


namespace MGUI {

//! Base class for 'shell' windows (application windows and dialogs).
class SHELL {
	public:

		class PLACEMENT;

		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Find SHELL from widget.
		//! @return NULL if not found.
		static SHELL* FindShell (
			Widget widget
			);
		#endif

		//! Get default background color for controls.
		static const COLOR& GetDftBackgroundColor ();

		//! Get default parent for child dialogs.
		//! Usually this will be the most-recently activated SHELL instance.
		static MDLGPARENT GetDftDlgParent ();

		//! Set default parent for child dialogs.
		//! Should only be called after modal dialog closed to restore previous default.
		static void SetDftDlgParent (
			MDLGPARENT dlgparent
			) { s_DftDlgParent = dlgparent; }

		//! Add a delegate to be called when the shell is closed
		//! The delegate must be valid for the life of the dialog
		void AddDelegateOnClose (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Attach menubar to shell.
		//! Once a menu is attached to the shell the original MENU instance does not need to be
		//! retained unless access to menu items is required for enabling/disabling and determining state.
		//! Should only be called from within v_CreateContent.
		void AttachMenuBar (
			MENU& menubar,						//!< Menubar to attach, cannot be changed after attachment
			bool AddSeparator = false		//!< Add horizontal separator line below menubar
			);

		//! Destroy the shell.
		void Destroy ();

		//! Show help dialog with associated help ID.
		void DlgShowHelp (
			) const;

		//! Force the shell on screen.
		//! Does nothing if the shell is already 100% on screen.
		//! If the window is too big to fit and bResizeIfNecessary is true, the
		//! window will be resized so that it fits.
		//! For an MGUI::TOPSHELL derived class, this method should only be 
		//! called after the shell has been opened (or in v_OnOpen()).
		void ForceOnScreen (
			bool bResizeIfNecessary = false
			);

		//! Get MDLGPARENT for use as parent for child dialogs.
		MDLGPARENT GetDlgParent () const;

		//! Get Help ID.
		HELPID GetHelpID (
			) const { return (m_HelpID); }

		//! Get main layout pane.
		LAYOUT_PANE& GetMainPane (
			) { return (m_MainPane); }

		//! Get shell 'placement' on screen.
		void GetPlacement (
			PLACEMENT& placement
			) const;

		//! Get shell pane.
		LAYOUT_PANE_SHELL& GetShellPane (
			) { return (m_ShellPane); }

		//! Get pane for status bar.
		//! Will create pane with default parameters if necessary.
		MGUI::LAYOUT_PANE& GetStatusBarPane ();

		//! Determine if shell has been 'created' yet.
		bool IsCreated () const
			{ return (m_shell != 0); }

		//! Send notification that shell has become active or inactive.
		//! Intended for use by shell implementation only.
		void NotifyActivate (
			bool activated
			);

		//! Set whether to allow process ID to appear in title.
		//! Default is to allow, whether actually appears depends on user preference settings.
		void SetAllowTitleProcessID (
			bool allow
			) { m_AllowTitleProcessID = allow; }

		//! Set the window icon.
		//! The iconid should refer to a 32x32 or larger icon.
		//! For it to show up on the Alt-Tab window, it apparently has to be 48x48.
		void SetIcon (
			ICONID iconid
			);

		//! Set shell 'placement' on screen.
		//! Return 'true' if successful, 'false' if not.
		bool SetPlacement (
			const PLACEMENT& placement,
			const char *PlacementKey = 0	//!< Key to associate with saving changes to position
			);

		//! Set Ini/Registry key for automatic shell placement.
		//! This will automatically load the placement and subsequently
		//! record any changes later made to the placement by the user.
		void SetPlacementByKey (
			const char *PlacementKey
			);

		//! Set title.
		void SetTitle (
			const MISTRING& title			//!< Title string (can be TEXTID also)
			);

		//! Bring the shell to the top of the display and make it the active shell.
		void Show ();

	protected:

		//! Constructor.
		SHELL (
			HELPID helpid
			);

		//! Destructor.
		virtual ~SHELL ();

		//! Call the delegates added with AddDelegateOnClose()
		void CallOnCloseDelegates ();

		//! Create pane at bottom of shell for statusbar.
		//! Will be done automatically by GetStatusBarPane() with default parameters if necessary.
		void CreateStatusBarPane (
			LAYOUT_ORIENTATION orientation = LAYOUT_ORIENTATION_Horizontal,
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get the ICONID of the window's title bar icon.
		//! Will return ICONID__NONE if not set.
		ICONID GetIcon (
			) const { return m_IconID; }

		//! Get placement key.
		const char* GetPlacementKey (
			) const { return (m_PlacementKey); }

		void UpdateTitle ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		friend class MGUI::TOPSHELL;
		friend class MGUI::DLGSHELL;
		friend class MGUI::DLGWIZARD;

		static MDLGPARENT s_DftDlgParent;
		static bool s_ShowPIDInTitle;
		static bool s_ShowSizeInTitle;
		static UINT16 s_TestMaxWidth;
		static UINT16 s_TestMaxHeight;

		Widget m_shell;
		LAYOUT_PANE m_MenuBarPane;
		CTRL_SEPARATOR *m_pMenuBarSeparator;
		LAYOUT_PANE_SHELL m_ShellPane;
		LAYOUT_PANE m_MainPane;
		LAYOUT_PANE m_StatusBarPane;
		HELPID m_HelpID;
		char *m_PlacementKey;
		bool m_AllowClose;
		bool m_AllowTitleProcessID;
		MISTRING m_Title;
		MENU *m_pMenuBar;
		ICONID m_IconID;
		bool m_bIconSent;
		MITHREAD::CRITICALSECTION m_CritSec;
		MILIST<DELEGATE_VOID_NOPARMS> m_DelegateListOnClose;
		MILIST<DELEGATE_VOID_NOPARMS> m_DelegateQueue;

		void AttachShell (Widget shell, bool AllowClose=true);
		void DetachShell ();
		void OnCloseRequest () { v_OnCloseRequest(); }
		static void OnNotifyXDelete (Widget, SHELL*, void*);
		static void OnNotifyXDestroy (Widget, SHELL*, void*);
		static void OnNotifyXEvent (Widget, SHELL*, XEvent*);

		// Unimplemented.
		SHELL (const SHELL&);
		SHELL& operator= (const SHELL&);

		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables

		//! Create shell contents.
		//! For details see MGUI::TOPSHELL or MGUI::DLGSHELL.
		virtual int v_CreateContent () = 0;

		//! Destroy the shell.
		virtual void v_Destroy () = 0;

		//! Called when user requests that shell be 'closed'.
		//! Usually results from an OS or Window Manager notification.
		virtual void v_OnCloseRequest () = 0;

		//! Called when shell becomes active or inactive.
		//! Default implementation does nothing.
		virtual void v_OnActivate (
			bool activated						//!< True if activated, false if deactivated
			);

	};


//! Container to encapsulate shell 'placement' for save/restore.
class SHELL::PLACEMENT {
	public:

		//! Get serialization item definition array.
		static const SERIALIZERITEM* SerialGetItemDef (
			);

		//! Constructor.
		PLACEMENT ();

		const LRECT2D& GetNormalPosition (
			) const { return (m_NormalPosition); }

		bool HasNormalPosition (
			) const { return (m_NormalPosition.IsValid() && m_NormalPosition.xinit >= 0 && m_NormalPosition.yinit >= 0); }

		bool HasNormalSize (
			) const { return (m_NormalPosition.GetWidth() > 16 && m_NormalPosition.GetHeight() > 16); }

		void IniRead (
			INIHANDLE IniHandle,
			const char *IniGroup,
			const char *IniName
			);

		void IniWrite (
			INIHANDLE IniHandle,
			const char *IniGroup,
			const char *IniName
			) const;

		bool IsMaximized (
			) const { return (m_IsMaximized); };

		void SetMaximized (
			bool Maximized
			) { m_IsMaximized = Maximized; }

		void SetNormalPosition (
			const LRECT2D& NormalPosition
			) { m_NormalPosition = NormalPosition; }

		void SetPositionTopLeft (
			INT32 x,
			INT32 y
			) { m_NormalPosition.Set(x,y,x,y); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		bool m_IsMaximized;
		LRECT2D m_NormalPosition;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace MGUI

#endif	// INC_MGUI_SHELL_H
