/**
 * \file mgui/toolcrosshair.h
 * \brief MGUI::TOOL_CROSSHAIR definitions
 *
 * \if NODOC
 * $Id: toolcrosshair.h_v 1.4 2011/12/09 20:22:58 mju Exp $
 *
 * $Log: toolcrosshair.h_v $
 * Revision 1.4  2011/12/09 20:22:58  mju
 * No context in v_draw.
 *
 * Revision 1.3  2011/06/21 13:59:08  mju
 * *** empty log message ***
 *
 * Revision 1.2  2010/11/01 15:39:16  mju
 * Add doRotate/Shear.
 *
 * Revision 1.1  2010/10/22 21:36:22  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TOOLCROSSHAIR_H
#define	INC_MGUI_TOOLCROSSHAIR_H

#ifndef	INC_MGUI_TOOLBASE_H
#include <mgui/toolbase.h>
#endif

namespace MGUI {

//! Crosshair tool allowing optional rotation and shear.
class TOOL_CROSSHAIR : public MGUI::TOOL_LIMITED {
	public:

		//! Constructor.
		explicit TOOL_CROSSHAIR (
			TOOLCANVAS *canvas
			);

		//! Destructor.
		virtual ~TOOL_CROSSHAIR ();

		//! Assignment from DPOINT2D (resets all other values).
		TOOL_CROSSHAIR& operator= (
			const DPOINT2D& rhs
			) { SetPoint(rhs,true); return (*this); }

		//! Get angle of rotation.
		double GetAngleRotate () const
			{ return (m_AngleRotate); }

		//! Get angle of shear.
		double GetAngleShear () const
			{ return (m_AngleShear); }

		//! Get point coordinates.
		const DPOINT2D& GetPoint () const
			{ return (m_PtCenter); }

		//! Set point.
		void SetPoint (
			const DPOINT2D& pt,
			bool reset = true					//!< Reset other values
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// Enumeration constants
		enum ZONE {
			ZONE_NoFunction = 0,
			ZONE_Inside,
			ZONE_Axis0,
			ZONE_Axis90,
			ZONE_Axis180,
			ZONE_Axis270,
			};
		// Tool position values
		DPOINT2D m_PtCenter;
		DPOINT2D m_PtEnd0;
		DPOINT2D m_PtEnd90;
		DPOINT2D m_PtEnd180;
		DPOINT2D m_PtEnd270;
		double m_AngleRotate;
		double m_AngleShear;
		// Values used during interaction
		ZONE m_PointerZone;

		void ComputeEndPoints ();		
		void ComputePointerZone (MGUI::POINT ptCanvas);
		bool DoRotate (MGUI::POINT ptCanvas);
		bool DoShear (MGUI::POINT ptCanvas);
		void UpdateCursor ();

		// TOOL_BASE overrides.
		virtual void v_DoDraw ();
		virtual int v_GenerateRegion (REGION2D& region) const;
		virtual void v_DoMove (double dx, double dy, ADJUSTFLAGS flags);
		virtual void v_OnClear ();
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace MGUI


#endif	// INC_MGUI_TOOLCROSSHAIR_H
