/*****************************************************************************
 *
 * \file mi32/htmlstyle.h
 * \brief HTML::STYLE and support classes
 *
 * $Log: htmlstyle.h_v $
 * Revision 1.9  2011/10/26 17:45:05  mju
 * Revert.
 *
 * Revision 1.7  2008/09/10 21:56:54  mju
 * Fix doxygen warning.
 *
 * Revision 1.6  2007/09/07 16:24:28  dwilliss
 * Made some private methods return bool instead of void
 *
 * Revision 1.5  2007/08/31 20:15:35  dwilliss
 * added some members
 *
 * Revision 1.4  2007/08/29 21:05:09  dwilliss
 * added more style things
 *
 * Revisi1.3  2007/08/29 15:54:20  dwilliss
 * copy/paste error in an enum name
 *
 * Revision 1.2  2007/08/29 15:45:05  dwilliss
 * got rid of include files we didn't need any more
 *
 * Revision 1.1  2007/08/29 15:42:24  dwilliss
 * Initial revision
 *
 *
 * Splitting up htmldoc.h and pulling style into its own include file
 *
 **/

#ifndef INC_MI32_HTMLSTYLE_H
#define INC_MI32_HTMLSTYLE_H

#ifndef INC_MI32_CSSPROPERTIES_H
#include <mi32/cssproperties.h>
#endif

#ifndef INC_MI32_ELEMSTYL_H
#include <mi32/elemstyl.h>
#endif

#ifndef INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef INC_MI32_SCRNUNIT_H
#include <mi32/scrnunit.h>
#endif

#ifndef INC_MGUI_CURSORID_H
#include <mgui/cursorid.h>
#endif

#ifndef INC_MI32_HTMLMEASUREMENT_H
#include <mi32/htmlmeasurement.h>
#endif

#ifndef INC_MI32_ENUM_H
#include <mi32/htmlenum.h>
#endif


class STRUTF8;
namespace MGD {
	class CONTEXT;
	}

namespace HTML {
	class STYLE;
	class ELEMENT;

#ifndef GENERATING_DOXYGEN_OUTPUT
class STYLESHEET;
struct INLINE_POS;
#endif

	//! The definition of a border in a STYLE
	class BORDER {
		public:
			enum TYPE {
				TYPE_None = 0,
				TYPE_Dotted,
				TYPE_Dashed,
				TYPE_Solid,
				TYPE_Double,
				TYPE_Groove,
				TYPE_Ridge,
				TYPE_Inset,
				TYPE_Outset
				};

			enum SIDE {
				SIDE_Top,
				SIDE_Bottom,
				SIDE_Left,
				SIDE_Right
				};

			BORDER();
			BORDER(const BORDER&);

			bool operator==(const BORDER& rhs) const;
			bool operator!=(const BORDER& rhs) const;

			void Combine (
				const BORDER& rhs
				);

			void Draw (
				MGD::CONTEXT* gc,
				SIDE side,
				INT32 x1,
				INT32 y1,
				INT32 x2,
				INT32 y2,
				const ELEMENT* Element,
				const BORDER& PrevBorder,
				const BORDER& NextBorder
				) const;

			const COLOR& GetColor (
				) const { return m_Color; }

			const MEASUREMENT& GetWidth (
				) const { return m_Width; }

			const ENUM<TYPE>& GetType (
				) const { return m_Type; }

			bool IsColorSet (
				) const { return m_bColorSet; }

			//! Return true if any part of the border is set
			bool IsSet (
				) const { return (m_Type.IsSet() || IsColorSet() || m_Width.IsSet()); }

#ifndef GENERATING_DOXYGEN_OUTPUT
		private:
			friend class STYLE;

			void Parse (
				const char* value,
				bool bWidth,
				bool bType,
				bool bColor
				);

			COLOR m_Color;
			MEASUREMENT m_Width;
			ENUM<TYPE> m_Type;
			bool m_bColorSet;

#endif
		};


	//! A style in an HTML document
	//! The style keeps track of which members are actually set
	class STYLE {
		public: 

			enum PSEUDO {
				PSEUDO_None = 0,			//! The base state
				PSEUDO_CurrentState, 	//! Either None, Hover or Active, depending on current state
				PSEUDO_FirstChild,		//! :first-child
				PSEUDO_Hover,				//! :hover
				PSEUDO_Active,				//! :active
				PSEUDO_Link,				//! :link
				PSEUDO_Visited,			//! :visited
				PSEUDO_Before,				//! :before
				PSEUDO_After,				//! :after
				PSEUDO_FirstLine,			//! :first-line
				PSEUDO_FirstLetter,		//! :first-letter
				PSEUDO_NUMPSEUDO
				};

			enum BORDER_COLLAPSE {
				BORDER_COLLAPSE_Separate = 0,
				BORDER_COLLAPSE_Collapse = 1
				};

			enum EMPTY_CELLS {
				EMPTY_CELLS_Hide = 0,
				EMPTY_CELLS_Show = 1
				};

			enum BACKGROUND_REPEAT {
				BACKGROUND_Repeat = 0,
				BACKGROUND_NoRepeat,
				BACKGROUND_RepeatX,
				BACKGROUND_RepeatY
				};

			enum BACKGROUND_POS {
				BACKGROUND_POS_Numeric = 0,	//
				BACKGROUND_POS_Top,
				BACKGROUND_POS_Bottom,
				BACKGROUND_POS_Left,
				BACKGROUND_POS_Right,
				BACKGROUND_POS_Center
				};

			enum DISPLAY {
				DISPLAY_Inline = 0,			//!< Inline
				DISPLAY_Block,					//!< Block layout
				DISPLAY_None,					//!< Not visible. Don't even take up space
				DISPLAY_InlineBlock,			//!< Object is rendered inline, but contents are rendered as a block element.
				DISPLAY_ListItem,
				DISPLAY_Table,
				DISPLAY_TableRow,
				DISPLAY_TableHeaderGroup,
				DISPLAY_TableRowGroup,
				DISPLAY_TableFooterGroup,
				DISPLAY_TableColumn,
				DISPLAY_TableColumnGroup,
				DISPLAY_TableCell,
				DISPLAY_TableCaption,
				DISPLAY_Marker
				};

			enum LISTPOSITION {
				LISTPOSITION_Outside = 0,
				LISTPOSITION_Inside
				};

			enum LISTTYPE {
				LISTTYPE_None = 0,		//!< Not a list item or no marker
				LISTTYPE_Disc,				//!< Default: for lists
				LISTTYPE_Circle,			//!< Outlined circles
				LISTTYPE_Square,			//!< Solid Squares
				LISTTYPE_Decimal,			//!< 1, 2, 3 ...
				LISTTYPE_LowerRoman,		//!< i, ii, iii, ...
				LISTTYPE_UpperRoman,		//!< I, II, III, ...
				LISTTYPE_LowerAlpha,		//!< a, b, c, ...
				LISTTYPE_UpperAlpha		//!< A, B, C, ...
				};

			enum CSSOVERFLOW {
				OVERFLOW_Visible = 0,		//!< (normal)
				OVERFLOW_Hidden				//!< clipped 
				};
				

			enum POSITION {
				POSITION_Static = 0,			//!< (normal)
				POSITION_Absolute,			//!< object is positioned relative to parent's position, or to the body object if parent is not positioned.
				POSITION_Relative				//!< object is positioned according to normal flow, then offset by top and left properties
				};

			enum VISIBILITY {
				VISIBILITY_Inherit = 0,		//!< Inherit from parent element
				VISIBILITY_Visible = 1,		//!< Item is visible
				VISIBILITY_Hidden				//!< Item is not visible but still takes up space
				};

			enum WHITESPACE {
				WHITESPACE_Normal = 0,		//!< Default
				WHITESPACE_NoWrap,			//!< Line breaks are suppressed.
				WHITESPACE_Pre					//!< Whitespace is preserved
				};

			enum CLEAR {
				CLEAR_None = 0,	// default
				CLEAR_Left,			// object is moved below any floating objects on left
				CLEAR_Right,		// object is moved below any floating objects on right
				CLEAR_Both			// object is moved below any floating objects
				};

			enum TEXT_TRANSFORM {
				TEXT_TRANSFORM_None = 0,
				TEXT_TRANSFORM_Capitalize,
				TEXT_TRANSFORM_UpperCase,
				TEXT_TRANSFORM_LowerCase
				};

			enum FLOAT {
				FLOAT_None = 0,
				FLOAT_Left,
				FLOAT_Right
				};

			enum VALIGN {
				VALIGN_Baseline = 0,
				VALIGN_Auto,				//!< Aligns according to the layout-flow setting
				VALIGN_Sub,
				VALIGN_Super,
				VALIGN_Top,
				VALIGN_Middle,
				VALIGN_Bottom,
				VALIGN_TextTop,
				VALIGN_TextBottom,
				VALIGN_Numeric			//!< Baseline set numerically, use GetVAlignNum() to get it
				};

			class EDGE {
				public:
					EDGE(BORDER::SIDE side);
					EDGE(const EDGE&);

					// Combine the styles of another EDGE style with this one.
					void Combine (
						const EDGE&
						);

					// Get the border settings for this edge.
					const BORDER& GetBorder (
						) const { return m_Border; }

					// Get the margin settings for this edge.
					const MEASUREMENT& GetMargin (
						) const { return m_Margin; }

					// Get the offset settings for absolute or relative positioning.
					const MEASUREMENT& GetOffset (
						) const { return m_Offset; }

					// Get the padding settings for this edge.
					const MEASUREMENT& GetPadding (
						) const { return m_Padding; }

					//! Get the total width of the edge, including
					//! margin, padding and border
					double GetWidth (
						MGD::CONTEXT* gc,
						const ELEMENT*
						) const;

					//! Return false if any members are set which cannot be inherited
					//! from a parent element.  Otherwise return true.
					bool IsInheritable (
						) const;

					//! See if this border would result in the same size as another border.
					//! This compares fonts, positions, margins, padding, borders,
					//! etc. and returns true if the styles are the same in all those
					//! ways. 
					bool CompareSize (
						const EDGE& other
						) const;

#ifndef GENERATING_DOXYGEN_OUTPUT
				private:
					friend class STYLE;
					BORDER m_Border;
					MEASUREMENT m_Margin;
					MEASUREMENT m_Padding;
					MEASUREMENT m_Offset;
#endif

				};

			class SELECTOR {
				public:
					SELECTOR();
					SELECTOR(const MISTRING&);
					SELECTOR(const SELECTOR&);
					SELECTOR& operator=(const SELECTOR&rhs);
					bool operator==(const SELECTOR& rhs) const;
					bool operator!=(const SELECTOR& rhs) const;
					~SELECTOR();

					//! Get the selector as text suitable for outputting to a
					//! style sheet. 
					//! Does not clear str
					void GetText (
						MISTRING& str
						) const;

					//! Returns true if selector obeys CSS rules for selector syntax
					bool IsValid (
						) const;

					STYLE::PSEUDO GetPseudo (
						) const;
#ifndef GENERATING_DOXYGEN_OUTPUT
				private:
					friend class STYLESHEET;
					class PRIV;
					PRIV* m_priv;
#endif
				};

			//!  ctor
			STYLE(const char* selector = 0, const char* css = 0);
			STYLE(const STYLE&);

			//! Increment the refcount of this style
			void AddRef (
				);

			//! Combine another style into this style.
			//! The members of the rhs style which are set are copied into this
			//! style.
			void Combine (
				const STYLE& rhs
				);

			//! Return true if everything set in this style can be inherited from
			//! a parent element.  Returns false if any of the non-inheritable
			//! parts are set.
			bool IsInheritable (
				) const;


			//! Generate the CSS for this style. 
			//! If bInline is true (the default), it assumes that the resulting
			//! string should be all on one line and that the caller will enclose
			//! the string in quotes (caller can specify which type of quotes)
			//! If bInline is false, the style will be output on multiple lines
			//! in a form suitable for inclusion in a css file or style element.
			void GenerateCSS (
				MISTRING& css,
				bool bInline = true,
				char quote = '\''
				) const;

			// Get the border spacing for tables
			const MEASUREMENT& GetBorderSpacing (
				) const { return m_BorderSpacing; }

			const MEASUREMENT& GetMeasurement (
				CSS_PROPERTY property
				) const;

			const TEXTSTYLEDIFF& GetTextStyle (
				) const { return m_TextStyle; }


			void GetTextStyle (
				TEXTSTYLE& TextStyle,
				MGD::CONTEXT* gc,
				const ELEMENT* Element
				) const;

			void Combine (
				const EDGE& rhs
				);

			//! See if this style would result in the same size as another style.
			//! This compares fonts, positions, margins, padding, borders,
			//! etc. and returns true if the styles are the same in all those
			//! ways. 
			bool CompareSize (
				const STYLE& other
				) const;

			// Get an edge.
			// This is useful when you want to loop through all 4 edges
			const EDGE& GetEdge (
				int num	//!< edge number 0-3 (don't assume anything about order)
				) const;

			// Get an edge.
			// This is useful when you want to loop through all 4 edges
			EDGE& GetEdge (
				int num	//!< edge number 0-3 (don't assume anything about order)
				);

			const EDGE& GetLeftEdge (
				) const { return m_Left; }

			const EDGE& GetRightEdge (
				) const { return m_Right; }

			const EDGE& GetTopEdge (
				) const { return m_Top; }

			const EDGE& GetBottomEdge (
				) const { return m_Bottom; }
			
			const MEASUREMENT& GetHeight (
				) const { return m_Height; }

			const MEASUREMENT& GetWidth (
				) const { return m_Width; }

			const MEASUREMENT& GetVAlignNum (
				) const { return m_VAlignNum; }

			// overide of size in m_TextStyle
			const MEASUREMENT& GetFontSize (
				) const { return m_FontSize;	}

			const MEASUREMENT& GetLineHeight (
				) const { return m_LineHeight;	}

			// Get the Text Indent
			const MEASUREMENT& GetIndent (
				) const { return m_Indent; }

			INT32 GetTabStop (
				) const { return m_TabStop; }

			INT32 GetZIndex (
				) const { return m_ZIndex; }

			const MISTRING& GetBackgroundImage (
				) const { return m_BackgroundImage; }

			const MISTRING& GetListStyleImage (
				) const { return m_ListStyleImage; }

			const COLOR& GetBGColor (
				) const { return m_BGColor; }

			const ENUM<BACKGROUND_POS>& GetBackgroundPosX (
				) const { return m_BackgroundPosX; }

			const MEASUREMENT& GetBackgroundPosNumX (
				) const { return m_BackgroundPosNumX; }

			const ENUM<BACKGROUND_POS>& GetBackgroundPosY (
				) const { return m_BackgroundPosY; }

			const MEASUREMENT& GetBackgroundPosNumY (
				) const { return m_BackgroundPosNumY; }

			const ENUM<BACKGROUND_REPEAT>& GetBackgroundRepeat (
				) const { return m_BackgroundRepeat; }

			const ENUM<TEXT_TRANSFORM>& GetTextTransform (
				) const { return m_TextTransform; }

			const ENUM<BORDER_COLLAPSE>& GetBorderCollapse (
				) const { return m_BorderCollapse; }

			const ENUM<CLEAR>& GetClear (
				) const { return m_Clear; }

			const MISTRING& GetContent (
				) const { return m_Content; }

			const MISTRING& GetCounterIncrement (
				) const { return m_CounterIncrement; }

			const MISTRING& GetCounterReset (
				) const { return m_CounterReset; }

#ifdef USE_MGUI
			const ENUM<MGUI::CURSORID>& GetCursor (
				) const { return m_Cursor; }
#endif

			const ENUM<DISPLAY>& GetDisplay (
				) const { return m_Display; }

			const ENUM<EMPTY_CELLS>& GetEmptyCells (
				) const { return m_EmptyCells; }

			const ENUM<FLOAT>& GetFloat (
				) const { return m_Float; }

			const ENUM<LISTPOSITION>& GetListPosition (
				) const { return m_ListPosition; }

			const ENUM<LISTTYPE>& GetListType (
				) const { return m_ListType; }

			const MEASUREMENT& GetMarkerOffset (
				) const { return m_MarkerOffset; }

			bool GetMinimizeWidth (
				) const { return m_bMinimizeWidth; }

			const MISTRING& GetOpenTypeFeatures (
				) const { return m_OpenTypeFeatures; }

			const ENUM<CSSOVERFLOW>& GetOverflow (
				) const { return m_Overflow; }

			const ENUM<POSITION>& GetPosition (
				) const { return m_Position; }

			const MISTRING& GetQuotes (
				) const { return m_Quotes; }

			const SELECTOR& GetSelector (
				) const { return m_selector; }

			const ENUM<VALIGN>& GetVAlign (
				) const { return m_VAlign; }

			const ENUM<VISIBILITY>& GetVisibility (
				) const { return m_Visibility; }

			const ENUM<WHITESPACE>& GetWhitespace (
				) const { return m_Whitespace; }

			bool IsBGColorSet (
				) const { return m_bBGColorSet; }

			bool IsMinimizeWidthSet (
				) const { return m_bMinimizeWidthSet; }

			//! Returns true if this style is shared by multiple elements
			bool IsShared (
				) const { return m_bIsShared; }

			//! Returns true if the style is for a PSEUDO state other than normal.
			bool IsSpecialState (
				) const { return m_bSpecialState; }

			//! Parse a bit of CSS into a style.
			//! This can be the parts inside quotes on a style="" attribute of
			//! an html element, or the part of the style between { } in the
			//! \<style\> section.
			void Parse(const MISTRING& str);

			//! Convert a series of {~} codes to CSS styles.
			//! Any codes not handled are returned in UnhandledCodes
			void ParseTildeCodes (
				const MISTRING& codes,
				MISTRING& UnhandledCodes
				);

			//! Decrement the refcount of this style and delete it if no longer used.
			//! Note: you cannot delete a STYLE directly.  You *must* use
			//! AddRef() and Release()
			void Release (
				);

			void SetBGColor (
				const COLOR& color
				) {
				m_BGColor = color;
				m_bBGColorSet = true;
				}

			void SetTextStyle (
				const TEXTSTYLE& TextStyle
				) { 
				m_TextStyle = TextStyle; 
				m_FontSize.Set(TextStyle.Height, static_cast<SCREENUNIT>(TextStyle.HeightUnits));
				m_LineHeight.Set(TextStyle.LineSpace, static_cast<SCREENUNIT>(TextStyle.SpaceUnits));
				}

			bool ParseFontSize (
				const STRUTF8& str
				);

			bool ParseFontStyle (
				const STRUTF8& str
				);

			bool ParseFontVariant (
				const STRUTF8& str
				);

			bool ParseFontWeight (
				const STRUTF8& str
				);

			void ParseLineHeight (
				const STRUTF8& str
				);

			bool ParseFont (
				const MISTRING& mistr
				);

#ifndef GENERATING_DOXYGEN_OUTPUT
		private:
			friend class STYLESHEET;
			friend class HTMLDOC;

			INT32 m_RefCount;
			bool m_bIsShared;	// True if it's in a style sheet

			SELECTOR m_selector;
			TEXTSTYLEDIFF m_TextStyle;

			EDGE m_Left;
			EDGE m_Right;
			EDGE m_Top;
			EDGE m_Bottom;
			MEASUREMENT m_Height;
			MEASUREMENT m_Width;
			MEASUREMENT m_VAlignNum;
			MEASUREMENT m_Indent;

			MEASUREMENT m_FontSize;	// overide of size in m_TextStyle
			MEASUREMENT m_LineHeight;	// overide of size in m_TextStyle
			MEASUREMENT m_BorderSpacing;
			INT32 m_ZIndex;
			INT32 m_TabStop;

			ENUM<BACKGROUND_POS> m_BackgroundPosX;
			MEASUREMENT m_BackgroundPosNumX;
			ENUM<BACKGROUND_POS> m_BackgroundPosY;
			MEASUREMENT m_BackgroundPosNumY;
			ENUM<BACKGROUND_REPEAT> m_BackgroundRepeat;
			ENUM<CLEAR> m_Clear;
#ifdef USE_MGUI
			ENUM<MGUI::CURSORID> m_Cursor;
#endif
			ENUM<DISPLAY> m_Display;
			ENUM<EMPTY_CELLS> m_EmptyCells;
			ENUM<FLOAT> m_Float;
			ENUM<LISTPOSITION> m_ListPosition;
			ENUM<LISTTYPE> m_ListType;
			ENUM<CSSOVERFLOW> m_Overflow;
			ENUM<BORDER_COLLAPSE> m_BorderCollapse;
			bool m_bMinimizeWidth;
			bool m_bMinimizeWidthSet;
			ENUM<POSITION> m_Position;
			ENUM<TEXT_TRANSFORM> m_TextTransform;
			ENUM<VALIGN> m_VAlign;
			ENUM<VISIBILITY> m_Visibility;
			ENUM<WHITESPACE> m_Whitespace;
			COLOR m_BGColor;
			bool m_bBGColorSet;
			MISTRING m_BackgroundImage;	// just a "url"
			MISTRING m_ListStyleImage;		// just a "url"
			bool m_bSpecialState;

			MISTRING m_CounterIncrement;
			MISTRING m_CounterReset;
			MISTRING m_Content;
			MISTRING m_Quotes;
			MISTRING m_OpenTypeFeatures;
			MEASUREMENT m_MarkerOffset;

			// This one can only be called from STYLESHEET::FindStyle.
			// It copy constructs a style but only copies the parts which can
			// be inherited.  The bool parameter is ignored.  It's just there to
			// force it to call this ctor rather than the normal copy ctor
			STYLE(const STYLE&, bool bInheritable);

			void ParseFontStyleFlag (
				const char* value,
				const char* expected,
				UINT32 flag
				);

			void SetSelector (
				const SELECTOR& selector
				);

			~STYLE();	
#endif // doxygen
		};

	} // end of HTML namespace


#endif

