/**
 * \file mcb.h <mi32/mcb.h>
 * \brief Include file for Mcb (MicroImages Callback) toolkit
 *
 * \if NODOC
 * $Id: mcb.h_v 1.29 2011/11/02 13:32:16 mju Exp $
 *
 * $Log: mcb.h_v $
 * Revision 1.29  2011/11/02 13:32:16  mju
 * Remove unused macros.
 *
 * Revision 1.28  2011/11/02 13:25:11  mju
 * Remove redundant hdr and clean up some dox.
 *
 * Revision 1.27  2006/02/16 16:36:56  dwilliss
 * replaced an ifdef's define with one that's not deprecated
 *
 * Revision 1.6  1996/12/12  22:20:31  mju
 * \endif
**/

#ifndef	INC_MI32_MCB_H
#define	INC_MI32_MCB_H

#ifndef	INC_MI32_MICALLBACKLIST_H
	#include <mi32/micallbacklist.h>
#endif

#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif


//! Base for any callback structures.
struct McbAnyCallbackStruct {
	UINT32 reason;

	McbAnyCallbackStruct (					//! Construct with specified reason value
		UINT32 ireason = 0
		):
		reason(ireason)
		{ }
	};


typedef void (*McbCallbackFunc)(McbAnyCallbackStruct*,void*);


#if defined(__cplusplus)

extern "C" {
#endif

struct McbLIST;

#define	MCB_PriorityNormal	0

//! Add callback to callback list with position determined by priority.
//!
//! If reasonmask is 0 all reasons will be selected.
//! \deprecated.  Use MICALLBACKLIST instead of McbLIST
DEPRECATED LIBEXPORT int McbAddCallbackExt (
	McbLIST *cblist, 							//!< Callback list to add to
	McbCallbackFunc cbfunc, 				//!< Callback function
	void *cbdata, 								//!< "Hook" data to pass to callback function
	UINT32 reasonmask, 						//!< Programmer-defined reasons for which callback will be called
	INT8 priority								//!< Priority
	);

//!	Add callback to callback list.
//!
//! \deprecated.  Use MICALLBACKLIST instead of McbLIST
DEPRECATED LIBEXPORT int McbAddCallback (
	McbLIST *cblist, 							//!< Callback list to add to
	McbCallbackFunc cbfunc, 				//!< Callback function
	void *cbdata, 								//!< "Hook" data to pass to callback function
	UINT32 reasonmask							//!< Programmer-defined reasons for which callback will be called
	);

//! Call callbacks in list selected by reason.
//!
//! The specified reason will be bitwise-anded with the reasonmask which was specified in
//!	McbAddCallback() to determine if the callback function will actually be called.  You can
//!	use McbAnyCallbackStruct for the structure type if you are only interested in specifying
//!	a callback reason and no additional information.
//! \deprecated.  Use MICALLBACKLIST instead of McbLIST
DEPRECATED LIBEXPORT void McbCallCallbacks (
	McbLIST *cblist, 							//!< Callback list
	McbAnyCallbackStruct *cbstruct, 		//!< Pointer to callback structure, first entry must be ULONG reason
	UINT32 reason								//!< Reason, will be ANDed with callback reasonmask
	);

//!	Check to see if callback list has a callback for a given reason.
//!
//! @return True if there is a callback in the callback list which matches the given reason.
//! \deprecated.  Use MICALLBACKLIST instead of McbLIST
DEPRECATED LIBEXPORT int McbHasCallbacks (
	McbLIST *cblist, 							//!< Callback list
	UINT32 reason								//!< Reason, will be ANDed with callback reasonmask
	);

//! Remove all callbacks from list.
//! \deprecated.  Use MICALLBACKLIST instead of McbLIST
DEPRECATED LIBEXPORT void McbRemoveAllCallbacks (
	McbLIST *cblist							//!< Callback list to remove all callbacks from
	);

//! Remove callback from list.
//!
//! If "cbfunc" or "cbdata" are NULL and/or the "reasonmask" is 0 they will not be used to "match"
//!	the callbacks.  For example, McbRemoveCallback(&cblist,NULL,data,0) will remove all callbacks
//!	that reference the specified "data".  This can be useful if that data is about to be freed.
//!	Selected callback "reasons" can be removed without removing the entire callback as well.
//! Callbacks can be removed from the list safely while being called.
//! \deprecated.  Use MICALLBACKLIST instead of McbLIST
DEPRECATED LIBEXPORT void McbRemoveCallback (
	McbLIST *cblist,							//!< Callback list to remove to 
	McbCallbackFunc cbfunc, 				//!< Callback function
	void *cbdata, 								//!< "Hook" data specified in McbAddCallback()
	UINT32 reasonmask							//!< Programmer-defined reasons specified in McbAddCallback()
	);

#if defined(__cplusplus)
}
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// This is internal, so don't document.
struct McbREC {
	McbCallbackFunc cbfunc;
	void *cbdata;
	UINT32 ReasonMask;
	INT8 Priority;
	McbREC *next;
	};
#endif //!< GENERATING_DOXYGEN_OUTPUT

//! Callback list.
//! \deprecated Use MICALLBACKLIST instead
DEPRECATED struct McbLIST {
	// MEMBERS
	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! These members are really internal, so we don't need to document them
	McbREC *first;
	McbREC *last;
	UINT16 callcount;
	UINT16 didremove;
	#endif //!< GENERATING_DOXYGEN_OUTPUT

	// CONSTRUCTION / DESTRUCTION

	McbLIST (
		):
		first(0),
		last(0),
		callcount(0),
		didremove(0)
		{ }

	~McbLIST (
		) {
		#ifdef WIN32
		#pragma warning (disable:4996)
		#endif
		McbRemoveAllCallbacks(this);
		#ifdef WIN32
		#pragma warning (default:4996)
		#endif
		}
	};

#undef LIBEXPORT

#endif // INC_MI32_MCB_H 
