/**
 * \file mi32/mdlg.h
 * \brief Definitions for standard dialogs
 *
 * \if NODOC
 * $Id: mdlg.h_v 1.127 2012/01/27 22:21:43 mju Exp $
 *
 * $Log: mdlg.h_v $
 * Revision 1.127  2012/01/27 22:21:43  mju
 * Remove ifdef checks.
 *
 * Revision 1.126  2011/11/02 13:51:30  mju
 * Remove redundant hdr.
 *
 * Revision 1.123  2010/01/08 19:13:16  mju
 * Add yes/noToAll buttons for mdlgquestion.
 *
 * Revision 1.120  2009/10/26 13:46:59  mju
 * Add mdlgGetFiles without custom control parms.
 *
 * Revision 1.115  2008/11/26 15:59:24  mju
 * Add mistring prompt variants for all mdlgGetFile fns and deprecated ones with void* prompt.
 *
 * Revision 1.114  2008/11/25 20:59:03  mju
 * Add mdlgGetFile variant using mistring prompt.
 *
 * Revision 1.109  2007/08/03 14:00:33  mju
 * Add messageflag_continueButton.
 *
 * Revision 1.102  2007/01/26 18:40:17  scowan
 * Added error code show dialogs that take an errorstate class reference.
 *
 * Revision 1.99  2006/11/29 20:43:06  dfriberg
 * Add updated GetFileSet functions
 *
 * Revision 1.94  2006/01/04 21:33:04  mju
 * Add mdlgSaveAsTextFile.
 *
 * Revision 1.90  2005/08/02 22:10:15  scowan
 * Added function to ask for file name component for new files.
 *
 * Revision 1.1  1998/12/28  16:29:52  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MDLG_H
#define	INC_MI32_MDLG_H

#ifndef	INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif

#ifdef WIN32
#pragma warning (disable:4996)	// Disable warning about deprecated declaration within header
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class ERRORSTATE;
class FILEPATH;
class FILEPATHLIST;
class GETSTRINGFILTER;
#endif	// GENERATING_DOXYGEN_OUTPUT


//-------------------------------------------------------------------------------------------------------------------
//! Base class for filter used in file/folder selection dialogs.
class GETFILEFILTER {
	public:

		//! Constructor.
		GETFILEFILTER (
			) { }

		//! Destructor.
		virtual ~GETFILEFILTER (
			) { }

		//! Method called when user attempts to select a file (or folder).
		//! @return 'true' if selection acceptable, 'false' if not.
		virtual bool OnSelection (
			const FILEPATH& path
			) = 0;

		//! Set message returned by filter using message lookup from [getfile] group.
		void SetFilterMsg (
			const char *msgkey
			);

		//! Set message returned by filter using MIUNICODE string.
		void SetFilterMsg (
			const MIUNICODE* string
			);

	private:
	   #ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_msgstr;

		GETFILEFILTER (const GETFILEFILTER&);
		GETFILEFILTER& operator= (const GETFILEFILTER&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//-------------------------------------------------------------------------------------------------------------------
//		Definitions for MdlgGetFile...														
//-------------------------------------------------------------------------------------------------------------------

typedef int (*MdlgGetFilesValidMultiProto) (const FILEPATHLIST& list, void*, char*);
typedef int (*MdlgGetFileFilterProto) (MIUNICODE*, char*, void*);

//		Return values for MdlgGetFileFilterProto
#define	FILT_NotUsable					0	//!< Not usable
#define	FILT_NotSelectable			FILT_NotUsable	//!<  Show on list but don't let user select 
#define	FILT_Usable						1	//!< Usable
#define	FILT_Selectable				FILT_Usable
#define	FILT_FolderUsable				5	//!< Folder is usable

enum GETFILEFLAGS {
	GETFILEFLAG_None =				0,
	GETFILEFLAG_NewOK =				0x00000001,	//!< Let user create new file
	GETFILEFLAG_DftNew =				0x00000002,	//!< Default to new file if user presses OK
	GETFILEFLAG_SkipOK =				0x00000004,	//!< Allow user to skip this selection
	GETFILEFLAG_OnlyNew =			0x00000008,	//!< Don't let user select existing files
	GETFILEFLAG_MakeNew =			0x00000010,	//!< Warn user if select existing file for overwrite
	GETFILEFLAG_FilterByExtn =		0x00000400,	//!< Filter by extension
	GETFILEFLAG_SkipNone =			0x00002000,	//!< Change "Skip" button to "None"
	GETFILEFLAG_AllowFewer =		0x00004000,	//!< Allow fewer than requested number of files for mdlgGetFileSet...()
	GETFILEFLAG_AllowAutoName =	0x00008000,	//!< Allow "auto-name" for dlgGetFileSet...()
	GETFILEFLAG_SelectDir =			0x00020000,	//!< Select directory instead of file
	GETFILEFLAG_AllowDefault =		0x00080000,	//!< Use provided filename for default new file
	GETFILEFLAG_AllowAllFiles =	0x00100000,	//!< Allow all file types option
	GETFILEFLAG_SelectFileOrDir = 0x00200000, //!< Allow selection of files OR directories based on filter 
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GETFILEFLAGS)
#endif


//-------------------------------------------------------------------------------------------------------------------

//!\addtogroup mdlg Mdlg functions
//!@{

//! Dialog to select single file with char* extension(s).
DEPRECATED int MdlgGetFile (
	MDLGPARENT dlgparent,
	FILEPATH& filename,								//!< File path/name passed/returned
	const char *extns,								//!< Extensions to show, if any
	const MISTRING& prompt,							//!< Prompt
	MdlgGetFileFilterProto filterfunc = 0,		//!< Filter function
	void *funcdata = 0,								//!< Data to pass to filter function
	GETFILEFLAGS flags = GETFILEFLAG_None
	);

//! Dialog to select single file with MISTRING extension(s).
DEPRECATED int MdlgGetFile (
	MDLGPARENT dlgparent,
	FILEPATH& filename,								//!< File path/name passed/returned
	const MISTRING& extns,							//!< Extensions to show, if any
	const MISTRING& prompt,							//!< Prompt
	MdlgGetFileFilterProto filterfunc = 0,		//!< Filter function
	void *funcdata = 0,								//!< Data to pass to filter function
	GETFILEFLAGS flags = GETFILEFLAG_None
	);

//! Dialog to select multiple files using char* extension list.
DEPRECATED int MdlgGetFiles (
	MDLGPARENT dlgparent,
	FILEPATH& filename,								//!< Default filename
	const char *extns,								//!< Extensions to show, if any
	const MISTRING& prompt,
	int minfiles,										//!< Minimum number of files required
	int maxfiles,										//!< Maximum allowed to be selected, 0 if no limit
	FILEPATHLIST& FileList,
	MdlgGetFileFilterProto filterfunc = 0,		//!< Filter function
	void *funcdata = 0,								//!< Data to pass to filter function
	GETFILEFLAGS flags = GETFILEFLAG_None,
	MdlgGetFilesValidMultiProto validfunc = 0
	);

#ifndef NODLLLINKAGE		// Can't default mistring to textid with nodlllinkage
//! Prompt for a filepath component name for creation or renaming of a file
ERRVALUE MdlgGetFileComponentName (
	MDLGPARENT parent,								//!< Parent for dialog
	const MISTRING& title,							//!< Dialog title
	const FILEPATH& ParentDirectory,				//!< Parent directory that 'ComponentName' will be created in
	MISTRING& ComponentName,						//!< File component name RETURNED
	const MISTRING& Extension = TEXTID__None,	//!< Extension to add on if user did not add one
	const MISTRING& label = TEXTID__None		//!< Label to place in front of the name, TEXTID_Generic_Name_C is the default
	);
#endif

//!@}

//----------------------------------------------------------------------------
//		Definitions for MdlgGetFolder...
//----------------------------------------------------------------------------

class GETFOLDERFILTER : public GETFILEFILTER {
	public:
		GETFOLDERFILTER () {}
	private:
		GETFOLDERFILTER (const GETFOLDERFILTER&);
		GETFOLDERFILTER& operator= (const GETFOLDERFILTER&);
	};

//! Flags for MdlgGetFolder.
enum GETFOLDERFLAGS {
	GETFOLDERFLAG_None =				0,
	GETFOLDERFLAG_AllowNew =		0x0001,		//!< Allow new folder to be created
	GETFOLDERFLAG_CheckWritable =	0x0002,		//!< Check if selected folder is writable
	GETFOLDERFLAG_CheckEmpty =		0x0004,		//!< Selected folder must be empty
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GETFOLDERFLAGS)
#endif


//!\addtogroup mdlg Mdlg functions
//!@{

//! Dialog to allow user to select folder.
ERRVALUE MdlgGetFolder (
	MDLGPARENT dlgparent,								//!< Parent for dialog
	FILEPATH& filepath,									//!< Folder path passed/returned
	const MISTRING& promptstr,							//!< Prompt string (also allows TEXTID)
	GETFOLDERFLAGS flags = GETFOLDERFLAG_None,	//!< Flags
	GETFOLDERFILTER *filter = 0						//!< Filter
	);

//!@}

//----------------------------------------------------------------------------
//		Definitions for MdlgShowMessage...													
//----------------------------------------------------------------------------

// Do not change the values of existing flags as they may be passed to DLLs.
enum MDLG_MESSAGEFLAGS {
	MDLG_MESSAGEFLAG_None =					0,
	MDLG_MESSAGEFLAG_AllowNoShowAgain =	0x00000001,	//!< Include "Don't show this message again" toggle
	MDLG_MESSAGEFLAG_AllowSaveText =		0x00000002,	//!< Allow message text to be saved to file
	MDLG_MESSAGEFLAG_CenterDialog =		0x00000004,	//!< Center dialog on screen
	MDLG_MESSAGEFLAG_NoCancel =			0x00000100,	//!< Don't include "cancel" button if default is to show
	MDLG_MESSAGEFLAG_AllowCancel =		0x00000200,	//!< Include cancel button if default is not to
	MDLG_MESSAGEFLAG_NoOK =					0x00000400,	//!< Don't include "ok" button
	MDLG_MESSAGEFLAG_ContinueButton =	0x00800000,	//!< Use "Continue" instead of "OK"
	MDLG_MESSAGEFLAG_RetryButton =		0x01000000,	//!< Use "Retry" instead of "OK"
	MDLG_MESSAGEFLAG_TraceButton =		0x02000000,	//!< Use "Trace" instead of "Details"
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MDLG_MESSAGEFLAGS)
#endif

// Do not change the values of existing types as they may be passed to DLLs.
//! Message type, used to select icon and window title.
enum MDLG_MESSAGETYPE {
	MDLG_MESSAGETYPE_Default =			0,			//!< No icon
	MDLG_MESSAGETYPE_Information =	1,			//!< 'i' (information) icon
	MDLG_MESSAGETYPE_Warning =			2,			//!< Exclamation point icon
	MDLG_MESSAGETYPE_Error =			3,			//!< circle/slash icon
	MDLG_MESSAGETYPE_Message =			4,			//!< No icon
	MDLG_MESSAGETYPE_Working =			5,			//!< Hourglass icon
	MDLG_MESSAGETYPE_Question =		6,			//!< Question mark icon
	};


//----------------------------------------------------------------------------

typedef int (*MESSAGEHOOKFUNC)(void* cbdata, MDLG_MESSAGETYPE msgtype, const MISTRING& msg, const MIUNICODE* details);

//----------------------------------------------------------------------------

//! Styles for MdlgAskQuestion().
enum DLGQ_STYLE {
	//! Possible buttons, will appear in order listed at bottom of dialog.
	DLGQ_BUTTON_Ok =				0x00000001,		//!< Cannot use with BUTTON_Yes
	DLGQ_BUTTON_Yes =				0x00000002,		//!< Cannot use with BUTTON_Ok
	DLGQ_BUTTON_No =				0x00000004,
	DLGQ_BUTTON_Retry =			0x00000008,
	DLGQ_BUTTON_Save =			0x00000010,
	DLGQ_BUTTON_SaveAs =			0x00000020,
	DLGQ_BUTTON_SaveAll =		0x00000040,
	DLGQ_BUTTON_Discard =		0x00000080,
	DLGQ_BUTTON_DiscardAll =	0x00000100,
	DLGQ_BUTTON_Skip =			0x00000200,
	DLGQ_BUTTON_Continue =		0x00000400,
	DLGQ_BUTTON_YesToAll =		0x00000800,
	DLGQ_BUTTON_NoToAll =		0x00001000,

	DLGQ_BUTTON_Cancel =			0x00010000,

	//! Common button combinations
	DLGQ_BUTTONS_OkCancel =				DLGQ_BUTTON_Ok | DLGQ_BUTTON_Cancel,
	DLGQ_BUTTONS_YesNo =					DLGQ_BUTTON_Yes | DLGQ_BUTTON_No,
	DLGQ_BUTTONS_YesNoCancel =			DLGQ_BUTTON_Yes | DLGQ_BUTTON_No | DLGQ_BUTTON_Cancel,
	DLGQ_BUTTONS_RetryCancel =			DLGQ_BUTTON_Retry | DLGQ_BUTTON_Cancel,
	DLGQ_BUTTONS_SaveDiscardCancel =	DLGQ_BUTTON_Save | DLGQ_BUTTON_Discard | DLGQ_BUTTON_Cancel,
	DLGQ_BUTTONS_ContinueCancel =		DLGQ_BUTTON_Continue | DLGQ_BUTTON_Cancel,

	DLGQ_STYLE_CenterText =	0x10000000	//!< Center text in the dialog
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLGQ_STYLE)
#endif

//! Responses for MdlgAskQuestion().
enum DLGRESPONSE {
	DLGRESPONSE_Cancel = -1000,				//!< Same as EUserCancel
	DLGRESPONSE_No = 0,
	DLGRESPONSE_Ok = 1,
	DLGRESPONSE_Yes = DLGRESPONSE_Ok,
	DLGRESPONSE_Continue = DLGRESPONSE_Ok,
	DLGRESPONSE_Save = 2,
	DLGRESPONSE_SaveAs = 3,
	DLGRESPONSE_SaveAll = 4,
	DLGRESPONSE_Discard = 5,
	DLGRESPONSE_Skip = DLGRESPONSE_Discard,
	DLGRESPONSE_DiscardAll = 6,
	DLGRESPONSE_Retry = 7,
	DLGRESPONSE_NoToAll = 8,
	DLGRESPONSE_YesToAll = 9
	};

//!\addtogroup mdlg Mdlg functions
//!@{

//! Dialog to ask user for numeric value (default window title).
ERRVALUE MdlgAskNumber (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& prompt,						//!< Prompt string
	double& value,									//!< Value passed/returned
	double minvalue,								//!< Minimum allowed value
	double maxvalue,								//!< Maximum allowed value
	int decplaces,									//!< Number of decimal places to show
	HELPID helpid = HELPID__None				//!< Help ID if help provided
	);

//! Dialog to ask user for numeric value (window title specified).
ERRVALUE MdlgAskNumber (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& dlgtitle,					//!< Dialog title
	const MISTRING& prompt,						//!< Prompt string
	double& value,									//!< Value passed/returned
	double minvalue,								//!< Minimum allowed value
	double maxvalue,								//!< Maximum allowed value
	int decplaces,									//!< Number of decimal places to show
	HELPID helpid = HELPID__None				//!< Help ID if help provided
	);

//! Dialog to ask user question with buttons for responses (default window title).
//! @return DLGRESPONSE value
DLGRESPONSE MdlgAskQuestion (
	MDLGPARENT parent, 							//!< Parent for dialog, NULL for default
	const MISTRING& text,						//!< Text to display
	DLGQ_STYLE style,								//!< Dialog style and buttons
	DLGRESPONSE dftresponse,					//!< Default response (DLGRESPONSE_Yes or DLGRESPONSE_No)
	HELPID helpid = HELPID__None				//!< Help ID if help provided
	);

//! Dialog to ask user question with buttons for responses (window title specified).
//! @return DLGRESPONSE value
DLGRESPONSE MdlgAskQuestion (
	MDLGPARENT parent, 							//!< Parent for dialog, NULL for default
	const MISTRING& dlgtitle,					//!< Title for dialog window
	const MISTRING& text,						//!< Text to display
	DLGQ_STYLE style,								//!< Dialog style and buttons
	DLGRESPONSE dftresponse,					//!< Default response (DLGRESPONSE_Yes or DLGRESPONSE_No)
	HELPID helpid = HELPID__None				//!< Help ID if help provided
	);

//! Dialog to ask user for string value (default window title).
ERRVALUE MdlgAskString (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& prompt,						//!< Prompt string
	MISTRING& string,								//!< String passed/returned
	int maxchars = 0,								//!< Maximum number of characters in string, 0 for no maximum
	HELPID helpid = HELPID__None,				//!< Help ID if help provided
	GETSTRINGFILTER *filter = 0				//!< Optional filter for string validation
	);

//! Dialog to ask user for string value (window title specified).
ERRVALUE MdlgAskString (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& dlgtitle,					//!< Dialog title
	const MISTRING& prompt,						//!< Prompt string
	MISTRING& string,								//!< String passed/returned
	int maxchars = 0,								//!< Maximum number of characters in string, 0 for no maximum
	HELPID helpid = HELPID__None,				//!< Help ID if help provided
	GETSTRINGFILTER *filter = 0				//!< Optional filter for string validation
	);

//! Modal dialog to allow user to examine raw memory buffer.
void MdlgExamineRawData (
	MDLGPARENT dlgparent,
	const void *buffer,
	int buffersize
	);

//! Retrieve default parent for creating dialog.
//! @return Default parent for creating dialog.
//! This function should be used by the individual Mdlg... function itself, not by the caller.
//!	If the default parent is desired, pass NULL as the parent for any Mdlg function.
MDLGPARENT MdlgGetDefaultParent (void);

//! Dialog for file and save string in user-specified encoding.
ERRVALUE MdlgSaveAsTextFile (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const MISTRING& string,					//!< String to save
	FILEPATH *pFilePath = 0					//!< Optional filepath passed/returned
	);

//! Arrange to intercept all messages.
//!
//! This is to be used by non-interactive applications such as the automated
//! test suite which don't work very well if they sit all night waiting for
//! somebody to click OK on an error dialog.  It allows you to register a 
//! function that will get called instead of popping up the error dialog.
void MdlgSetMessageHookFunc (
	MESSAGEHOOKFUNC HookFunc,
	void* cbdata
	);

//! Display message dialog with text for specified error state.
//!
//! @return If AllowCancel flag specified will return 0 or EUserCancel, otherwise the error code is returned.
int MdlgShowErrorState (
	MDLGPARENT parent, 						//!< Parent for dialog, NULL for default
	const ERRORSTATE& ErrorState, 								//!< Error code
	MDLG_MESSAGEFLAGS flags = MDLG_MESSAGEFLAG_AllowSaveText	//!< Flags
	);

//! Display message window with specified text.
//! @return EUserCancel or 0.
int MdlgShowMessage (
	MDLGPARENT parent, 											//!< Parent for dialog, NULL for default
	MDLG_MESSAGETYPE msgtype,									//!< Message type
	const MISTRING& msgtext,									//!< Message string, multiple lines allowed, TEXTID implied.
	MDLG_MESSAGEFLAGS flags = MDLG_MESSAGEFLAG_None,	//!< Flags
	HELPID helpid = HELPID__None,								//!< Help ID (HELPID__None for no 'help' button)
	const char *inibasetag = 0,								//!< Ini base tag for savable settings (eg. MDLG_MESSAGEFLAG_AllowNoShowAgain)
	const MIUNICODE *details = 0									//!< Optional message "details"
	);

//! Display message dialog with text for specified error code.
//!
//! @return If AllowCancel flag specified will return 0 or EUserCancel, otherwise the error code is returned.
int MdlgShowMessageCode (
	MDLGPARENT parent, 						//!< Parent for dialog, NULL for default
	int errcode, 								//!< Error code
	MDLG_MESSAGETYPE msgtype = MDLG_MESSAGETYPE_Default,		//!< Message type
	MDLG_MESSAGEFLAGS flags = MDLG_MESSAGEFLAG_AllowSaveText	//!< Flags
	);

//! Display (modeless) dialog showing standard product information.
void MdlgShowProductInfo (
	MDLGPARENT dlgparent = 0
	);

//!@}

#ifdef WIN32
#pragma warning (default:4996)	// Restore warning about deprecated declaration
#endif

#endif	// INC_MI32_MDLG_H

