/**
 * \file micallbacklist.h <mi32/micallbacklist.h>
 * \brief Include file for MICALLBACKLIST
 *
 * \if NODOC
 * $Id: micallbacklist.h_v 1.4 2011/10/11 17:30:13 mju Exp $
 *
 * $Log: micallbacklist.h_v $
 * Revision 1.4  2011/10/11 17:30:13  mju
 * Add log.
 *
 * \endif
 **/

#ifndef	INC_MI32_MICALLBACKLIST_H
#define	INC_MI32_MICALLBACKLIST_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


//! Callback list.
class CLASSLIBEXPORT MICALLBACKLIST {
	public:

		enum PRIORITY {
			PRIORITY_High = -1,
			PRIORITY_Normal = 0,
			PRIORITY_Low = 1
			};

		struct ANYCALLBACKSTRUCT {
			UINT32 reason;

			//! Default constructor, performs no initialization
			ANYCALLBACKSTRUCT () {}				

			//! Construct with specified reason value
			ANYCALLBACKSTRUCT (					
				UINT32 ireason
				):
				reason(ireason)
				{ }
			};

		typedef void (*CALLBACKFUNC)(ANYCALLBACKSTRUCT*,void*);

		// CONSTRUCTION / DESTRUCTION

		MICALLBACKLIST (
			);

		~MICALLBACKLIST (
			);
			
		//! Add callback to callback list with position determined by priority.
		//!
		//! If reasonmask is 0 all reasons will be selected.
		ERRVALUE Add (
			CALLBACKFUNC cbfunc, 				//!< Callback function
			void *cbdata, 								//!< "Hook" data to pass to callback function
			UINT32 reasonmask, 						//!< Programmer-defined reasons for which callback will be called
			PRIORITY priority = PRIORITY_Normal							//!< (PRIORITY_PriorityHigh, PRIORITY_PriorityNormal, PRIORITY_PriorityLow)
			);

		//! Call callbacks in list selected by reason.
		//!
		//! The specified reason will be bitwise-anded with the reasonmask which was specified in
		//!	Add() to determine if the callback function will actually be called.  You can
		//!	use McbAnyCallbackStruct for the structure type if you are only interested in specifying
		//!	a callback reason and no additional information.
		void CallCallbacks (
			ANYCALLBACKSTRUCT *cbstruct, 		//!< Pointer to callback structure, first entry must be ULONG reason
			UINT32 reason								//!< Reason, will be ANDed with callback reasonmask
			);

		//!	Check to see if callback list has a callback for a given reason.
		//!
		//! @return True if there is a callback in the callback list which matches the given reason.
		bool HasCallbacks (
			UINT32 reason								//!< Reason, will be ANDed with callback reasonmask
			) const;

		//! Remove all callbacks from list.
		void RemoveAll (
			);

		//! Remove callback from list.
		//!
		//! If "cbfunc" or "cbdata" are NULL and/or the "reasonmask" is 0 they will not be used to "match"
		//!	the callbacks.  For example, Remove(NULL,data,0) will remove all callbacks
		//!	that reference the specified "data".  This can be useful if that data is about to be freed.
		//!	Selected callback "reasons" can be removed without removing the entire callback as well.
		//! Callbacks can be removed from the list safely while being called.
		void Remove (
			CALLBACKFUNC cbfunc, 				//!< Callback function
			void *cbdata, 								//!< "Hook" data specified in Add()
			UINT32 reasonmask							//!< Programmer-defined reasons specified in Add()
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV* m_priv;
		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

#undef LIBEXPORT
#undef CLASSLIBEXPORT

#endif //!<  INC_MI32_MICALLBACKLIST_H 
