/**
 * \file trans2dc.h <mi32/trans2dc.h>
 * \brief Classes for 2D coordinate transformation
 *
 * \if NODOC
 * $Id: trans2dc.h_v 1.75 2011/10/07 13:24:53 mju Exp $
 *
 * $Log: trans2dc.h_v $
 * Revision 1.75  2011/10/07 13:24:53  mju
 * Remove rect hdr.
 *
 * Revision 1.74  2011/09/22 21:40:05  mju
 * Now just includes trans2daffine/mapgen hdrs.
 *
 * Revision 1.73  2011/09/21 18:09:10  mju
 * Move trans2d_affine class defns to own hdr.
 *
 * Revision 1.72  2010/11/19 16:18:46  mju
 * Add getInput/OutputFormulasDisplay.
 *
 * Revision 1.70  2010/08/24 16:40:01  mju
 * Add setExcludeFlippedPieces.
 *
 * Revision 1.69  2010/08/24 16:16:36  mju
 * Fix hasReverseOrientation.
 *
 * Revision 1.68  2010/06/08 21:19:56  mju
 * Add isGeneral, hasForward/Inverse.
 *
 * Revision 1.66  2008/11/20 15:46:13  mju
 * Add trans2d_mapgen replacements for all in-use transGen fns.
 *
 * Revision 1.65  2008/01/04 21:41:34  mju
 * Add hasReverseOrientation.
 *
 * Revision 1.64  2007/10/05 16:53:16  mju
 * Add convertInverse returning separate x/y.
 *
 * Revision 1.63  2007/08/13 21:48:23  mju
 * Add equality methods for trans2d_mapgen.
 *
 * Revision 1.62  2007/07/13 21:04:54  mju
 * Add convertInputToMap.
 *
 * Revision 1.61  2006/10/13 17:50:00  scowan
 * Made arc world function take a trans2d affine.
 *
 * Revision 1.58  2005/12/15 22:49:41  mju
 * Add getInput/OutputCtrlPoints.
 *
 * Revision 1.57  2005/12/15 22:13:18  mju
 * Make all trans2d_mapgen "get" methods const.
 *
 * Revision 1.55  2005/09/08 16:38:20  scowan
 * Better deprecate the readGeoreference method since it calls a deprecated method.
 *
 * Revision 1.54  2005/06/03 14:36:08  mju
 * Add isEqual method and comparison operators.
 *
 * Revision 1.53  2005/06/01 20:01:18  mju
 * Change densification function to use MIDOUBLEARRAY and only define if in GEOM dll.
 *
 * Revision 1.52  2005/04/07 19:51:07  mju
 * Fix getValues to have yscale/rot in proper order.
 *
 * Revision 1.51  2005/04/07 17:31:44  mju
 * Add affine.getValuesForward.
 *
 * Revision 1.44  2004/11/15 23:26:48  dwilliss
 * TRANS2D_AFFINE now exported from geom.dll.  4 methods no longer inlineable.
 *
 * Revision 1.42  2004/07/09 20:30:06  mju
 * Add mapgen IsInput/OutputTransIdentity.
 *
 * Revision 1.41  2004/07/07 22:02:07  mju
 * Return errvalue from getaffine.
 *
 * Revision 1.40  2004/06/24 20:39:55  mju
 * Add Apply...Reverse operations for affine.
 *
 * Revision 1.39  2004/06/22 15:30:55  mju
 * remove trans2d_carto as no longer used, replaced by spatref system.
 *
 * Revision 1.36  2004/01/20 14:30:54  scowan
 * Added is equivalent to method for affine.
 *
 * Revision 1.32  2003/08/13 20:18:34  scowan
 * Added bool value to disable cancel class.
 *
 * Revision 1.27  2002/04/29 17:39:03  dwilliss
 * Added an ApplyPrescale method to TRANS2D_AFFINE
 *
 * Revision 1.26  2002/04/29 17:00:33  mju
 * Add GetInputTrans methods.
 *
 * Revision 1.25  2002/03/07 15:52:12  mju
 * Add AFFINE::setctrlpoints using CTRLPOINT3.
 *
 * Revision 1.24  2001/08/17 20:36:01  scowan
 * Added method to set affine with control points.
 *
 * Revision 1.19  2000/05/12 16:15:53  scowan
 * Added disect methods
 *
 * Revision 1.18  2000/03/13 16:03:26  mju
 * Add MAPGEN::ConvertForward/Inverse for generic point array.
 *
 * Revision 1.17  2000/03/07 16:52:51  scowan
 * Added method to cast to const trans parm automatically.
 *
 * Revision 1.15  2000/01/26 17:13:44  scowan
 * Added update and clear methods.
 *
 * Revision 1.9  1999/05/26 14:09:02  mju
 * Add AFFINE.ReverseDirection() method.
 *
 * Revision 1.1  1999/04/19  14:15:57  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_TRANS2DC_H
#define	INC_MI32_TRANS2DC_H

#ifndef	INC_MI32_TRANS2DAFFINE_H
#include <mi32/trans2daffine.h>
#endif

#ifndef	INC_MI32_TRANS2DMAPGEN_H
#include <mi32/trans2dmapgen.h>
#endif

#endif	// INC_MI32_TRANS2DC_H
