/**
 * \file mi32/xdefns.h
 * \brief Definitions for MicroImages library functions which use X toolkit
 *
 * \if NODOC
 * $Log: xdefns.h_v $
 * Revision 3.839  2012/01/13 16:12:32  mju
 * Remove unused fn.
 *
 * Revision 3.838  2012/01/13 16:05:47  mju
 * Remove unused fns.
 *
 * Revision 3.837  2012/01/12 22:31:04  mju
 * Use const for tag in get/setNamedUserData.
 *
 * Revision 3.836  2012/01/11 22:37:33  mju
 * Remove unused prototypes.
 *
 * Revision 3.835  2012/01/09 16:30:25  mju
 * Don't need mstatus hdr.
 *
 * Revision 3.834  2011/12/15 20:58:34  mju
 * Remove unused mxCreatePixmap.
 *
 * Revision 3.833  2011/11/03 14:39:51  mju
 * Use strtodeg hdr and remove other unneeded hdrs.
 *
 * Revision 3.832  2011/11/03 13:11:31  mju
 * Move menu/buttonitemarray defns to own hdrs.
 *
 * Revision 3.830  2011/09/30 16:45:28  mju
 * Remove unused mxToolTip fns.
 *
 * Revision 3.827  2011/09/01 19:20:12  bcolombe
 * Add MxStringIsUsingAntiAlias
 *
 * Revision 3.825  2011/06/24 21:52:54  bcolombe
 * Add MxStringDraw that takes color and cliprect for xft
 *
 * Revision 3.820  2011/04/20 18:48:14  mju
 * Add mxGetDefaultSystemXLFD.
 *
 * Revision 3.819  2011/04/20 15:25:21  mju
 * Add mxGetDefaultUIFont.
 *
 * Revision 3.817  2010/04/07 18:47:58  mju
 * Add function to set exitmainloop back to 0
 *
 * Revision 3.814  2008/09/17 16:37:28  mju
 * Add mxTitleSetString variant with parm for whether to show process ID.
 *
 * Revision 3.812  2008/04/11 15:06:04  mju
 * Add mxSetUserData.
 *
 * Revision 3.810  2008/01/10 15:54:14  mju
 * Use const for mxIconGetPixmap...
 *
 * Revision 3.809  2008/01/10 14:45:59  mju
 * Add mxRedrawWidget.
 *
 * Revision 3.807  2007/11/09 18:32:49  mju
 * Use mistatus hdr.
 *
 * Revision 3.804  2007/10/10 21:53:30  dwilliss
 * function to convert bitmap pattern to pixmap now uses sytlepattern_bitmap
 * instead of just an array of bytes
 *
 * Revision 3.802  2007/09/11 20:27:16  dwilliss
 * Added MxMainLoopUntilConditionMet
 *
 * Revision 3.801  2007/05/04 14:15:08  dwilliss
 * got rid of prototype of retired function
 *
 * Revision 3.792  2007/04/02 15:01:03  mju
 * Add mxWaitForUnmapWithIdleDelegate.
 *
 * Revision 3.789  2007/03/20 17:11:43  scowan
 * Made text get font name take a mistirng.
 *
 * Revision 3.782  2007/02/26 14:26:11  mju
 * Add mxTextClearXe, rmv mxTextSetFilenameXe.
 *
 * Revision 3.779  2007/02/12 15:26:28  dwilliss
 * Added an MxCreateIconPushButton that takes an RVC::STYLE instead of a
 * numeric style object id.
 * Documented the fact that all other functions that took numeric style
 * handles for icons ignore that parameter now (everything always passes 0
 * anyway).
 *
 * Revision 3.778  2007/02/02 14:56:56  mju
 * Change deprecations for mxMakeObjectList/UC.
 *
 * Revision 3.776  2006/12/14 20:42:18  mju
 * Move mxMakeObjectList here.
 *
 * Revision 3.770  2006/11/14 14:55:34  mju
 * Add mxDestroyTopLevel.
 *
 * Revision 3.761  2006/09/28 17:22:38  mju
 * Use objitem for raster expression region setup.
 *
 * Revision 3.760  2006/09/28 16:50:17  mju
 * Change floodzone to use objitem.
 *
 * Revision 3.759  2006/09/21 17:19:41  scowan
 * Converted hatch pattern editor to take a objitem.
 *
 * Revision 3.758  2006/09/21 15:16:46  mju
 * Add buttonitemarray.
 *
 * Revision 3.754  2006/09/12 16:01:46  scowan
 * Basin function now takes objitem.
 *
 * Revision 3.753  2006/09/11 21:57:42  scowan
 * Another region function now takes a objitem.
 *
 * Revision 3.751  2006/09/07 21:15:36  scowan
 * Changed raster trace function to take a objitem.
 *
 * Revision 3.742  2006/05/04 15:53:59  dwilliss
 * added a button row flag
 *
 * Revision 3.741  2006/03/07 17:07:51  mju
 * Add mxPopupAndWait.
 *
 * Revision 3.736  2005/11/18 14:43:41  mju
 * Add delegate to mxCreateMenuFile for special items.
 *
 * Revision 3.731  2005/09/12 19:10:16  scowan
 * Changed a prototype to take a objitem.
 *
 * Revision 3.730  2005/08/18 16:18:42  dwilliss
 * Overloaded a func and deprecated the old version
 *
 * Revision 3.726  2005/03/31 18:23:23  mju
 * Add MxSetFocusToWidget.
 *
 * Revision 3.721  2005/02/17 15:17:22  mju
 * Add defaults for unused parms in saveDrawable/SnapshotToRaster.
 *
 * Revision 3.720  2005/01/20 18:34:30  scowan
 * Changed mx select element function parameters.
 *
 * Revision 3.719  2005/01/18 15:57:50  mju
 * Add iconid variants for mxCreateIcon...
 *
 * Revision 3.717  2005/01/14 23:57:57  dwilliss
 * RASTYPE constants switched around to be the same as RVC::IMAGE::CELLTYPE
 * Eliminated 2-bit raster type
 *
 * Revision 3.643  2003/09/15 13:49:56  fileserver!dwilliss
 *
 * \endif
**/

#if defined (MAC_OSX) && defined(__x86_64__)
#define USE_XFT 1
#endif

#if !defined(INC_MI32_XDEFNS_H) && (!defined(WIN32_NATIVE) || defined(GENERATING_DOXYGEN_OUTPUT))

#define	INC_MI32_XDEFNS_H

#define	NoXmMessageDialogTypes	//  So don't allow XmDIALOG_ERROR, WARNING, etc 

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#ifndef	_XLIB_H_
	#include <X11/Xlib.h>
#endif

#ifndef	_XUTIL_H
	#include <X11/Xutil.h>
#endif

#ifdef EDITRES
	#include <X11/Xmu/Editres.h>
#endif

#ifndef	_Xm_h
	#include <Xm/Xm.h>
#endif

#ifndef	_XmToggle_h
	#include <Xm/ToggleB.h>
#endif

#ifndef	INC_MI32_DISPINFO_H
	#include <mi32/dispinfo.h>
#endif

#ifndef INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef	INC_MI32_TEXTBASE_H
	#include <mi32/textbase.h>		//! For TEXTBASEORIGIN
#endif

#ifndef	INC_MI32_STRTODEG_H
	#include <mi32/strtodeg.h>		// For DEGTOSTRFLAGS
#endif

#ifndef	INC_MI32_MSYS_H
	#include <mi32/msys.h>
#endif

#ifndef	INC_RVC_OBJTYPE_H
	#include <rvc/objtype.h>
#endif

#ifndef	INC_MI32_MG2ENUMS_H
	#include <mi32/mg2enums.h>	
#endif

#ifndef	INC_MI32_ICONID_H
	#include <mi32/iconid.h>
#endif

#ifndef	INC_MI32_XTABPAGE_H
	#include <mi32/xtabpage.h>
#endif

#ifndef	INC_MI32_XBASE_H
	#include <mi32/xbase.h>
#endif

#ifndef	INC_MI32_XTARGS_H
	#include <mi32/xtargs.h>
#endif

#ifndef	INC_MI32_DELEGATE_H
	#include <mi32/delegate.h>
#endif


#ifdef index
#undef index
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class REGION2D;
struct MICONSTRUCT;
struct RGNELEMLIST;
struct TEXTSTYLE;
struct MTIMER;
class TRANS2D_MAPGEN;
struct UNITCONV;
struct TEXTMETRICS;
class FILEPATH;
namespace MGD {
	class DEVICE;
	class DEVICE_BINARY;
	}
namespace RVC {
	class OBJITEM;
	class OBJITEMLIST;
	class RASTER;
	class STYLE;
	class STYLEPATTERN_BITMAP;
	}
namespace SPATREF {
	class COORDREFSYS;
	}
#endif


/*----------------------------------------------------------------------------*/
/*		Helper macros																				*/
/*----------------------------------------------------------------------------*/

#define	XtSetArgI(arg,c,n,v)	((void)(arg[c].name=(char*)(n),arg[c].value=(XtArgVal)(v),c++))

	// The sizeof operators are used to ensure that the size of the variable where
	// userData will be stored is at least as large as a pointer.  Normally the
	// expression inside the [] evaluates to 1.  mju 1994-05-10
#define	MxGetUserData(w,p)	{ Arg _mxgetuserdataarg[sizeof(*p)-sizeof(void*)+1]; XtSetArg(_mxgetuserdataarg[0],XmNuserData,p); XtGetValues(w,_mxgetuserdataarg,1); }

// Macros for defining class methods to add Xt callbacks / event handlers
#define	MXXTCB_ADD(_clsname)			inline void MxXtCallbackAdd (Widget w, const char *name, void (*f)(Widget,_clsname*,void*)) { XtAddCallback(w,(name),reinterpret_cast<XtCallbackProc>(f),this); }
#define	MXXTCB_REMOVE(_clsname)		inline void MxXtCallbackRemove (Widget w, const char *name, void (*f)(Widget,_clsname*,void*)) { XtRemoveCallback(w,(name),reinterpret_cast<XtCallbackProc>(f),this); }
#define	MXXTEH_ADD(_clsname)			inline void MxXtEventHandlerAdd (Widget w, EventMask emask, void (*f)(Widget,_clsname*,XEvent*,Boolean*), bool nmi=false) { XtAddEventHandler(w,(emask),(Boolean)(nmi),reinterpret_cast<XtEventHandler>(f),this);	}
#define	MXXTEH_REMOVE(_clsname)		inline void MxXtEventHandlerRemove (Widget w, EventMask emask, void (*f)(Widget,_clsname*,XEvent*,Boolean*), bool nmi=false) { XtRemoveEventHandler(w,(emask),(Boolean)(nmi),reinterpret_cast<XtEventHandler>(f),this);	}
#define	MXXTTO_ADD(_clsname)			inline XtIntervalId MxXtTimeOutAdd (Widget, unsigned long delay, void (*f)(_clsname*,XtIntervalId*)) { return(XtAppAddTimeOut(MsysAppGetXContext(),(delay),reinterpret_cast<XtTimerCallbackProc>(f),this));	}


/*----------------------------------------------------------------------------*/

extern "C" {

extern int _MxShadowThickness;	//!<  Shadow thickness for widgets that have shadows 
extern int _MxSpaceNarrow;			//!<  Narrow spacing (eg, inside Frame) between widgets 
extern int _MxSpaceNormal;			//!<  Normal spacing between widgets 
extern int _MxSpaceWide;			//!<  Wide spacing between widgets 

}	//! extern "C"

/*----------------------------------------------------------------------------*/
/*		Definitions for EditColor																*/
/*----------------------------------------------------------------------------*/

#define	EDITCOLOR_AllowTransp		0x0002 //!< Allows transparency toggle
#define	EDITCOLOR_AllowTransPct		0x0004 //!< Allows setting of transparency 0-100%

struct EDITCOLORINFO {
	COLOR color;
	Pixel pixel;					//!< Pixel value to use on PseudoColor displays
	UINT8 CanChangeColor;		//!< Set if colormap entry for pixel value is writable
	UINT8 AllowTransparent;		//!< Allow this color to be set to "transparent"
	UINT8 DefaultSelected;		//!< Default to having this item 'selected'
	UINT8 spare;
	char *name;						//!< Name for color, will be translated through resource file
	};


/*----------------------------------------------------------------------------*/
/*		Menu Item definitions																	*/
/*----------------------------------------------------------------------------*/

struct MENUITEM {
	mutable char *label;				//!< Label for the item, if "-" a separator will be generated, NULL to indicate end of array.
	unsigned long flags;				//!< Must be same size as pointer for proper alignment on the DEC Alpha
	XtCallbackProc callback;		//!< Callback function; NULL if none
	void *callback_data;				//!< client_data for callback()
	const char *help;					//!< Help id
	const MENUITEM *subitems;		//!< Submenu items; NULL if none
	void *userdata;					//!< Used to set userData resource if NULL passed to MxCreateMenuBar
	UINT32 icid;						//!< Interface Component ID for customization
	mutable Widget widget;			//!< Widget created for this menu item
	Widget *pwidget;					//!< If non-NULL, pointed-to value will be set to widget when created
	};

struct MENUBARITEM {
	mutable char *label;				//!< Label for the item, NULL to indicate end of array.
	const MENUITEM *items;			//!< First pulldown menu items, NULL to indicate end.
	Widget *widget_list;				//!< Array of widgets to save into
	mutable Widget widget;			//!< Widget created for this menu bar item
	const char *help;					//!< Help id
	UINT32 icid;						//!< Interface Component ID for customization
	Widget *pwidget;					//!< If non-NULL, pointed-to value will be set to widget when created
	};

#define MxMENU_SELECTED				0x0001
#define MxMENU_ONE_OF_MANY			0x0002	//!< radio button gadget
#define MxMENU_TOGGLE				0x0004	//!< single ToggleButtonGadget
#define MxMENU_DISABLED				0x0008	//!< button is initially not "Sensitive"
#define MxMENU_LABEL					0x0010	//!< Just a label
#define MxMENU_FORCENAME			0x0020	//!< Set labelString resource to label value so doesn't read from resource file
#define MxMENU_LABELMIUNICODE		0x0040	//!< Label is a pointer to a MIUNICODE string
#define MxMENU_IGNORE				0x0080	//!< Ignore this item, don't put on menu
#define MxMENU_NOGADGET				0x0100	//!< Don't use "gadget" for this item
#define MxMENU_NOCENTERLABEL		0x0200	//!< Don't center if label
#define MxMENU_PROGRAMMER			0x0400	//!< Item only appears on "Programmer" systems
#define MxMENU_NOCHGCBDATA			0x0800	//!< Don't change callback_data for this item
#define MxMENU_FREELABEL			0x1000	//!< Free label field after item created
#define MxMENU_NOCUSTOMSUBMENU	0x2000	//!< Don't show submenu when customizing


enum MxEDITMENUFLAGS {	//! Flags for MxEditMenuCreate
	MxEDITMENUFLAG_None =			0x0000,
	MxEDITMENUFLAG_NoSearch =		0x0001,
	MxEDITMENUFLAG_NoReplace =		0x0002,
	MxEDITMENUFLAG_NoInsertFile = 0x0004,
	MxEDITMENUFLAG_AllowFormat =  0x0008	//!< For if widgit is WYSIWYG
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MxEDITMENUFLAGS)
#endif

/*----------------------------------------------------------------------------*/
/*		Definitions used by MxCreateButtonRow												*/
/*----------------------------------------------------------------------------*/

struct BUTTONITEM {
	char *label;						//!< the label for the item
	UINT32 flags;						//!< see below
	XtCallbackProc callback;		//!< Callback function; NULL if none
	void *callback_data;				//!< client_data for callback() if NULL passed to MxCreateButtonRow
	const char *help;					//!< Help id
	void *userdata;					//!< Used to set userData resource if NULL passed to MxCreateButtonRow
	mutable Widget pbwidget;		//!< Widget created
	const char *IconName;			//!< name of the bitmap fill pattern to use for the icon (16 chars max)
	const char *ClassName;			//!< if not NULL, will override "class" passed to MxCreateIconButtonXXXX
	UINT32 icid;						//!< Interface Component ID for customization
	const MIUNICODE *ToolTip;			//!< ToolTip string for override
	Pixel	BackgroundPixel;			//!< Background color of icon if MxBUTTON_BACKGROUND_PIXEL is set
	Widget *pwidget;					//!< If non-NULL, pointed-to value will be set to widget when created
	};

#define	MxBUTTON_DISABLED				0x00000001	//!< Button is initially not "Sensitive"
#define	MxBUTTON_TOGGLE				0x00000002	//!< Create toggle (radio or checkbox) button
#define	MxBUTTON_NOINDICATOR			0x00000004	//!< Don't show indicator
#define	MxBUTTON_SELECTED				0x00000008	//!< Toggle button initially selected
#define	MxBUTTON_ONE_OF_MANY			0x00000010	//!< Use one-of-many indicator, enforce radio behavior
#define	MxBUTTON_DEFAULT				0x00000020	//!< Button gets default border (Not implemented)
#define	MxBUTTON_MULTI_CLICK			0x00000040	//!< Button allows multiple clicks in rapid succession
#define	MxBUTTON_IGNORE				0x00000080	//!< Ignore this entry in the BUTTONITEM array
#define	MxBUTTON_PROGRAMMER			0x00000100	//!< Don't display button unless in Programmer mode
#define	MxBUTTON_NOMANAGE				0x00000200	//!< Don't manage button when created (RowColumn button-rows only)
#define	MxBUTTON_ICON_MENU			0x00000400	//!< Create icon popup menu for button, callback_data points to MENUITEM*
#define	MxBUTTON_ICON_MENU_FILE		0x00000800	//!< Create icon popup menu for button, callback_data points to menu file name
#define	MxBUTTON_BACKGROUND_PIXEL	0x00001000	//!< Use BackgroundPixel from ButtonItem
#define	MxBUTTON_NOCUSTOMSUBMENU	0x00002000	//!< Don't show submenu when customizing
#define	MxBUTTON_FORCENAME			0x00004000	//!< Set labelString resource to label value so doesn't read from resource file
#define	MxBUTTON_LABELMIUNICODE		0x00008000	//!< Label is a pointer to a MIUNICODE string (implies FORCENAME)
#define	MxBUTTON_ALLOWRIGHTCLICK	0x00010000	//!< Allow right click, will call same callback as left, use event pointer in cb struct to test if needed

#define  MxBUTTONROW_NoIcons		0x0001	//!< Don't create and use icons for this button row (passed in flags for MxCreateIconButtonRow()
#define	MxBUTTONROW_RowColumn	0x0002	//!< Use RowColumn for button-row instead of form
#define	MxBUTTONROW_XmiPane	0x0004	//!< Use XmiPane for button-row instead of form

/*----------------------------------------------------------------------------*/
/*		Flags for MxCreateIcon...																*/
/*----------------------------------------------------------------------------*/

#define MXICON_Stippled 				MICONFLAG_Stippled
#define MXICON_DefaultBackground		MICONFLAG_DefaultBackground
#define MXICON_StippledSelected		MICONFLAG_StippledSelected
#define MXICON_Size16					MICONFLAG_Size16
#define MXICON_Size32					MICONFLAG_Size32
#define MXICON_NoDarkenSelected		MICONFLAG_NoDarkenSelected

/* Flags for MxSelectIcon() */

#define	SELICON_Resize				0x01			//!<  Resize icons smaller than specified minimum size 


/*----------------------------------------------------------------------------*/
/*		Flags used by MxPopupGet... and MxCreateDouble() etc...						*/
/*----------------------------------------------------------------------------*/

#define GET_NoMin					(0x00000011)	//! Don't use or show
#define GET_NoMax					(0x00000022)
#define GET_NoRange				(0x00000033)
#define GET_NoDft					(0x00000004)
#define GET_InvValid				(0x00000008)	//! For GetString
#define GET_RastValRGB16UINT2	(0x00000008)	//! For MxPromptRastValCreate
#define GET_RangeMask			(0xFFFFFFFC)

#define GET_NoShowMin			(0x00000010)	//! Use, but don't show
#define GET_NoShowMax			(0x00000020)
#define GET_NoShowRange			(0x00000030)	//! Obsolete
#define GET_ShowUnitLabel		(0x00000040)	//! Show "unit" label
#define GET_ShowRange			(0x00000080)	//! Set if want range displayed

#define GET_NoCancel				(0x00000100)
#define GET_LabelTop				(0x00000200)	//! label on top of text widget 
#define GET_DftRet				(0x00000400)	//! retval contains default
#define GET_AddOne				(0x00000800)	//! Add 1 before showing dft/range

#define GET_DegsNS				(0x00000000)	//! For Lat/Lon prompt (yes, this is zero)
#define GET_DegsEW				(0x00001000)	//! For Lat/Lon prompt
#define GET_ToUpper				(0x00002000)	//! Convert characters to upper case as entered
#define GET_DetectNULL			(0x00004000)	//! Treat 7FFF as empty string (for databases)
#define GET_BlankZero			(0x00008000)	//! Show 0 values as blank
#define GET_RightJustify		(0x00010000)	//! Right justify within field
#define GET_OutputOnly			(0x00020000)	//! Output-only prompt
#define GET_PromptGiven			(0x00040000)	//! For DBQueryEdit Only
#define GET_DegsNoDir			(0x00080000)	//! For Lat/Lon "range" prompt, don't show direction
#define GET_NoManage				(0x00100000)
#define GET_NoForm				(0x00200000)	//! For MxCreatePrompt...
#define GET_PopupForceStr		(0x00400000)	//! In MxPopup... use specified string instead of doing lookup
#define GET_HasScale				(0x00800000)	//! Put "Scale" widget in prompt
#define GET_Degs360EW			(0x01000000)	//! For Longitude allow entry/display of W360 to E360
#define GET_Unicode				(0x02000000)	//! Make string widget Unicode
#define GET_PromptUnicode		(0x04000000)	//! Prompt string is really (MIUNICODE*)
#define GET_LeftJustify			(0x08000000)	//! Left justify numeric value within field
#define GET_UseLogScale			(0x20000000)	//! Set if want logarithmic scale, must set GET_HasScale also
#define GET_Password				(0x40000000)	//! Show "*"'s in text widget

#define	MxPromptRastValCR_ValueChanged	0x0001

#define	MxPromptCR_ValueChanged			0x00000001	//!<  User has changed prompt value 
#define	MxPromptCR_ValueChangeDrag		0x00000002	//!<  User dragging prompt scale slider 
#define	MxPromptCR_Activate				0x00000004	//!<  User pressed \<Enter\> while in prompt 

struct MxPromptCallbackStruct {
	UINT32 reason;							//!<  Callback reason 
	Widget prompt;							//!<  Prompt widget 
	void *value;							//!<  Pointer to prompt value 
	};

typedef void (*MxPromptCallbackFunc)(MxPromptCallbackStruct*, void*);

#define	MXPRMTCB_ADD(_clsname)		inline void MxPromptCallbackAdd (Widget w, void (*f)(MxPromptCallbackStruct*,_clsname*), UINT32 flags = MxPromptCR_ValueChanged) const { MxPromptAddCallback(w,reinterpret_cast<MxPromptCallbackFunc>(f),(void*)this,(flags)); }
#define	MXPRMTCB_REMOVE(_clsname)	inline void MxPromptCallbackRemove (Widget w, void (*f)(MxPromptCallbackStruct*,_clsname*), UINT32 flags) const { MxPromptRemoveCallback(w,reinterpret_cast<MxPromptCallbackFunc>(f),(void*)this,(flags)); }

	/* Prompt field types */
#define MXPROMPT_ASTRING	0		//!<  ASCII string 
#define MXPROMPT_USTRING	1		//!<  Unicode string 
#define MXPROMPT_ACHAR		2		//!<  Single ASCII character 
#define MXPROMPT_UCHAR		3		//!<  Single Unicode character 
#define MXPROMPT_DOUBLE		10		//!<  64-bit floating-point 
#define MXPROMPT_FLOAT		11		//!<  32-bit floating-point 
#define MXPROMPT_UINT32		12		//!<  Unsigned 32-bit number 
#define MXPROMPT_INT32		13		//!<  Signed 32-bit number 
#define MXPROMPT_UINT16		14		//!<  Unsigned 16-bit number 
#define MXPROMPT_INT16		15		//!<  Signed 16-bit number 
#define MXPROMPT_UINT8		16		//!<  Unsigned 8-bit number 
#define MXPROMPT_INT8		17		//!<  Signed 8-bit number 
#define MXPROMPT_LATLON		18		//!<  Latitude or Longitude stored as double 

/*----------------------------------------------------------------------------*/
/*		Values by MxPositionWindowRel															*/
/*----------------------------------------------------------------------------*/

enum MxRELPOSN {
	MxRELPOSN_Default =			0,
	MxRELPOSN_AutoLeftRight =	1,
	MxRELPOSN_RightTop =			2,
	MxRELPOSN_LeftTop =			3,
	MxRELPOSN_RightCenter =		4,
	MxRELPOSN_LeftCenter =		5,
	MxRELPOSN_RightBottom =		6,
	MxRELPOSN_LeftBottom =		7,
	MxRELPOSN_TopLeft =			8,
	MxRELPOSN_BottomLeft =		9,
	MxRELPOSN_TopCenter =		10,
	MxRELPOSN_BottomCenter =	11,
	MxRELPOSN_TopRight =			12,
	MxRELPOSN_BottomRight =		13,
	MxRELPOSN_MatchBottom =		14,
	MxRELPOSN_MatchTop =			15,
	MxRELPOSN_MatchLeft =		16,
	MxRELPOSN_MatchRight =		17
	};

/*----------------------------------------------------------------------------*/
/*		Values used by MxLockWindowSize()													*/
/*----------------------------------------------------------------------------*/

#define	MxLOCKSIZE_MinHeight		0x1
#define	MxLOCKSIZE_MinWidth		0x2
#define	MxLOCKSIZE_MaxHeight		0x4
#define	MxLOCKSIZE_MaxWidth		0x8
#define	MxLOCKSIZE_Min				(MxLOCKSIZE_MinHeight | MxLOCKSIZE_MinWidth)
#define	MxLOCKSIZE_Max				(MxLOCKSIZE_MaxHeight | MxLOCKSIZE_MaxWidth)
#define	MxLOCKSIZE_MinMax			(MxLOCKSIZE_Min | MxLOCKSIZE_Max)

/*----------------------------------------------------------------------------*/
/*		Misc constants																				*/
/*----------------------------------------------------------------------------*/

#define	MxBCOL_ALIGN_BEGINNING	1

extern "C" {

/*----------------------------------------------------------------------------*/
/*		Function prototypes (keep in alphabetical order!)								*/
/*----------------------------------------------------------------------------*/

//!\addtogroup MsysApp MsysApp Functions
//!@{

//! Get X Application Context.
XtAppContext MsysAppGetXContext ();

//! Get X Display pointer.
Display* MsysAppGetXDisplay ();

//! Set X Display pointer and application context.
void MsysAppSetXDisplayContext (
	Display *xdisplay,							//!< X Display pointer
	XtAppContext xappcontext					//!< Xt application context
	);

//!@}
//!\addtogroup X11 X11 Functions
//!@{

			/* Generic callbacks */

//! The XmNhelpCallback to any widget that you want to give help to.
//! The callback data should be a string containing the refcode of the
//! help entry.  If the name of the widget is the same as it's refcode,
//! you can just pass null for the callback data.
void MxcbDisplayHelp (
	Widget,										//!< The widget to display help for
	void*,
	void*
	);

//! OptionMenu callback for setting UINT8 value.
//! The userdata parameter specified in MxCreateOptionMenuExt() must be a pointer to the UINT8 variable or member to be set.
//! The MENUITEM.callback_data will contain the values to be set in the specified variable.
void MxcbOptionUBYTE (
	Widget w,
	void *voption,
	void*
	);

//! Set the exit process
void MxcbSetExit (
	Widget,
	void*,
	void*
	);

//! Callback for toggling bool variable/member.
//! The callback data must be a pointer to the 'bool' value to toggle. 
void MxcbToggleBool (
	Widget,
	void *vvalue,
	XmToggleButtonCallbackStruct *cbs
	);

//! Callback for toggling UINT8 variable/member.
//! The callback data must be a pointer to the UINT8 value to toggle. 
void MxcbToggleUBYTE (
	Widget,
	void *vvalue,
	XmToggleButtonCallbackStruct *cbs
	);

#define	MxcbOptionUINT8		MxcbOptionUBYTE
#define	MxcbToggleUINT8		MxcbToggleUBYTE

//! Main entry point for Edit / Attribute Database.
int Main_DatabaseEditor (
	Widget parent								//!< Parent widget
	);

//! Handle f.delete from TWM or f.kill from MWM.
//! This function should be called after any dialog is created.  It will
//! add support for the "Close" item on the MWM menu (f.kill) and the
//! f.delete (Be Polite) in TWM.  It uses the standard WM_DELETE_WINDOW
//! protocol so anything that sends WM_DELETE_WINDOW will be trapped by
//! this callback.  It is also assumed that the resource deleteResponse
//! has been set to XmDO_NOTHING.  If you do not pass a shell widget, the
//! widget hierarchy of the widget you do pass will be searched for the
//! nearest shell widget.
//! If you pass NULL for the callback function, a static local callback
//! which calls XBeep() will be added for you.
//! See mxadwcb.c for more information about what is going on.
void MxAddDeleteWindowCallback (
	Widget widget,
	XtCallbackProc cbproc,
	void* cbdata
	);

//! Make labels a uniform width and right justify them.
//! This routine takes an array of widgets and makes them all as wide as the widest one in the list.
//! This forces XmALIGNMENT_CENTER for buttons and XmALIGNMENT_END for everything else.
void MxAlignLabels (
	Widget* widgets,								//!< Array of widgets to align
	int num											//!< Number of widgets to align
	);

//! Make specified widgets the same width, adjust alignment if requested.
//! This routine takes an array of widgets and makes them all as wide as the widest one in the list.
//! Flags:
//!		\li 1 = Force XmALIGNMENT_CENTER for buttons and XmALIGNMENT_END for everything else
//!		\li 2 = Force XmALIGNMENT_BEGINNING for everything
void MxAlignWidgets (
	Widget* widgets,								//!< Array of widgets to align
	int num,											//!< Number of widgets to align
	UINT32 flags									//!< Flags
	);


#define MxANGLECONTROL_Quadrant1		0x00000001
#define MxANGLECONTROL_Quadrant2		0x00000002
#define MxANGLECONTROL_Quadrant3		0x00000004
#define MxANGLECONTROL_Quadrant4		0x00000008
#define MxANGLECONTROL_FullCircle	0x0000000F	//!< All of the above
#define MxANGLECONTROL_ZeroAtTop		0x00000010

//! Create a graphical control for entering an angle.
//!
//!	Creates a drawing area which will allow the user to
//! graphically enter an angle.  Note that no numeric prompt
//! is created.  If you want one, create it yourself, then
//! call MxAngleControlSetPrompt().
//! By default, the control allows for 360 degrees.  If you want
//! to limit it to one or more quadrants, use one or more of the
//! following flags ORd together.
//!
//!		\li MxANGLECONTROL_Quadrant1		Upper Right
//!		\li MxANGLECONTROL_Quadrant2		Upper Left
//!		\li MxANGLECONTROL_Quadrant3		Lower Left
//!		\li MxANGLECONTROL_Quadrant4		Lower Right
//!
//! There's no guarentee what you'll get if you give three quadrants
//! or two which are diagonaly opposite.
//!
//!	If you intend to add a numeric prompt, you can also pass the
//! following flag to the create function.
//!
//!		\li MxANGLECONTROL_ZeroAtTop
//!	
//!	The default is for zero to be along the right X axis and increase
//!	counter-clockwise.  If you give the ZeroAtTop flag, then zero is
//! at the top (go figure!) and increases clockwise.
//! 
Widget MxAngleControlCreate (
	Widget parent,
	double *pValue,		//!< Pointer to angle to modify (radians)
	int radius,				//!< Radius of control
	UINT32 flags,			//!< 
	Arg *callerargs,
	int numargs
	);


//!  Set the numeric prompt for an angular prompt (graphic control).
//!
//! MxAngleControlCreate only creates the graphic control for setting 
//! an angle.  If you also want a numeric prompt to reflect the angle,
//! you can create the prompt yourself, placing it wherever you want
//! then call this function.
ERRVALUE MxAngleControlSetPrompt (
	Widget control,
	Widget prompt
	);

//! Create "application shell".
//!
//! @return unrealized topLevelShell widget
Widget MxAppShellCreate (
	const char *posnkey,						//!< Name of shell (used for window positioning) NULL ok
	Arg* arg,			 						//!< Argument list
	int numargs			 						//!< Number of arguments
	);

//! Assign cursor to a window.
//!
//! @return 0
void MxAssignCursor (
	Display* display,
	Window window,
	int cursor
	);

//! Erase specified widget, optionally generating exposure notification.
void MxClearWidget (
	Widget widget,
	bool sendexposure = true
	);

//! Convert a value from one unit to another.
//!
//! @return converted value
//! This function allows you to convert distances from various units to another.
//! This differs from the normal unit conversion functions in that is can convert
//! to and from screen pixels.  It uses MxTopLevel() to get the display pixel
//! size.  Units can be any of the following:
//!		\li\b SCREENUNIT_Inches
//!		\li\b SCREENUNIT_mm
//!		\li\b SCREENUNIT_Points
//!		\li\b SCREENUNIT_Pixels
//!		\li\b SCREENUNIT_1OverInches
//!		\li\b SCREENUNIT_MetersAtCurrentScale
double MxConvertScreenUnits (
	double val,										//!< Value to convert
	int InUnits,									//!< Input units (SCREENUNIT_Xxxx values)
	int OutUnits									//!< Output units
	);

//! Convert an XmString to MIUNICODE.
//!
//! @return a pointer to the MIUNICODE string
//! Free the returned pointer using MmFree() when you are done with it.
//! The input string is not changed by this function in any way.
MIUNICODE* MxConvertXmStringToUC (
	XmString	xmstr									//!< XmString to convert
	);

//! Create column of equal-sized pushbuttons and/or togglebuttons.
//!
//! @return the widget
//! There is one flag defined for the flags parameter:
//! \li\b MxBCOL_ALIGN_BEGINNING				Align the labels to the left edge
//!			NOTE: If you pass 0, the labels will be centered.
//!		Flag field bit values:
//! \li\b MxBUTTON_DISABLED Button is initially not "Sensitive"
//!  \li\b MxBUTTON_TOGGLE		Create a toggle button
//!  \li\b MxBUTTON_ONE_OF_MANY	Use one-of-many indicator, enforce radio button behavior
//!  \li\b MxBUTTON_NOINDICATOR	Don't display indicator, button will appear pushed in when selected
//!  \li\b MxBUTTON_SELECTED	Toggle button is initially selected
//!  \li\b MxBUTTON_IGNORE		Ignore this entry, no button will be created for this item
//!  \li\b MxBUTTON_PROGRAMMER	Only display button if in "programmer" mode
//!
//! The outermost widget created will have no margins so it will exactly cover all the buttons.  You should
//! therefore set Offset resource values when making attachments.  Typically 10 is a good value for offsets
//! from the edge of the Dialog and between groups of items in the dialog (ie. between the button column and
//! a vertical separator).
Widget MxCreateButtonColumn (
	Widget,										//!< Parent widget (usually a form)
	BUTTONITEM*,								//! List of button descriptions (see below)
	void*,										//! Data to pass to all callbacks (NULL to use value in buttonitem)
	void*,										//! Value for userData resource (NULL to use value in buttonitem)
	int,											//!< Flags
	Arg*,											//! Argument list for outermost widget created (for attachments)
	int											//!< Number of entried in argument list
	);

//! Create form with button and XeText widget for selection.
//!
//! @return Managed form widget
//! If the userData resource is set in the arglist it will be used for that resource in the button, but
//! NOT the text widget.  If the XmNcolumns resource is set in the arglist it will be used for the text
//! widget.  The form will also be managed after it is created.
Widget MxCreateButtonFieldXe (
	Widget form,								//!< Parent widget
	const char* name,							//!< Label for button
	Widget* xetext,							//!< XeText widget RETURNED (NULL if not needed)
	Widget* button,							//!< Button widget RETURNED (NULL if not needed)
	XtCallbackProc cbfunc,					//!< Callback for button (NULL if none) XmNactivateCallback
	void* cbdata,								//!< Calback data
	Arg* arglist,								//!< Arguments for form
	int argcount								//!< Number of arguments in arglist
	);

//! Create row of equal-sized pushbuttons and/or togglebuttons.
//!
//! @return Managed XmForm or XmRowColumn eidget containing buttons
//! Flags:
//! \li\b MxBUTTONROW_NoIcons	Don't create icon buttons (forced set by this function)
//! \li\b MxBUTTONROW_RowColumn	Use RowColumn widget instead of Form for button-row
//!
//! 	Flag field bit values:
//!  \li\b MxBUTTON_DISABLED	Button is initially not "Sensitive"
//!  \li\b MxBUTTON_TOGGLE		Create a toggle button
//!  \li\b MxBUTTON_ONE_OF_MANY	Use one-of-many indicator, enforce radio button behavior if RowColumn
//!  \li\b MxBUTTON_NOINDICATOR	Don't display indicator, button will appear pushed in when selected
//!  \li\b MxBUTTON_SELECTED	Toggle button is initially selected
//!  \li\b MxBUTTON_IGNORE		Ignore this entry, no button will be created for this item
//!  \li\b MxBUTTON_PROGRAMMER	Only display button if in "programmer" mode
//!  \li\b MxBUTTON_NOMANAGE	Don't manage button (only allowed if MxBUTTONROW_RowColumn flag set)
//!  \li\b MxBUTTON_ICON_MENU	Create popup menu for icon button, callback_data is MENUITEM array
//!
//! The outermost widget created will have no margins so it will exactly cover all the buttons.  You should
//! therefore set Offset resource values when making attachments.  Typically 10 is a good value for offsets
//! from the edge of the Dialog and between groups of items in the dialog (ie. between the button row and a
//! horizontal separator).
Widget MxCreateButtonRow (
	Widget pwidget,							//!< Parent widget
	const BUTTONITEM *buttonitem,			//!< List of button descriptions (see below)
	void *cbdata,								//!< Data to pass to all callbacks if value in buttonitem is NULL
	void *userdata,							//!< Value for userData resource if value in buttonitem is NULL
	int flags,									//!< Flags
	const Arg*,									//! Argument list for outermost widget created (for attachments)
	int argcount								//!< Number of entries in argument list
	);

//! Create XmFormDialog and setup help and popdown callbacks.
//!
//! @return XmFormDialog widget or NULL if parameter error
Widget MxCreateFormDialog (
	Widget pwidget,							//!< Parent widget
	const char* titlestr,					//!< Title string, must not be NULL
	const char* helpstr,						//!< Help string, must not be NULL or "NoHelp"
	XtCallbackProc cbpopdn,					//!< Callback to handle when use presses [X] to close dialog
	void *cbdata,								//!< Data to pass to callback
	Arg *arglist,								//!< Arguments for dialog creation
	int argcount								//!< Number of argumentsb
	);


}	// end of Extern C

//! Drop-in replacement for XmCreateFormDialog.  
//! Copes with title i18n by looking title up in resource file and setting 
//! the window property to a UTF8 string which modern window managers can
//! cope with.
//!
//! @return XmFormDialog widget or NULL if parameter error
Widget MxCreateFormDialog (
	Widget pwidget,							//!< Parent widget
	const char* titlestr,					//!< Title string, must not be NULL
	Arg *arglist,								//!< Arguments for dialog creation
	int argcount								//!< Number of argumentsb
	);

extern "C" {

Widget MxCreateFrame (
	Widget pwidget,							//!< Parent widget
	const XTARGS& arg,						//!< Widget creation arguments
	const char *label = 0					//!< Option string for label resource lookup, NULL for no label
	);

//! Create a row of equal-sized icon push and/or toggle buttons.
//!
//! @return Managed XmForm or XmRowColumn widget containing buttons
//! Flags:
//!  \li\b MxBUTTONROW_NoIcons	Don't create icon buttons (forced set by this function)
//!  \li\b MxBUTTONROW_RowColumn	Use RowColumn widget instead of Form for button-row
//! See also: MxCreateButtonRow
Widget MxCreateIconButtonRow (
	Widget pwidget,							//!< Parent widget
	const MIUNICODE *iconfile,				//!< NULL for "ICONS.REF"
	const char *icongroup,					//!< Icon group if all from same group (NULL for "standard")
	const BUTTONITEM *buttonitem,			//!< List of button descriptions (see MxCreateButtonRow())
	void *cbdata,								//!< Data to pass to all callbacks if value in buttonitem is NULL
	void *userdata,							//!< Value for userData resource if value in buttonitem is NULL
	int flags,									//!< Flags
	const Arg *arglist,						//!< Argurment list for outermost widget containing buttons
	int argcount								//!< Number of entries in argument list
	);

//! Create Label widget with icon instead of text.
//! @return Managed widget
//! You may pass NULL for the LabelName and the widget will be assigned
//! a generic resource name AND no ToolTip will be generated.
Widget MxCreateIconLabel (
	Widget pwidget,							//!< Parent widget
	int iconhandle,							//!< **IGNORED**
	const char *IconClass,					//!< Icon class used if icon object is not already open
	const char *IconName,					//!< Icon name
	const char *LabelName,					//!< Widget Label name, also used for ToolTip
	const Arg *arglist,						//!< Arguments for button
	int argcount								//!< Number of arguments in arglist
	);

}	// end of extern "C"

//! Create Label widget with icon instead of text.
//! @return Managed widget
//! You may pass NULL for the LabelName and the widget will be assigned
//! a generic resource name AND no ToolTip will be generated.
Widget MxCreateIconLabel (
	Widget pwidget,							//!< Parent widget
	ICONID iconid,								//!< Icon ID
	const MISTRING& tooltip,				//!< ToolTip text
	const Arg *arglist,						//!< Arguments for button
	int argcount								//!< Number of arguments in arglist
	);

extern "C" {
//! Create Label widget with icon instead of text, extended version.
//!
//! @return Managed widget
//! Flags:
//! 	\li\b MXICON_Stippled		Create a "dimmed" version for use when widget is made insensitive
//! 				only if 'StippleName' is NULL.
//! 	\li\b MXICON_Size16		Resize to 16x16 if smaller
//! 	\li\b MXICON_Size32		Resize to 32x32 if smaller
//! The StippleName field is useful for creating an icon that changes depending on some user input.  The change
//! is implemented by setting the sensitivity of the icon.  This assumes that you will not be needing the icon
//! to be insensitive.
//! Note that you must specify the MXICON_Stippled flag if you want to make the icon label insensitive.
//! If you do not, the appearance will be the same as the "sensitive" mode.
//! You may pass NULL for the LabelName and the widget will be assigned
//! a generic resource name AND no ToolTip will be generated.
Widget MxCreateIconLabelExt (
	Widget pwidget,							//!< Parent widget
	int iconhandle,							//!< **IGNORED**
	const char *IconClass,					//!< Icon class used if icon object is not already open
	const char *IconName,					//!< Icon name
	const char *StippleName,				//!< Stipple icon name, can be NULL
	const char *LabelName,					//!< Widget label name, also used for ToolTip
	const Arg *arglist,						//!< Arguments for button
	int argcount,								//!< Number of arguments in arglist
	UINT32 flags								//!< MXICON_... flags
	);

//! Create popup menu widget with specified icon.
//!
//! @return Managed widget
//! This function will create a menubar and a cascade button that will end up looking like a simple
//! push button icon.  If the icon is pressed though, the list specified by the 'items' field will
//! be popped up and the user then may select from that list.
Widget MxCreateIconPopupMenuButton (
	Widget pwidget,							//!< Parent widget
	int iconhandle,							//!< **IGNORED**
	const char *IconClass,					//!< Icon class used if icon object is not already open
	const char *IconName,					//!< Icon name
	const char *LabelName,					//!< Widget label name used for ToolTip
	const MENUITEM *items,					//!< List of menu items
	Widget *widget_list,						//!< List of widgets created returned but the array is not allocated
	void *data,									//!< Callback data if not specified in the MENUITEM array
	void *udata,								//!< User data if not specified in the MENUITEM array
	const Arg *arglist,						//!< Arguments for button
	int argcount								//!< Number of argumentd in arglist
	);

#ifndef NO_DEPRECATED
//! Create Pushbutton widget with specified icon.
//! @return Managed widget
//! You may pass NULL for the LabelName and the widget will be assigned
//! a generic resource name AND no ToolTip will be generated.
DEPRECATED_MSG("There's another version that doesn't have a numeric style object id for the 2nd parameter") Widget MxCreateIconPushButton (
	Widget pwidget,							//!< Parent widget
	int iconhandle,							//!< **IGNORED**
	const char *IconClass,					//!< Icon class used if icon object is not already open
	const char *IconName,					//!< Icon name
	const char *LabelName,					//!< Widget label name, also used for ToolTip
	XtCallbackProc,							//!< Callback for XmNactivateCallback
	void *cbdata,								//!< Callback data
	const Arg *arglist,						//!< Arguments for button
	int argcount								//!< Number of arguments in arglist
	);
#endif

}
//! Create Pushbutton widget with specified icon.
//! @return Managed widget
//! You may pass NULL for the LabelName and the widget will be assigned
//! a generic resource name AND no ToolTip will be generated.
Widget MxCreateIconPushButton (
	Widget pwidget,							//!< Parent widget
	ICONID iconid,								//!< Icon ID
	const MISTRING& tooltip,				//!< ToolTip text
	XtCallbackProc cbfunc,					//!< Callback for XmNactivateCallback
	void *cbdata,								//!< Callback data
	const Arg *arglist,						//!< Arguments for button
	int argcount								//!< Number of arguments in arglist
	);

//! Create Pushbutton widget with specified icon.
//! @return Managed widget
//! You may pass NULL for the LabelName and the widget will be assigned
//! a generic resource name AND no ToolTip will be generated.
Widget MxCreateIconPushButton (
	Widget pwidget,							//!< Parent widget
	const char *IconGroup,					//!< Icon group used if icon object is not already open
	const char *IconName,					//!< Icon name
	const char *LabelName,					//!< Widget label name, also used for ToolTip
	XtCallbackProc,							//!< Callback for XmNactivateCallback
	void *cbdata,								//!< Callback data
	const Arg *arglist,						//!< Arguments for button
	int argcount,								//!< Number of arguments in arglist
	UINT32 flags = 0,							//!< MXICON_... flags
	RVC::STYLE* StyleObj = 0				//!< Style object to read icon from (pass NULL for standard icons.ref)
	);

extern "C" {

//! Create PushButton widget with specified icon (extended version).
//!
//! @return Managed widget
//! Flags:
//! 	\li\b MXICON_Stippled		Create a "dimmed" version for use when widget is made insensitive
//! 	\li\b MXICON_Size16		Resize to 16x16 if smaller
//! 	\li\b MXICON_Size32		Resize to 32x32 if smaller
//! Note that you must specify the MXICON_Stippled flag if you want to make the icon label insensitive.
//! If you do not, the appearance will be the same as the "sensitive" mode.
//! You may pass NULL for the LabelName and the widget will be assigned
//! a generic resource name AND no ToolTip will be generated.
Widget MxCreateIconPushButtonExt (
	Widget pwidget,							//!< Parent widget
	int iconhandle,							//!< **IGNORED**
	const char *IconClass,					//!< Icon class used if icon object is not already open
	const char *IconName,					//!< Icon name
	const char *LabelName,					//!< Widget label name, also used for ToolTip
	XtCallbackProc,							//!< Callback for XmNactivateCallback
	void *cbdata,								//!< Callback data
	const Arg *arglist,						//!< Arguments for button
	int argcount,								//!< Number of arguments in arglist
	UINT32 flags								//!< MXICON_... flags
	);

//! Create ToggleButton widget with specified icon.
//!
//! @return Managed widget
//! The ToggleButton will not have a separate "indicator".  The entire button will appear pushed-in when
//! selected.
//! You may pass NULL for the LabelName and the widget will be assigned
//! a generic resource name AND no ToolTip will be generated.
Widget MxCreateIconToggleButton (
	Widget pwidget,							//!< Parent widget
	int iconhandle,							//!< **IGNORED**
	const char *IconClass,					//!< Icon class used if icon object is not already open
	const char *IconName,					//!< Icon name
	const char *LabelName,					//!< Widget label name, also used for ToolTip
	void(*)(Widget, void*, XmToggleButtonCallbackStruct*),	//! Callback for XmNactivateCallback
	void *cbdata,								//!< Callback data
	const Arg *arglist,						//!< Arguments for button
	int argcount								//!< Number of arguments in arglist
	);

//! Create ToggleButton widget with specified icon (extended version).
//!
//! @return Managed widget
//! Flags:
//! 	\li\b MXICON_Stippled		Create "dimmed" version for use when widget is made insensitive
//! 	\li\b MXICON_StippledSelected	Create "dimmed" version for use when widget is "selected" and insensitive
//! 	\li\b MXICON_Size16		Resize to 16x16 if smaller
//! 	\li\b MXICON_Size32		Resize to 32x32 if smaller
//! 	\li\b MXICON_NoDarkenSelected	Don't darken background of button when "selected" (icons must be different)
//! Note that you must specify the MXICON_Stippled flag if you want to make the icon label insensitive.
//! If you do not, the appearance will be the same as the "sensitive" mode.  If a different appearance
//! is desired when the ToggleButton is both "selected" and "insensitive" you must specify the
//! MXICON_StippledSelected flag as well.
//! The ToggleButton will not have a separate "indicator".  The entire button will appear pushed-in when
//! selected.
Widget MxCreateIconToggleButtonExt (
	Widget pwidget,							//!< Parent widget
	int iconhandle,							//!< **IGNORED**
	const char *IconClass,					//!< Icon class used if icon object is not already open
	const char *UnselIconName,				//!< Icon name to use for "unselected" state
	const char *SelectedIconName,			//!< Icon name to use for "selected" state
	const char *LabelName,					//!< Widget label name used for ToolTip
	void(*)(Widget, void*, XmToggleButtonCallbackStruct*),	//! Callback for XmNactivateCallback
	void *cbdata,								//!< Callback data
	const Arg *arglist,						//!< Arguments for button
	int argcount,								//!< Number of arguments in arglist
	UINT32 flags								//!< MXICON_... flags
	);

//! Create XImage from specified widget.
//!
//! @return Error < 0
//! This function will create a ZPixmap image using the depth and visual information from
//! the widget provided.  If a multi-line image is created each line will be 32-bit aligned.
int MxCreateImageFromWidget (
	Widget widget,								//!< Widget to create image from (NULL for toplevel)
	XImage** ximage,							//!< Image pointer returned
	void** buffer,								//!< Buffer allocated and returned
	int width,									//!< Width of image
	int height									//!< Height of image
	);

//! Create button and XeText widget.
//!
//! @return Managed widget
Widget MxCreateLabelFieldXe (
	Widget form,
	const char* name,
	Widget* xetext,
	Widget* label,
	Arg* arglist,
	int argcount
	);

//! Create OptionMenu for selection Latitude-Longitude format.
//!
//! @return Managed OptionMenu widget
//! The "format" will be passed to the callback as the callback data.  Use MxGetUserData() to
//! retrieve the specified "userdata" value from the callback widget.
Widget MxCreateLatLonFormatOptionMenu (
	Widget pwidget,							//!< Parent widget
	char *label,								//!< Label for OptionMenu
	UINT32 dftfmt,								//!< Default format (DMSTYPE_...)
	void(*)(Widget, void *, void *),		//! OptionMenu callback
	void *userdata,							//!< Userdata attached to widget passed to callback
	Arg *arglist,								//!< Argument list
	int argcount								//!< Argument count
	);

//! Create a popup menu.
//! @return Unmanaged widget
//! This function creates a popup menu which is activated by pressing the right mouse button over the
//! parent widget.  Do not manage the widget returned yourself because that will cause the menu to
//! pop up on its own.
Widget MxCreatePopupMenu (
	Widget parent,								//!< Widget to check RMB event on
	const char *title,						//!< No idea what this is for, fake it
	const MENUITEM *items,					//!< List of menu items
	Widget *widget_list,						//!< List of widgets created returned but the array is not allocated
	void *data,									//!< Callback data if not specified in the MENUITEM array
	void *udata								//!< User data if not specified in the MENUITEM array
	);

//! Create a pulldown menu.
//! @return Unmanaged widget
Widget MxCreatePulldownMenu (
	Widget parent,								//!< Parent menu widget
	const MIUNICODE *label,					//!< Menu label
	const MENUITEM *items,					//!< List of menu items
	void *cbdata,								//!< Callback data if not specified in the MENUITEM array
	void *udata									//!< User data if not specified in the MENUITEM array
	);

//! Creates and manage XmLabel from char* string with resource lookup.
//! @return XmLabel widget.
Widget MxCreateLabel (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Label name, will be used for resource lookup.
	Arg *arglist,								//!< Arguments
	int argcount								//!< Number of arguments
	);

}	//! End of extern "C"

//! Creates and manage XmLabel from MIUNICODE* string.
//! @return XmLabel widget.
Widget MxCreateLabel (
	Widget parent,								//!< Parent widget
	const MISTRING& label,					//!< String for label.
	Arg *arglist,								//!< Arguments
	int argcount								//!< Number of arguments
	);

extern "C" {

//! Creates a popup menu without an event handler.
//! @return Menu widget
Widget MxCreateManualPopupMenu (
	Widget parent,								//!< Parent widget
	const char *title,						//!< Title for menu
	const MENUITEM *items,					//!< Items for menu
	Widget *widget_list,
	void *data,
	void *udata
	);

//! Create menubar with complete pull-down menu system.
//! @return Unmanaged menu bar (RowColumn) widget
//! The sides and top of the menubar are automatically attached to the parent
Widget MxCreateMenuBar (
	Widget pwidget,							//!< Parent widget, should be a Form or subclass thereof
	const MENUBARITEM *menus,				//!< Array defining meu bar items, last item must have label of NULL
	void *clientdata,							//!< Callback data to use if not specified in menu item
	void *userdata								//!< Value to set "userData" resource to for each gadget in menu
	);

//! Create OptionMenu with Label and CascadeButton.
//! @return Unmanaged OptionMenu widget
//! This function will create an option menu.  One (and only one) of the items in the item list should
//! have the MxMENU_SELECTED flag set.  This will be the deault value for the menu.
//! When the user selects something off of the option menu, that item's callback funciton will be called
//! and given its callback data.  The userdata parameter allows you to attach something to the XmNuserData
//! resource of all the items.
Widget MxCreateOptionMenu (
	Widget pwidget,							//!< Parent widget
	const char *title,						//!< Title for menu
	const MENUITEM *items,					//!< Items for option menu, last element must have label of NULL
	void *userdata								//!< Value to set "userData" resource to for each gadget in menu
	);

//! Create OptionMenu with specified resources and manage it.
//! @return Managed OptionMenu widget
//! This funciton will create an option menu.  One (and only one) of the items in the item list should
//! have the MxMENU_SELECTED flag set.  This will be the deault value for the menu.
//! When the user selects something off of the option menu, that item's callback funciton will be called
//! and given its callback data.  The userdata parameter allows you to attach something to the XmNuserData
//! resource of all the items.
Widget MxCreateOptionMenuExt (
	Widget pwidget,							//!< Parent widget
	const char *title,						//!< Title for menu
	const MENUITEM *items,					//!< Items for option menu, last element must have label of NULL
	void *userdata,							//!< Value to set "userData" resource to for each gadget in menu
	const Arg *arglist,						//!< Arguments for resource settings
	int argcount								//!< Number of arguments in arglist
	);

//! Create and manage PushButton widget and add callback.
//! @return Managed widget
Widget MxCreatePushButton (
	Widget pwidget,							//!< Parent widget
	const char *name,							//!< Widget name
	XtCallbackProc,							//!< Callback for XmNactivateCallback
	void *cbdata,								//!< Callback data
	Arg *arglist,								//!< Arguments for button
	int argcount								//!< Number of arguments in arglist
	);

//! Create ListButton for selecting text character encoding (CHAR_ENCODING).
//! @return Widget
Widget MxCreateTextEncodingLB (
	Widget pwidget,							//!< Parent widget
	const char *label,						//!< Label for button
	CHAR_ENCODING& encoding,				//!< Encoding reference, value will be changed when selection made
	ENCODELIST_FLAGS enclistflags,		//!< Flags specifying which encodings are allowed
	Arg *arglist,								//!< Parameters for attacment
	int argcount								//!< Number of arglist entries
	);

//! Create and manage ToggleButton widget and add callback.
//! @return Managed widget
Widget MxCreateToggleButton (
	Widget pwidget,							//!< Parent widget
	const char *name,							//!< Widget name
	void(*)(Widget, void*, XmToggleButtonCallbackStruct*),	//! Callback for XmNvalueChangedCallback
	void *cbdata,								//!< Callback data
	Arg *arglist,								//!< Arguments for button
	int argcount								//!< Number of arguments in arglist
	);

//! Create an XmString from Unicode.
//! @return Converted string
//! Creates an XmString for use in labels, buttons, XmLists, etc...
//! Use this function instead of XmStringCreateXxx().
//! Call XmStringFree() to free the string.
//! Automatically handles selecting fonts for I18N.
XmString MxCreateXmStringUC (
	const MIUNICODE* const string,			//!< String to convert
	MGD::INTERFACETEXTSTYLE style = MGD::INTERFACETEXTSTYLE_Normal
	);

//! Destroy a popup menu.
void MxDestroyPopupMenu (
	Widget parent,								//!< Parent widget
	Widget popup								//!< Popup menu to destroy
	);

//! Destroy TopLevel widget.
void MxDestroyTopLevel (
	);

//! Display help for given ID.
void MxDisplayHelp (
	Widget parent,								//!< Parent widget
	const char* helpid								//!< Help ID
	);

//! Display help for given HELPID.
void MxDisplayHelpByID (
	Widget parent,								//!< Parent widget
	HELPID helpid
	);

//! Allow user to edit X bitmaps.
//!
//! Error < 0
int MxEditBitmapCursor (
	Widget pwidget,
	FILEPATH *filename,
	int flags
	);

//! Dialog allowing user to edit bitmap patterns.
//!
//! Error < 0
int MxEditBitmapPattern (
	Widget pwidget,							//!< Parent widget for dialog
	const RVC::OBJITEM& StyleObjItem,
	INT32 pattnum,								//!< Pattern number
	const COLOR& varcolor,					//!< Variable color value passed/returned
	UINT32 flags								//!< None currently defined
	);

//! Dialog allowing user to edit color levels for one or more colors.
//!
//! @return Error code if < 0 (EUserCancel if user cancels) or 0 if no error
//! Flags:
//! 	\li\b EDITCOLOR_AllowTransp	Allows transparency toggle.
//!		\li\b EDITCOLOR_AllowTransPct		Allows setting of transparency 0-100%
int MxEditColor (
	Widget parent,								//!< Parent widget
	EDITCOLORINFO *EditColorInfo,			//!< List of color specifiecations (POINTER IS RETAINED FOR INTERNAL USE)
	int  numcolors,							//!< Number of colors in list
	UINT32 flags,								//!< Flag bits
	Widget *notused							//!< Obsolete
	);

//! Dialog allowing user to edit hatch patterns.
//!
//! Error < 0
int MxEditHatchPattern (
	Widget pwidget,							//!< Parent widget for dialog
	const RVC::OBJITEM& StyleObjItem,	//!< Location of style object to edit hatch patterns on
	INT32 pattnum,								//!< Pattern number
	COLOR *varcolor,							//!< Variable color value passed/returned
	UINT32 flags								//!< None currently defined
	);

//! Create a BUTTONITEM array for a Cut/Copy/Paste/Undo/Find button row
//!
//! This functions the same as MxEditMenuCreate() but it creates an array
//! of BUTTONITEMS for an icon button row instead.
//!
//! @param handle A pointer to the edit menu's private data <i>(returned)</i>
//!   It should be passed to MxEditMenuAddCallbacks() once the text widget 
//!   has been created.
//! @param flags Flags, Possible values are:
//!   \li <b>MxEDITMENUFLAG_NoInsertFile Don't provide an "Insert File" menu item</b>
//!   \li <b>MxEDITMENUFLAG_NoSearch Don't provide a "Find" menu item</b>
//!   \li <b>MxEDITMENUFLAG_NoReplace Don't provide a "Replace" menu item</b>
//!   \li <b>MxEDITMENUFLAG_None None of the above</b>
//! @param undofunc Function to call when "undo" is selected.  If 0, then
//!  the menu won't have an "undo" option.
//! @param undodata Passed to the undofunc as its second parameter
//!
//! @returns array of MENUITEMS.  Do not free this pointer.
//!
//! @see MxEditMenuCreate()
const BUTTONITEM *MxEditButtonsCreate (
	void **handle, 
	MxEDITMENUFLAGS flags = MxEDITMENUFLAG_None, 
	void(*undofunc)(Widget, void*) = 0, 
	void *undodata = 0
	);


//! Create a MENUITEM array for an edit menu.
//!
//! Creating an edit menu is a two step process.  First you call MxEditMenuCreate()
//! to create the MENUITEM array which can be used to create the menu bar for
//! a window.  Then you call MxEditMenuAddCallbacks() to once the text widget 
//! has been created.
//! 	
//! @param handle A pointer to the edit menu's private data <i>(returned)</i>
//!   It should be passed to MxEditMenuAddCallbacks() once the text widget 
//!   has been created.
//! @param flags Flags, Possible values are:
//!   \li <b>MxEDITMENUFLAG_NoInsertFile Don't provide an "Insert File" menu item</b>
//!   \li <b>MxEDITMENUFLAG_NoSearch Don't provide a "Find" menu item</b>
//!   \li <b>MxEDITMENUFLAG_NoReplace Don't provide a "Replace" menu item</b>
//!   \li <b>MxEDITMENUFLAG_None None of the above</b>
//! @param undofunc Function to call when "undo" is selected.  If 0, then
//!  the menu won't have an "undo" option.
//! @param undodata Passed to the undofunc as its second parameter
//!
//! @returns array of MENUITEMS.  Do not free this pointer.
//!
//! @see MxEditButtonsCreate()
//! \deprecated Use the version which takes a shell as the first parameter
DEPRECATED const MENUITEM *MxEditMenuCreate (
	void **handle, 
	MxEDITMENUFLAGS flags = MxEDITMENUFLAG_None, 
	void(*undofunc)(Widget, void*) = 0, 
	void *undodata = 0
	);

}

//! Create a MENUITEM array for an edit menu.
//!
//! Creating an edit menu is a two step process.  First you call MxEditMenuCreate()
//! to create the MENUITEM array which can be used to create the menu bar for
//! a window.  Then you call MxEditMenuAddCallbacks() to once the text widget 
//! has been created.
//! 	
//! @param shell The shell which will contain the menubar.  Actually, you can
//!	pass any widget within the shell.  It will find the shell for you.
//! @param handle A pointer to the edit menu's private data <i>(returned)</i>
//!   It should be passed to MxEditMenuAddCallbacks() once the text widget 
//!   has been created.
//! @param flags Flags, Possible values are:
//!   \li <b>MxEDITMENUFLAG_NoInsertFile Don't provide an "Insert File" menu item</b>
//!   \li <b>MxEDITMENUFLAG_NoSearch Don't provide a "Find" menu item</b>
//!   \li <b>MxEDITMENUFLAG_NoReplace Don't provide a "Replace" menu item</b>
//!   \li <b>MxEDITMENUFLAG_None None of the above</b>
//! @param undofunc Function to call when "undo" is selected.  If 0, then
//!  the menu won't have an "undo" option.
//! @param undodata Passed to the undofunc as its second parameter
//!
//! @returns array of MENUITEMS.  Do not free this pointer.
//!
//! @see MxEditButtonsCreate()
const MENUITEM *MxEditMenuCreate (
	Widget shell,
	void **handle, 
	MxEDITMENUFLAGS flags = MxEDITMENUFLAG_None, 
	void(*undofunc)(Widget, void*) = 0, 
	void *undodata = 0
	);

extern "C" {
//! Add callbacks for Cut/Copy/Paste/Find button row.
//!
//! See MxEditButtonsCreate() for more information.
//!
//! @param handle Handle returned from MxEditButtonsCreate()
//! @param textwidget The XeText widget to monitor.
void MxEditButtonsAddCallbacks (
	void* handle, 
	Widget textwidget
	);

//! Add callbacks to an Edit Menu.
//!
//! See MxEditMenuCreate() for more information.
//! Note, you can add EditMenu callbacks to multiple text widgets as long as
//! they're in the same dialog.  The menu's callbacks will operate on which
//! ever text widget had focus last.
//!
//! @param handle Handle returned from MxEditMenuCreate()
//! @param textwidget The XeText widget to monitor.
void MxEditMenuAddCallbacks (
	void* handle, 
	Widget textwidget
	);

//! Dialog allowing user to edit symbols.
//!
//! @return Error < 0
int MxEditSymbol (
	Widget pwidget,							//!< Parent widget for dialog
	const RVC::OBJITEM& StyleObjItem,
	INT32 pattnum,								//!< Pattern number
	COLOR *varcolor,							//!< Variable color value passed/returned
	UINT32 flags								//!< None currently defined
	);

//! Add mouse-wheel scroll support to an X widget.
//! The "owner" widget is the widget which will watch
//! for mouse wheel events, and will scroll the given
//! scroll bar.  The increment parameter is the number of
//! "units" to scroll by.  If 0, it will scroll by about 90% of 
//! the scroll bar's page increment value.
void MxEnableMouseWheel (
	Widget owner,
	Widget scrollbar,
	int increment = 3
	);

//! Add mouse-wheel scroll support to an XmList widget.
//! This function is the same as MxEnableMouseWheel, except that it
//! retrieves the XmNverticalScrollBar resource from the widget
//! for you.
void MxListEnableMouseWheel (
	Widget owner,
	int increment = 3
	);

//! Find child widget with specified name.
//! @return Child widget or NULL if none found
Widget MxFindNamedChildWidget (
	Widget parent,								//!< Parent widget
	char *name									//!< Widget name to look for
	);

//! Ensures the contents of a dialog window will become visible before returning.
void MxForceDialog (
	Widget w										//!< Any widget in the dialog (BulletinBoard child or a DialogShell)
	);

//! Force window to be entirely on screen.
//! This function in automatically invoked by MxWaitForUnmap() and MxMainLoopUntilExitSet().
//! If the widget is an ApplicationShell it should be called after the shell is realized.
void MxForceWindowOnScreen (
	Widget,
	int
	);

//! Fix the size of a window to compensate for broken Apple X11 server.
//! The Apple X11 server incorrectly sizes some dialogs to be 22 pixels
//! higher than they should be.  This happens to be the height of the title
//! bar.  To compensate,  call MxFixWindowSizeForAppleX11 just before
//! managing the main form
void MxFixWindowSizeForAppleX11 (
	Widget mainform
	);

//! Get pixel value corresponding to specified color.
Pixel MxGetColorPixel (
	COLOR const& color						//!< Color
	);

//! Get pixel value corresponding to specified color.
//! This version ignores the display device's ICM profile, if any.
Pixel MxGetColorPixelNoICM (
	COLOR const& color						//!< Color
	);

//! Get fontList resource as read from [xfontlist] in tntproc.ini.
//!
//! Notes:  Do not free the XmFontList that is returned.  It is kept
//! 			 internally for global use.  Requires a TopLevel to be present
//! 			 and MxInitProc() must have been called already.  This
//!			 fontList allows proper display of XmStr's for use in i18n.
XmFontList MxGetDefaultFontList (
	);

//! Get XLFD for default system font to use if no other available.
const char* MxGetDefaultSystemXLFD ();

//! Get XLFD for default user interface font.
const char* MxGetDefaultUIFont ();

//! Fill in DISPLAYINFO structure from a widget.
void MxGetDisplayInfo (
	Widget widget,								//!< Widget to use as reference (NULL to use toplevel)
	DISPLAYINFO *displayinfo				//!< Information about display/scree/visual returned
	);

//! Get dimensions of DrawingArea widget.
void MxGetDrawingAreaSize (
	Widget dawidget,							//!< DrawingArea widget, if 0 will return sizes of 0
	INT16& width,								//!< Width in pixels returned
	INT16& height								//!< Height in pixels returned
	);

//! Fill in XFontStruct structure from a widget.
//!
//! @return Error code < 0 or 0 if no error
int MxGetFontStructFromWidget (
	Widget widget,								//!< Widget to use
	XFontStruct *fontstruct					//!< Font information structure to fill in
	);

//! Return the maximum ascent of the fonts in the default font list.
//! Ascent is the part of the font above the baseline
//!
//! @param bForClipping If true, the ascent may include bits of glyphs which
//! normally extend past the design ascent.  If false, it uses the ascent
//! as specified by the font designer.
int MxGetMaxFontAscent (
	bool bForClipping = false
	);

//! Return the maximum descent of the fonts in the default font list.
//! Descent is the part of the font below the baseline
//!
//! @param bForClipping If true, the descent may include bits of glyphs which
//! normally extend below the design descent.  If false, it uses the descent
//! as specified by the font designer.
int MxGetMaxFontDescent (
	bool bForClipping = false
	);

//! Return the maximum height (ascent+descent) of the fonts in the default font list.
//!
//! @param bForClipping If true, the height may include bits of glyphs which
//! normally extend outside the design descent.  If false, it uses the height
//! as specified by the font designer.
int MxGetMaxFontHeight (
	bool bForClipping = false
	);

//! Return the maximum character width in the fonts in the default font list.
//! @see MxGetTypicalFontWidth()
int MxGetMaxFontWidth();

//! Return the typical character width in the fonts in the default font list.
//! This will be the mean width of all glyphs font.  In the case where the
//! default font list contains multiple fonts, this will return the maximum
//! typical width of all fonts. I.e., each font has a typical width of its
//! own, and this will return the mean width of the widest font.
//! @see MxGetMaxFontWidth()
int MxGetTypicalFontWidth ();

//! Same as MxGetTypicalFontWidth2, but only check fonts of a given style.
//! If no fonts of the requested style exist, returns typical "normal" width.
int MxGetTypicalFontWidth2 (
	MGD::INTERFACETEXTSTYLE style = MGD::INTERFACETEXTSTYLE_Normal
	);

//! Get a tagged user data resource for a widget.
//!
//! @return Pointer to attached data or NULL if no match
void *MxGetNamedUserData (
	Widget widget,								//!< Widget data is attached to
	const char *name							//!< String used as a tag
	);

//! Get actual size of scrollable area in ScrolledWindow.
void MxGetScrollAreaSize (
	Widget,										//!< ScrolledWindow widget
	int *width,									//!< Width returned (NULL if don't need)
	int *height									//!< Height returned (NULL if don't need)
	);

//! Traverses up widget tree to closest parent of class "Shell".
//! @return Shell widget
//! Used to get a shell widget when a parent is needed for creating pop-ups.
Widget MxGetShell (
	Widget widget								//!< Widget to start traversal at
	);

//! Return pixel matching specified "standard color" index.
//! @return Pixel value
Pixel MxGetStdColorPixel (
	int index									//!< Standard color index (0 - NUMSTDCOLORS-1)
	);

//! Fill in array of "standard" color pixels.
void MxGetStdColorPixels (
	Pixel *stdpixel							//!< Array to contain result, must have at least NUMSTDCOLORS entries
	);

//! Fill in COLOR array for "standard colors".
void MxGetStdColors (
	COLOR stdcolor[NUMSTDCOLORS]			//!< Color array filled in
	);

//! Get main TNT system preferences.
int MxGetSystemPreferences (
	Widget pwidget								//!< Parent widget
	);

//! Traverses up widget tree to TopLevelShell.
//!
//! @return Shell widget
//! Used to get a shell widget when a parent is needed for creating pop-ups.
Widget MxGetTopLevel (
	Widget widget								//!< Widget to start traversal at
	);

//! Traverses up widget tree to WMShell.
//!
//! @return Shell widget
//! Used to get a shell widget when a parent is needed for creating pop-ups.
Widget MxGetTopShell (
	Widget widget								//!< Widget to start traversal at
	);

}	// Extern "C"

//! Obtain value of userData resource attached to widget.
inline void *MxGetUserDataPtr (
	Widget widget								//!< Widget to obtain userData for
	) {
	void *ptr;
	Arg arg;
	XtSetArg(arg,XmNuserData,&ptr);
	XtGetValues(widget,&arg,1);
	return (ptr);
	}


extern "C" {

//! Return string for widget label.
//!
//! @return string
//! This function will access the XmNlabekString resource for the specified widget and convert it to
//! a text string.
MIUNICODE *MxGetWidgetLabelString (
	Widget widget								//!< Widget to return string for
	);

//! Get widget dimensions.
void MxGetWidgetSize (
	Widget widget,
	Dimension& width,
	Dimension& height
	);

//! Returns borders around window up to but not including root window.
//!
//! This function computes the borders taking into account the size of all gadgets/decorations added by
//! the window manager.  It is useful for determing the size of a window needed to fill the entire screen.
int MxGetWindowBorders (
	Widget widget,								//!< Widget to use for borders
	int *left,									//!< Left border returned
	int *right,									//!< Right border returned
	int *top,									//!< Top border returned
	int *bottom									//!< Bottom border returned
	);

//! Returns the window position
void MxGetWindowPosition (
	Widget w,									//!< Widget to find position for
	int *px,										//!< X coordinate returned
	int *py										//!< Y coordinate returned
	);

//! Add callback to free icon when widget destroyed.
//! This function should be used for every icon attached to a widget.  It will be automatically
//! called for the various MxCreateIcon... functions (eg. MxCreateIconPushButton).
void MxIconAddDestroyCallback (
	Widget widget,								//!< Widget to add callback to
	MICONSTRUCT *								//! Icon to add
	);

//! Draw icon on X drawable in "dimmed" colors.
//! Note that if a "dimmed" pixmap is not available fr the icon the normal pixmap will be used.
void MxIconDrawDimmed (
	Display *display,							//!< X display
	Drawable drawable,						//!< X drawable (Window or Pixmap)
	GC gc,										//!< X graphic context
	int x,										//!< X coordinate
	int y,										//!< Y coordinate
	MICONSTRUCT *icon							//!< Icon to draw
	);

//! Draw icon on X drawable in "normal" colors.
void MxIconDrawNormal (
	Display *display,							//!< X display
	Drawable drawable,						//!< X drawable (Window or Pixmap)
	GC gc,										//!< X graphic context
	int x,										//!< X coordinate
	int y,										//!< Y coordinate
	MICONSTRUCT *icon							//!< Icon to draw
	);

//! Get "dimmed" X pixmap for icon.
//!
//! @return X pixmap
//! Note that if a dimmed pixmap is not available the normal pixmap will be returned.
Pixmap MxIconGetPixmapDimmed (
	const MICONSTRUCT *icon							//!< Icon
	);

//! Get "normal" X pixmap for icon.
//!
//! @return X pixmap
Pixmap MxIconGetPixmapNormal (
	const MICONSTRUCT *icon							//!< Icon
	);

//! Determine if process exiting.
int MxIsExitingProcess (
	);

//! Determine if TimeOut callback invoked from main event loop.
//!
//! @return TRUE if in main event loop, FALSE if in modal dialog event loop
int MxIsTimeOutInMainLoop (
	);

//! Create composite widget containing button/text for selection from list.
//!
//! @return ButtonField (form) widget or NULL if out of memory
//! MIUNICODE *GetItemFunc (itemnum, funcdata)
//! 	int itemnum		Item number (0 - NumItems-1)
//! 	void *funcdata		Hook data
//! 	Returns			ALLOCATED pointer to string (WILL be free()d)
//! void NotifyFunc (newitem, funcdata)
//! 	int newitem		Item number selected
//! 	void *funcdata		Hook data
//! \deprecated Use MGUI::FORM_COMBOBOX
DEPRECATED Widget MxListButtonCreate (
	Widget pw,									//!< Parent widget
	const char *label,						//!< resource name
	int NumItems,								//!< Number of items in list
	int maxlen,									//!< Maximum item length (characters)
	MIUNICODE*(*GetItemFunc)(int,void*),	//! Function to retriece items for list
	void(*NotifyFunc)(int,void*),			//! Function to notify application when item selected
	void *FuncData,							//!< Hook data for functions
	Arg *arglist,								//!< Argument list for ButtonField creation
	int argcount								//!< Number of arguments
	);

//! Set help Id for "list button" pop-in dialog.
//! \deprecated Use MGUI::FORM_COMBOBOX
DEPRECATED void MxListButtonSetHelpID (
	Widget listbutton,						//!< ListButton widget
	const char* helpid						//!< Help ID
	);

//! Set current item for "list button".
//! \deprecated Use MGUI::FORM_COMBOBOX
DEPRECATED void MxListButtonSetItem (
	Widget listbutton,						//!< ListButton widget
	int itemnum,								//!< New item number
	int notify									//!< Set if NotifyFunc should be called
	);

//! Set number of items for list button.
//! \deprecated Use MGUI::FORM_COMBOBOX
DEPRECATED void MxListButtonSetNumItems (
	Widget listbutton,						//!< ListButton widget
	int NumItems								//!< New number of items
	);

//! Set title for "list button" pop-in dialog
//! \deprecated Use MGUI::FORM_COMBOBOX
DEPRECATED void MxListButtonSetTitle (
	Widget listbutton,						//!< ListButton widget
	const char* title							//!< Title for dialog
	);

//! Disable keyboard selection of list entries.
//! This will remove the event handler which permits key selection.  If no event handler is currently
//! logged the request will be ignored.
void MxListDisableKeySelect (
	Widget list									//!< List widget
	);

//! Enable keyboard selection of list entries.
//! This will add an event handler which permits rapid selection of entries in a list via the keyboard.
//! When the user types a character the first entry in the list (beginning with the specifed start
//! position) whose first nonblank character is greater than or equal to the typed character will be
//! selected.  This comparison in non-case-sensitive and assumes that the portion of the list to be
//! searched is sorted in ascending non-case-sensitive order.
//! This function may be called multiple times with no ill effects.  The last specified "start" value
//! will be used.  MxListDisableKeySelect() does not need to be called between invocations unless
//! desired.
void MxListEnableKeySelect (
	Widget list,								//!< List widget
	int start									//!< Starting entry to begin search (first entry is 1 like in list)
	);

//! Lock specified window minimum/maxumum sizes to current size.
//! Flags:
//!  \li\b MxLOCKSIZE_MinHeight	Set minimum height to current height
//!  \li\b MxLOCKSIZE_MinWidth	Set minimum height to current width
//!  \li\b MxLOCKSIZE_MaxHeight	Set maximum height to current height
//!  \li\b MxLOCKSIZE_MaxWidth	Set maximum height to current width
//!  \li\b MxLOCKSIZE_Min		Set minimum size to current size
//! \li\b MxLOCKSIZE_Max		Set maximum size to current size
//!  \li\b MxLOCKSIZE_MinMax	Don't allow window to be resized at all
//!
//! Note that this function MUST be invoked AFTER the window is managed or realized.
void MxLockWindowSize (
	Widget widget,								//!< Widget (ancestor shell will be used if not shell itself)
	UINT32 flags = MxLOCKSIZE_MinMax		//!< Flags - MxLOCKSIZE_MinMax will be used if 0
	);

//! Lock specified window minimum/maximum sizes to an absolute size.
//! Flags:
//!  \li\b MxLOCKSIZE_MinHeight	Set minimum height to given height
//!  \li\b MxLOCKSIZE_MinWidth	Set minimum height to given width
//!  \li\b MxLOCKSIZE_MaxHeight	Set maximum height to given height
//!  \li\b MxLOCKSIZE_MaxWidth	Set maximum height to given width
//!  \li\b MxLOCKSIZE_Min		Set minimum size to given size
//!  \li\b MxLOCKSIZE_Max		Set maximum size to given size
//!  \li\b MxLOCKSIZE_MinMax	Don't allow window to be resized at all
//!
//! Note that this function MUST be invoked AFTER the window is managed or realized.
void MxLockWindowSizeAbs (
	Widget widget,								//!< Widget (ancestor shell will be used if not shell itself)
	int height,									//!< Maximum/minimum height
	int width,									//!< Maximum/minimum width
	UINT32 flags								//!< Flags - MxLOCKSIZE_MinMax will be used if 0
	);

//! Lock specified window minimum/maximum sizes relative to current size.
//! Flags:
//!  \li\b MxLOCKSIZE_MinHeight	Set minimum height to current height
//!  \li\b MxLOCKSIZE_MinWidth	Set minimum height to current width
//!  \li\b MxLOCKSIZE_MaxHeight	Set maximum height to current height
//!  \li\b MxLOCKSIZE_MaxWidth	Set maximum height to current width
//!  \li\b MxLOCKSIZE_Min		Set minimum size to current size
//!  \li\b MxLOCKSIZE_Max		Set maximum size to current size
//! Note that this function MUST be invoked AFTER the window is managed or realized.
//! This function will set the minimum or maximum sizes of the shell to be the current
//! size plus dheight and dwidth.
void MxLockWindowSizeRel (
	Widget widget,								//!< Widget (ancestor shell will be used if not shell itself)
	int dheight,								//!< Change in height
	int dwidth,									//!< Change in width
	UINT32 flags								//!< Flags - MxLOCKSIZE_MinMax will be used if 0
	);

//! Perform "main" Xt event processing loop until MxSetExitProcess() is called.
//! This function is used instead of XtAppMainLoop() to handle event processing while allowing a nice
//! exit from the program instead of forcing the programmer to use exit().
//! Example:
//!
//! @code
//!		static void CB_Exit (
//!		   Widget w,
//!		   void *clientdata,
//!		   XmAnyCallbackStruct *cbdata
//!		   ) {
//!		      MxSetExitProcess();
//!		      }
//!
//!		main (
//! 	   int argc,
//! 	   char *argv[]
//! 	   ) {
//! 	   Widget toplevel;
//!
//! 	   toplevel = MxInit(&argc,argv,FallbackList);
//!
//! 	   ... create application widgets and such
//!
//! 	   XtRealizeWidget(toplevel);
//!		   MxMainLoopUntilExitSet();
//! 	   XtDestroyWidget(toplevel);
//!		   MxMainLoopWhilePending();
//! 	   return(0);
//! 	   }
//! @endcode
void MxMainLoopUntilExitSet (
	);

//! Main loop until a delegate returns true
void MxMainLoopUntilConditionMet (
	DELEGATE_BOOL_NOPARMS Delegate
	);

//! Perform "main" Xt event processing while events are pending.
//! See MxMainLoopUntilExitSet() for details.
void MxMainLoopWhilePending (
	);

//! Put list of file / object names in XmList.
//! \deprecated Use RVC::UI::FORM_OBJECT_LIST.
DEPRECATED void MxMakeObjectList (
	Widget list, 								//!< XmList widget
	const RVC::OBJITEMLIST& items			//!< File / object name linked list
	);

//! Make list widget scroll to be centered on specified item position.
void MxMakeListPosCentered (
	Widget listw,								//!< List widget
	int listitem								//!< Item to center on
	);

//! Make list widget position visible.
//! If 0 is passed for "listitem", the last item in the list will be made visible.
void MxMakeListPosVisible (
	Widget listw,								//!< List widget
	int listitem								//!< Item to make visible
	);

//! Load menu file, creating pulldown/cascading menus.
int MxMenuFileLoad (
	Widget pwidget,							//!< Parent widget
	const FILEPATH& filepath,				//!< Path to file to load
	XtCallbackProc CB_Internal,			//!< Function to call for "internal" options
	XtCallbackProc CB_Executable = 0,		//!< Function to call for 'executable' options
	FastDelegate<void(Widget, const char*, int)> DelegateAddSpecial = 0
	);

//! Create icon pulldown menu with items read from file.
//!
//! @return Menu widget
Widget MxMenuFileCreateIconPulldown (
	Widget pwidget,							//!< Parent widget
	const FILEPATH& filepath,
	XtCallbackProc CB_Internal,			//!< Callback function
	int IconHandle,
	const char *IconGroup,
	const char *IconName,
	const char *LabelName,
	const Arg *arglist,						//!< Argument list
	int argcount								//!< Number of arguments
	);

//! Count number of items in MENUITEM array including terminating item.
//!
//! @return number of items
int MxMenuGetNumItems (
	const MENUITEM *items
	);

//! Initialize flags to MxMENU_SELECTED if item's callback_data matched given value.
void MxMenuInitSelectedByCallbackData (
	MENUITEM *menuitem,						//!< Menu definition array
	void *cbdata								//!< Callback data to match
	);

//! Initialize flags to MxMENU_SELECTED if item's userdata matches given value.
void MxMenuInitSelectedByUserData (
	MENUITEM *menuitem,						//!< Menu definition array
	void *userdata								//!< User data to match
	);

//! Find and return the widget for a given menu item.
//!
//! @return Widget corresponding to menu item or NULL if cannot find
//! This routine finds and returns the widget for a given menu item.  This should be a string formatted as:
//!		"/file/open/..."
//! If one of the menu entries has a "/" in it, another delimiter can be used as long as it is consistent.
Widget MxMenuItemWidget (
	Widget menubar,							//!< The menubar widget
	const char *path							//!< Path
	);

//! Clear "message line".
void MxMsgLineClear (
	Widget msgline								//!< Message widget from MxMsgLineCreate()
	);

//! Create "message line" at bottom of form for displaying messages.
//!
//! @return "Message Line" widget (XeText)
//! The message widget will be attached to the bottom, left and right sides of the parent, assuming it
//! is a Form widget.  In addition, the offsets and highlightThickness will be set to 0.  The message
//! widget will not be editable.
//! WARNING: Do not use XmTextSetString() on the message widget or your program will crash.
Widget MxMsgLineCreate (
	Widget pwidget,							//!< Parent widget
	Arg *arglist,								//!< Resource argument list
	int argcount								//!< Number of entries in arglist
	);

//! Set message using first line for given error code.
void MxMsgLineSetError (
	Widget msgline,							//!< Message widget from MxMsgLineCreate()
	ERRVALUE errcode							//!< Error code
	);

//! Set message line to specified text.
void MxMsgLineSetText (
	Widget msgline,							//!< Message widget from MxMsgLineCreate()
	const MISTRING& string					//!< Text to set (also allows TEXTID)
	);

//! Set message using "timer".
void MxMsgLineSetTimer (
	Widget msgline,								//!< Message widget from MxMsgLineCreate()
	const MTIMER& timer,							//!< Timer
	TEXTID textid = TEXTID__None				//!< Text resource ID, default is TEXTID_TimeTo_Process
	);

//! Append items to OptionMenu.
void MxOptionMenuAppendItems (
	Widget omwidget,							//!< OptionMenu widget to append to 
	const MENUITEM *items,					//!< Items to append
	void *udata									//!< User data
	);

//! Get the cascade button of an option menu.
//! You can use this to adjust the size, but don't depend on it actually
//! being an XmCascadeButtonGadget, because it may change
Widget MxOptionMenuGetButton (
	Widget omwidget
	);

//! Get the label button of an option menu.
Widget MxOptionMenuGetLabel (
	Widget omwidget
	);

//! Remove last item from OptionMenu.
void MxOptionMenuRemoveLast (
	Widget omwidget							//!< OptionMenu widget
	);

//! Replace all items in an OptionMenu with a new set of items.
//! This function is useful if the items in an OptionMenu must be changed, but attachments for the
//! OptionMenu must be maintained.
void MxOptionMenuReplaceItems (
	Widget omwidget,							//!< OptionMenu widget returned by MxCreateOptionMenu
	const MENUITEM *newitems,				//!< New menu item list
	void *userdata								//!< Userdata, if NULL will use value in MENUITEM field
	);

//! Change currently selected item in an OptionMenu.
//! @return True if successful, false if not
//! This function will update the text shown in the OptionMenu button.  The callback for that item will NOT be called.
Boolean MxOptionMenuSetItem (
	Widget omwidget,					//!< OptionMenu widget returned by MxCreateOptionMenu
	int itemnum									//!< Item number from original MENUITEM array
	);

//! Set selected item by matching callback_data in MENUITEM array.
//! @return TRUE if successful, FALSE if not
Boolean MxOptionMenuSetItemByData (
	Widget widget,
	const MENUITEM *menuitem,
	void *callback_data
	);

//! Set sensitivity for an OptionMenu item.
//!
//! @return True if successful, false if not
//! This function will set the sensitivity in the OptionMenu for that item.  The callback for that item will NOT be called.
Boolean MxOptionMenuSetItemSensitive (
	Widget omwidget,							//!< Optionmenu widget returned by MxCreateOptionMenu
	int itemnum,								//!< Item number from original MENUITEM array
	Boolean flag								//!< TRUE/FALSE
	);

//! Set sensitivity for an OptionMenu.
void MxOptionMenuSetSensitive (
	Widget omwidget,
	int sensitive
	);

//! Create "normal" pixmap from fill pattern.
//!
//! @return error code
int MxPixmapFromFillPattern (
	Widget pwidget,							//!< Parent widget
	Pixel bgpixel,
	const RVC::STYLEPATTERN_BITMAP& Pattern,
	Pixmap *pixmap,								//!< Pointer to pixmap to be created
	bool bToGrayScale = false					//!< If true, convert to gray scale
	);

//! Create for for polygom fitting parameters.
//!
//! @return Form widget or NULL
Widget MxPolyfitCreateForm (
	Widget parent,								//!< Parent widget
	void(*action)(void*),
	void *userdata,
	Arg *arglist,								//!< Arguments for form
	int argcount								//!< Number of arguments in arglist
	);

//!  Get polygon fitting parameters.
//!
//! @return 0 or EBadFuncParm
int MxPolyfitGetParm (
	Widget wPolyfit,							//!< widget returned from MxPolyfitCreateForm
	void *vpf									//!< passed/returned, actually internal structure will be copied to programmer supplied pointer
	);

//! Set polygon fitting parameters.
int MxPolyfitSetParm (
	Widget wPolyfit,							//!< widget returned from MxPolyfitCreateForm
	void *vpf									//!< passed/returned
	);

//! Read values from .ini file.
//!
//! @return error code
int MxPolyfitIniRead (
	void *vpf									//!< passed/returned
	);

//! Write values to .ini file.
//!
//! @return error code
int MxPolyfitIniWrite(
	void *vpf									//!< passed/returned
	);

//! Replace all items in a Pulldown Menu with a new set of items.
void MxPopupMenuReplaceItems (
	Widget menupane,							//!< Menu to replace items
	const MENUITEM *items,					//!< New items
	void *udata
	);

//! Pop-up dialog with programmer-supplied form, optional buttons and OK/Cancel options.
//!
//!	<pre>
//!		Widget CreateWidgetFunc (form, CreateWidgetData, arglist, argcount)
//!
//!			Widget form						Form to create widget in
//!			void *CreateWidgetData		As specified above
//!			Arg *arglist					Arguments for attachment
//!			int argcount					Argument count
//!	</pre>
//!
//!	@return 0 for "OK", EUserCancel for "Cancel"
int MxPopupModalDialog (
	Widget dlgparent,							//!< Parent for dialog
	const char* dlgname,						//!< Name used to generate dialog title
	const char* helpid,						//!< Help ID, NULL if none
	Widget (*CreateWidgetFunc)(Widget,void*,Arg*,int),	//! Function to create widget to go in dialog above separator
	void* CreateWidgetData,					//!< Data to pass to CreateWidgetFunc
	const BUTTONITEM *extrabuttons = 0,	//!< Extra buttons to place between "Cancel" and "Help" if desired
	bool (*CheckOkFunc)(void*) = 0		//!< Function to check if OK to close dialog, will be called when user presses "OK" button
	);

//! Popup transient menu which will automatically be destroyed when popped down.
void MxPopupTransientMenu (
	Widget parent,								//!< Parent widget
	XButtonPressedEvent *event,			//!< ButtonPress event which triggered popup, used for positioning
	const char *title,						//!< Menu title
	const MENUITEM *items,					//!< Null-terminated array of menu items
	void *cbdata = 0,							//!< Callback data to use if not specified in MENUITEM
	void *udata = 0,							//!< Value for userData resource if not specified in MENUITEM
	bool waitforpopdown = false			//!< 'true' to wait until menu is popped down before returning
	);

//! Position one window relative to another.
//!
//!	The relative position will overlap any previously placed windows.
//!	Also, if the shell the window is placed next to moves, the window
//!	will NOT move with it.
//!
//! <pre>
//! MxRELPOSN_RightTop      
//! MxRELPOSN_LeftTop       
//! MxRELPOSN_RightCenter     TL TC TR
//! MxRELPOSN_LeftCenter    LTSSSSSSSSRT
//! MxRELPOSN_RightBottom     SWWWWWWS
//! MxRELPOSN_LeftBottom      SW    WS
//! MxRELPOSN_TopLeft       LCSW    WSRC
//! MxRELPOSN_BottomLeft      SW    WS
//! MxRELPOSN_TopCenter       SWWWWWWS
//! MxRELPOSN_BottomCenter  LBSSSSSSSSRB
//! MxRELPOSN_TopRight        BL BC BR
//! MxRELPOSN_BottomRight
//! </pre>
//!
void MxPositionWindowRel (
	Widget child,								//!< Window to be position
	Widget base,								//!< Window to position relative to
	MxRELPOSN relposn							//!< Relative position
	);


//! Add callback to prompt callback list.
//!
//! @return Error < 0
//! void cbfunc (cbstruct, cbdata)
//!  MxPromptCallbackStruct *cbstruct	Callback structure
//! 	void *cbdata				Data specified in MxPromptAddCallback()
//! MxPromptCallbackStruct {
//! 	UINT32 reason		Callback reason
//! 	Widget prompt		Prompt widget
//! 	}
//! Callback reasons
//!  MxPromptCR_ValueChanged		Pointed-to value for prompt has changed
//!  MxPromptCR_ValueChangeDrag	User is dragging XmScale slider
//!  MxPromptCR_Activate		User pressed \<Enter\> while in text widget
int MxPromptAddCallback (
	Widget prompt,								//!< Prompt widget
	void(*cbfunc)(MxPromptCallbackStruct*,void*),	//! Callback function
	void *cbdata,								//!< Callback data
	UINT32 reasonmask							//!< Requested callback reasons
	);

//! Clear current prompt value.
//!
//! This will set the prompt such that no default value is desplayed (blank string).
void MxPromptClearValue (
	Widget prompt								//!< Prompt widget
	);

//! Add callback to "Color Prompt" callback list.
//!
//! @return Error < 0
//! void cbfunc (cbstruct, cbdata)
//!  MxPromptCallbackStruct *cbstruct	Callback structure
//! 	void *cbdata				Data specified in MxPromptAddCallback()
//! MxPromptCallbackStruct {
//! 	UINT32 reason		Callback reason
//! 	Widget prompt		Prompt widget
//! 	}
//! Callback reasons
//!  MxPromptCR_ValueChanged		Pointed-to value for prompt has changed
int MxPromptColorAddCallback (
	Widget prompt,								//!< Prompt widget
	void(*cbfunc)(MxPromptCallbackStruct*,void*),	//! Callback function
	void *cbdata,								//!< Callback data
	UINT32 reasonmask							//!< Requested callback reasons
	);

//! Create "Color Prompt".
//!
//! flags:
//!   EDITCOLOR_AllowTransp		Adds transparency toggle
//!   EDITCOLOR_AllowTransPct		Allows setting of transparency 0-100%
//!
//! @return Color prompt widget
Widget MxPromptColorCreate (
	Widget pwidget,							//!< Parent widget
	const char *name,							//!< Name for label
	COLOR *color,								//!< Color to be updated by prompt
	UINT32 flags,								//!< Flags to pass to MxEditColor
	Arg *arglist,								//!< Arguments for attachment
	int argcount								//!< Number of arguments
	);

//! Get label widget for "Color Prompt".
//!
//! @return Label widget (NULL if "name" was NULL when created)
Widget MxPromptColorGetLabel (
	Widget prompt								//!< Prompt widget
	);

//! Update "Color Prompt" for external change to color.
void MxPromptColorUpdate (
	Widget prompt								//!< Color prompt
	);

//! Exported API for creating prompt string widgets for a single, non-NULL-terminated char (char or MIUNICODE).
//!
//! @return Form or Text widget
Widget MxPromptCreateChar (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name of widget
	char *str,									//!< Prompt string
	const char *help,							//!< Help code (NULL for default)
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Arguments
	int numargs									//!< Number of arguments
	);

//! Create form containing Label and Text widgets prompting for DOUBLE value.
//!
//! @return Form or Text widget
//! Valid Flags:
//! 	\li\b GET_DftRet		Use return value for default
//! 	\li\b GET_LabelTop		Label on top of text widget (defaults to left)
//! 	\li\b GET_NoForm		Don't create Form widget, just Text widget
//! 	\li\b GET_NoManage		Don't "Manage" widget returned
//! 	\li\b GET_NoMin		Ignore minimum
//! 	\li\b GET_NoMax		Ignore maximum
//! 	\li\b GET_AddOne		Add 1 to value for user, subtract 1 before return
//! 	\li\b GET_NoShowRange		Don't show the range (GET_NoShowMin|GET_NoShowMax)
//! 	\li\b GET_NoShowMin		Use the min but don't show it
//! 	\li\b GET_NoShowMax		Use the max but don't show it
//! 	\li\b GET_NoDft		There is no default value
//! 	\li\b GET_RightJustify	Right-justify value in text widget (default)
//! 	\li\b GET_LeftJustify		Left-justify value in text widget
//! 	\li\b GET_OutputOnly		Prompt is "output-only"
//! 	\li\b GET_HasScale		Add "Scale" widget to prompt (requires min/max settings)
//! 	\li\b GET_UseLogScale		Use "logarithmic" scaling (must also specify GET_HasScale)
//! 	\li\b GET_BlankZero		Show zero as blank
//! 	\li\b GET_ShowUnitLabel	Show "unit" label
//!
//! Logarithmically-scaled prompts will automatically adjust the range of the scale slider based
//! on the current value within the limits set by MxPromptSetLogScaleRange() and the minimum and
//! maximum (if any) allowed values of the prompt.  In the case of log-scaled prompts, the minimum
//! value must be greater than 0.  Also, no maximum value is required.  This is useful in cases where
//! a scale is useful but the extent of the range is very large or has no upper bound.
//! If GET_ShowUnitLabel is specified you must invoke MxPromptSetUnitConvCode() or set the unit
//! name yourself, otherwise the units will show as '?????'.
Widget MxPromptCreateDouble (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	double *ret,								//!< Location of return value
	double dft,									//!< Default value
	double min,									//!< Minimum value
	double max,									//!< Maximum value
	int places,									//!< Decimal places to show
	const char *help,							//!< Help code (NULL for default)
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Argument list used for widget returned
	int n											//!< Number of entries in argument list
	);

//! Create form containing Label and Text widgets prompting for FLOAT value.
//!
//! @return Form or Text widget
//! Valid Flags:
//! 	\li\b GET_DftRet		Use return value for default
//! 	\li\b GET_LabelTop		Label on top of text widget (defaults to left)
//! 	\li\b GET_NoForm		Don't create Form widget, just Text widget
//! 	\li\b GET_NoManage		Don't "Manage" widget returned
//! 	\li\b GET_NoMin		Ignore minimum
//! 	\li\b GET_NoMax		Ignore maximum
//! 	\li\b GET_AddOne		Add 1 to value for user, subtract 1 before return
//! 	\li\b GET_NoShowRange		Don't show the range (GET_NoShowMin|GET_NoShowMax)
//! 	\li\b GET_NoShowMin		Use the min but don't show it
//! 	\li\b GET_NoShowMax		Use the max but don't show it
//! 	\li\b GET_NoDft		There is no default value
//! 	\li\b GET_RightJustify	Right-justify value in text widget (default)
//! 	\li\b GET_LeftJustify		Left-justify value in text widget
//! 	\li\b GET_OutputOnly		Prompt is "output-only"
//! 	\li\b GET_HasScale		Add "Scale" widget to prompt (requires min/max settings)
//! 	\li\b GET_UseLogScale		Use "logarithmic" scaling (must also specify GET_HasScale)
//! 	\li\b GET_BlankZero		Show zero as blank
//! 	\li\b GET_ShowUnitLabel	Show "unit" label
//!
Widget MxPromptCreateFloat (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	float *ret,									//!< Location of return value
	float dft,									//!< Default value
	float min,									//!< Minimum value
	float max,									//!< Maximum value
	int places,									//!< Decimal places to show
	const char *help,							//!< Help code (NULL for default)
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Argument list used for widget returned
	int n											//!< Number of entries in argument list
	);

//! Create form containing Label and Text widgets prompting for INT8 value.
//!
//! @return Form or Text widget
Widget MxPromptCreateINT8 (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	INT8 *ret,									//!< Location of return value
	INT8 dft,									//!< Default value
	INT8 min,									//!< Minimum value
	INT8 max,									//!< Maximum value
	const char *help,							//!< Help code (NULL for default)
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Argument list used for widget returned
	int n											//!< Number of entries in argument list
	);

//! Create form containing Label and Text widgets prompting for INT16 value.
//!
//! @return Form or Text widget
//! Valid Flags:
//! 	\li\b GET_DftRet		Use return value for default
//! 	\li\b GET_LabelTop		Label on top of text widget (defaults to left)
//! 	\li\b GET_NoForm		Don't create Form widget, just Text widget
//! 	\li\b GET_NoManage		Don't "Manage" widget returned
//! 	\li\b GET_NoMin		Ignore minimum
//! 	\li\b GET_NoMax		Ignore maximum
//! 	\li\b GET_AddOne		Add 1 to value for user, subtract 1 before return
//! 	\li\b GET_NoShowRange		Don't show the range (GET_NoShowMin|GET_NoShowMax)
//! 	\li\b GET_NoShowMin		Use the min but don't show it
//! 	\li\b GET_NoShowMax		Use the max but don't show it
//! 	\li\b GET_NoDft		There is no default value
//! 	\li\b GET_RightJustify	Right-justify value in text widget (default)
//! 	\li\b GET_LeftJustify		Left-justify value in text widget
//! 	\li\b GET_OutputOnly		Prompt is "output-only"
//! 	\li\b GET_HasScale		Add "Scale" widget to prompt
//! 	\li\b GET_BlankZero		Show zero as blank
Widget MxPromptCreateINT16 (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	INT16 *ret,									//!< Location of return value
	INT16 dft,									//!< Default value
	INT16 min,									//!< Minimum value
	INT16 max,									//!< Maximum value
	const char *help,							//!< Help code (NULL for default)
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Argument list used for widget returned
	int numargs									//!< Number of entries in argument list
	);

//! Create form containing Label and Text widgets prompting for INT32 value.
//!
//! @return Form or Text widget
//! Valid Flags:
//! 	\li\b GET_DftRet		Use return value for default
//! 	\li\b GET_LabelTop		Label on top of text widget (defaults to left)
//! 	\li\b GET_NoForm		Don't create Form widget, just Text widget
//! 	\li\b GET_NoManage		Don't "Manage" widget returned
//! 	\li\b GET_NoMin		Ignore minimum
//! 	\li\b GET_NoMax		Ignore maximum
//! 	\li\b GET_AddOne		Add 1 to value for user, subtract 1 before return
//! 	\li\b GET_NoShowRange		Don't show the range (GET_NoShowMin|GET_NoShowMax)
//! 	\li\b GET_NoShowMin		Use the min but don't show it
//! 	\li\b GET_NoShowMax		Use the max but don't show it
//! 	\li\b GET_NoDft		There is no default value
//! 	\li\b GET_RightJustify	Right-justify value in text widget (default)
//! 	\li\b GET_LeftJustify		Left-justify value in text widget
//! 	\li\b GET_OutputOnly		Prompt is "output-only"
//! 	\li\b GET_HasScale		Add "Scale" widget to prompt
//! 	\li\b GET_BlankZero		Show zero as blank
Widget MxPromptCreateINT32 (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	INT32 *ret,									//!< Location of return value
	INT32 dft,									//!< Default value
	INT32 min,									//!< Minimum value
	INT32 max,									//!< Maximum value
	const char *help,							//!< Help code (NULL for default)
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Argument list used for widget returned
	int n											//!< Number of entries in argument list
	);

//! Create form containing Label and Text widgets prompting for LatLon value.
//!
//! @return Form or Text widget
//! Valid Flags:
//! 	\li\b GET_DftRet		Use return value for default
//! 	\li\b GET_LabelTop		Label on top of text widget (defaults to left)
//! 	\li\b GET_NoForm		Don't create Form widget, just Text widget
//! 	\li\b GET_NoManage		Don't "Manage" widget returned
//! 	\li\b GET_NoMin		Ignore minimum
//! 	\li\b GET_NoMax		Ignore maximum
//! 	\li\b GET_NoShowRange		Don't show the range (GET_NoShowMin|GET_NoShowMax)
//! 	\li\b GET_NoShowMin		Use the min but don't show it
//! 	\li\b GET_NoShowMax		Use the max but don't show it
//! 	\li\b GET_NoDft		There is no default value
//! 	\li\b GET_DegsNS		Coordinates are in Latitude
//! 	\li\b GET_DegsEW		Coordinates are in Longitude
//! 	\li\b GET_DegsNoDir		No direction for Lat/Lon value
//! 	\li\b GET_RightJustify	Right-justify value in text widget (default)
//! 	\li\b GET_LeftJustify		Left-justify value in text widget
//! 	\li\b GET_OutputOnly		Prompt is "output-only"
Widget MxPromptCreateLatLon (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	double *ret,								//!< Location of return value
	double dft,									//!< Default value
	double min,									//!< Minimum value
	double max,									//!< Maximum value
	int places,									//!< Decimal places to show
	const char *help,							//!< Help code (NULL for default)
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Argument list used for widget returned
	int n											//!< Number of entries in argument list
	);

//! Create numeric prompt field.
//!
//! @return Form or Text widget
//! Prompt types:
//! 	\li\b MXPROMPT_DOUBLE		64-bit floating-point
//! 	\li\b MXPROMPT_FLOAT		32-bit floating-point
//! 	\li\b MXPROMPT_UINT32		Unsigned 32-bit number
//! 	\li\b MXPROMPT_INT32		Signed 32-bit number
//! 	\li\b MXPROMPT_UINT16		Unsigned 16-bit number
//! 	\li\b MXPROMPT_INT16		Signed 16-bit number
//! 	\li\b MXPROMPT_UINT8		Unsigned 8-bit number
//! 	\li\b MXPROMPT_INT8		Signed 8-bit number
//!
///! Valid Flags:
//! 	\li\b GET_DftRet		Use return value for default
//! 	\li\b GET_LabelTop		Label on top of text widget (defaults to left)
//! 	\li\b GET_NoForm		Don't create Form widget, just Text widget
//! 	\li\b GET_NoManage		Don't "Manage" widget returned
//! 	\li\b GET_NoMin		Ignore minimum
//! 	\li\b GET_NoMax		Ignore maximum
//! 	\li\b GET_AddOne		Add 1 to value for user, subtract 1 before return
//! 	\li\b GET_NoShowRange		Don't show the range (GET_NoShowMin|GET_NoShowMax)
//! 	\li\b GET_NoShowMin		Use the min but don't show it
//! 	\li\b GET_NoShowMax		Use the max but don't show it
//! 	\li\b GET_NoDft		There is no default value
//! 	\li\b GET_RightJustify	Right-justify value in text widget (default)
//! 	\li\b GET_LeftJustify		Left-justify value in text widget
//! 	\li\b GET_OutputOnly		Prompt is "output-only"
//! 	\li\b GET_HasScale		Add "Scale" widget to prompt (requires min/max settings)
//! 	\li\b GET_UseLogScale		Use "logarithmic" scaling (must also specify GET_HasScale)
//! 	\li\b GET_BlankZero		Show zero as blank
//! 	\li\b GET_ShowUnitLabel	Show "unit" label
Widget MxPromptCreateNum (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	int type,									//!< Prompt type (MXPROMPT_...)
	void *ret,									//!< Location of return value
	double dft,									//!< Default value
	double min,									//!< Minimum value
	double max,									//!< Maximum value
	int places,									//!< Decimal places to show
	const char *help,							//!< Help code (NULL for default)
	char **comboitems,						//!< NULL-terminated array of items for combo-menu, NULL if no combo menu
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Argument list used for widget returned
	int n											//!< Number of entries in argument list
	);

//! Create string prompt field.
//!
//! @return Form or Text widget
//! Prompt types:
//! 	\li\b MXPROMPT_ASTRING	ASCII string
//! 	\li\b MXPROMPT_USTRING	Unicode string
//!
//! Valid Flags:
//! 	\li\b GET_LabelTop		Label on top of text widget (defaults to left)
//! 	\li\b GET_NoForm		Don't create Form widget, just Text widget
//! 	\li\b GET_NoManage		Don't "Manage" widget returned
//! 	\li\b GET_ToUpper		Convert all characters typed to upper-case
//! 	\li\b GET_Unicode		String is actually (MIUNICODE *)
//! 	\li\b GET_RightJustify	Right-justify value in text widget
//! 	\li\b GET_LeftJustify		Left-justify value in text widget (default)
Widget MxPromptCreateString (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	char *str,									//!< Pointer to string
	int maxlen,									//!< Maximum allowed length for string in characters
	const char *help,							//!< Help code
	UINT32 flags,								//!< Flags
	Arg *args,									//!< argument list used for widget returned
	int numargs									//!< Number of entries in argument list
	);

//! Create form containing Label and Text widgets prompting for UINT8 value.
//!
//! @return Form or Text widget
Widget MxPromptCreateUINT8 (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	UINT8 *ret,									//!< Location of return value
	UINT8 dft,									//!< Default value
	UINT8 min,									//!< Minimum value
	UINT8 max,									//!< Maximum value
	const char *help,							//!< Help code (NULL for default)
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Argument list used for widget returned
	int n											//!< Number of entries in argument list
	);

//! Create form containing Label and Text widgets prompting for UINT16 value.
//!
//! @return Form or Text widget
//! Valid Flags:
//! 	\li\b GET_DftRet		Use return value for default
//! 	\li\b GET_LabelTop		Label on top of text widget (defaults to left)
//! 	\li\b GET_NoForm		Don't create Form widget, just Text widget
//! 	\li\b GET_NoManage		Don't "Manage" widget returned
//! 	\li\b GET_NoMin		Ignore minimum
//! 	\li\b GET_NoMax		Ignore maximum
//! 	\li\b GET_AddOne		Add 1 to value for user, subtract 1 before return
//! 	\li\b GET_NoShowRange		Don't show the range (GET_NoShowMin|GET_NoShowMax)
//! 	\li\b GET_NoShowMin		Use the min but don't show it
//! 	\li\b GET_NoShowMax		Use the max but don't show it
//! 	\li\b GET_NoDft		There is no default value
//! 	\li\b GET_RightJustify	Right-justify value in text widget (default)
//! 	\li\b GET_LeftJustify		Left-justify value in text widget
//! 	\li\b GET_OutputOnly		Prompt is "output-only"
//! 	\li\b GET_HasScale		Add "Scale" widget to prompt
//! 	\li\b GET_BlankZero		Show zero as blank
Widget MxPromptCreateUINT16 (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	UINT16 *ret,								//!< Location of return value
	UINT16 dft,									//!< Default value
	UINT16 min,									//!< Minimum value
	UINT16 max,									//!< Maximum value
	const char *help,							//!< Help code (NULL for default)
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Argument list used for widget returned
	int n											//!< Number of entries in argument list
	);

//! Create form containing Label and Text widgets prompting for UINT32 value.
//!
//! @return Form or Text widget
//! Valid Flags:
//! 	\li\b GET_DftRet		Use return value for default
//! 	\li\b GET_LabelTop		Label on top of text widget (defaults to left)
//! 	\li\b GET_NoForm		Don't create Form widget, just Text widget
//! 	\li\b GET_NoManage		Don't "Manage" widget returned
//! 	\li\b GET_NoMin		Ignore minimum
//! 	\li\b GET_NoMax		Ignore maximum
//! 	\li\b GET_AddOne		Add 1 to value for user, subtract 1 before return
//! 	\li\b GET_NoShowRange		Don't show the range (GET_NoShowMin|GET_NoShowMax)
//! 	\li\b GET_NoShowMin		Use the min but don't show it
//! 	\li\b GET_NoShowMax		Use the max but don't show it
//! 	\li\b GET_NoDft		There is no default value
//! 	\li\b GET_RightJustify	Right-justify value in text widget (default)
//! 	\li\b GET_LeftJustify		Left-justify value in text widget
//! 	\li\b GET_OutputOnly		Prompt is "output-only"
//! 	\li\b GET_HasScale		Add "Scale" widget to prompt (requires min/max settings)
//! 	\li\b GET_BlankZero		Show zero as blank
Widget MxPromptCreateUINT32 (
	Widget parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	UINT32 *ret,								//!< Location of return value
	UINT32 dft,									//!< Default value
	UINT32 min,									//!< Minimum value
	UINT32 max,									//!< Maximum value
	const char *help,							//!< Help code (NULL for default)
	UINT32 flags,								//!< Flags
	Arg *args,									//!< Argument list used for widget returned
	int n											//!< Number of entries in argument list
	);

//! Create form containing Label and Text widgets prompting for Unicode value.
//!
//! @return Form or Text widget
MDLGPARENT MxPromptCreateUnicode (
	MDLGPARENT parent,								//!< Parent widget
	const char *name,							//!< Name for Label widget (if any)
	MIUNICODE *str,								//!< Locat
	int maxlen,									//!< Maximum allowed length for string in characters
	const char *help,
	UINT32 flags, 
	Arg *args,
	int numargs
	);

//! Disable automatic traversal when user presses \<Enter\>.
void MxPromptDisableTraversal (
	Widget prompt								//!< Prompt widget
	);

//! Set double value in prompt.
//!
//! @return 0
int MxPromptSetValueDouble (
	Widget prompt,								//!< Widget to prompt to set value
	double val									//!< Value to set it to
	);

//! Convert double prompt to a lat/lon prompt.
//!
//! This function allows conversion between double ane lat/lon values.  Useful for projection queries.
int MxPromptDoubleToLatLon (
	Widget prompt								//!< Widget to prompt to set value
	);

//! Set float value in prompt.
//!
//! @return 0
int MxPromptSetValueFloat (
	Widget prompt,								//!< Widget to prompt to set value
	float val									//!< Value to set it to
	);

//! Get Label widget from composite form returned by MxPromptCreate...
//!
//! @return Label widget (NULL if GET_NoForm used when creating prompt)
Widget MxPromptGetLabel (
	Widget prompt								//!< Prompt widget
	);

//! Return current min/max range settings
void MxPromptGetMinMax (
	Widget widget,								//!< Prompt widget
	double *min,								//!< Minimum value returned
	double *max									//!< Maximum value returned
	);

//! Get Scale widget from composite form returned by MxPromptCreate...
//!
//! @return Scale widget, NULL if no Scale widget available
Widget MxPromptGetScale (
	Widget prompt								//!< Prompt widget
	);

//! Get height of standard prompt widget.
//!
//! @return height
int MxPromptGetStdHeight (
	void
	);

//! Return unit Label widget for prompt.
//!
//! @return Unit label
Widget MxPromptGetUnitLabel (
	Widget widget								//!< Prompt widget
	);

//! Get UserData attached to the prompt.
//!
//! @return UserData, NULL if none available
void * MxPromptGetUserData (
	Widget prompt								//!< Prompt widget
	);

//! Get the numeric value out of a numeric prompt.
//!
//! @return 1 if successful, 0 if not
int MxPromptGetValueDouble (
	Widget widget,								//!< Prompt widget
	double *pval								//!< Returned numeric value
	);

//! Get the string value out of a string prompt.
//!
//! @return 1 if successful, 0 if not
int MxPromptGetValueUnicode (
	Widget widget,								//!< Prompt widget
	MIUNICODE **pval								//!< Returned string value
	);

//! Get the text value out of a prompt.
//!
//! @return widget
//! Same as old MxPromptGetText but named in such a way as to remind the programmer that it's an XeText widget.
Widget MxPromptGetXeText (
	Widget widget
	);

//! Determine if value has been set for prompt.
//!
//! @return TRUE if value is set, FALSE if not
int MxPromptHasValue (
	Widget prompt								//!< Prompt widget
	);

//! Return pointer to MENUITEM array for supported lat-lon formats.
//!
//! @return MENUITEM array
MENUITEM * MxPromptLatLonGetFormatMenu (
	XtCallbackProc cbfunc,					//!< Callback function
	int isoption								//!< Set if for OptionMenu
	);

//! Set lat/lon format in prompt.
void MxPromptLatLonSetFormat (
	Widget widget,								//!< Widget to prompt to set format
	DEGTOSTRFLAGS flags						//!< Flags
	);

//! Set lat/lon value in prompt.
//! @return 0
int MxPromptLatLonSetValue (
	Widget prompt,								//!< Widget to prompt to set value
	double val									//!< Value to set it to
	);

//! Convert lat/lon prompt to a double prompt.
//! This function allows conversion between double and lat/lon values.  Useful for projection queries.
int MxPromptLatLonToDouble (
	Widget prompt								//!< Widget to prompt to set value
	);

//! Set long value in prompt.
//! @return 0
int MxPromptSetValueINT32 (
	Widget prompt,								//!< Widget to prompt to set value
	INT32 val									//!< Value to set it to
	);

//! Remove callback from prompt callback list.
//! @return Error < 0
//! Callback reasons
//!  \li\b MxPromptCR_ValueChanged				Pointed-to value for prompt has changed
//!  \li\b MxPromptCR_ValueChangeDrag			User is dragging XmScale slider
//!  \li\b MxPromptCR_Activate					User pressed \<Enter\> while in text widget
int MxPromptRemoveCallback (
	Widget prompt,								//!< Prompt widget
	void(*cbfunc)(MxPromptCallbackStruct*,void*),	//! Callback function
	void *cbdata,								//!< Callback data
	UINT32 reasonmask							//!< Requested callback reasons
	);

//! Set number of decimal places for floating-point prompt.
void MxPromptSetDecPlaces (
	Widget prompt,								//!< Prompt widget
	int places									//!< Number of decimal places
	);

//! Set "editable" status for prompt.
void MxPromptSetEditable (
	Widget prompt,								//!< Prompt widget
	int editable								//!< TRUE if prompt should be editable, FALSE if not
	);

//! Set the encoding used by the prompt widget.
//! This function lets you set the character encoding of the data buffer
//! used by the widget for a prompt created via MxPromptCreateString().
//! The encoding is any string returned from MucNumToName().  By default,
//! the widget assumes the buffer is in ISO-2022 encoding (ISO-Latin-1
//! with escape sequences used to switch to other character sets).
int MxPromptSetEncoding (
	Widget prompt,								//!< Prompt widget
	char *encoding,							//!< Encoding to use
	UINT32 flags								//!< Flags for future expansion
	);

//! Set function to be called when user presses \<Enter\> in prompt.
void MxPromptSetEnterCallback (
	Widget widget,								//!< Prompt widget
	void(*func)(Widget, void*),			//! Callback function
	void *cbdata								//!< Callback data
	);

//! Override default format for prompt field.
//! Note: This has no effect on LatLon and String prompts (will be retained for LatLonToDouble though).
void MxPromptSetFormat (
	Widget prompt,								//!< Prompt widget
	char *format								//!< New format string (printf style, max 40 characters)
	);

//! Set minimum range for logarithmic scale.
//! This function is used for logarithmic-scale prompts to set the minimum range.  The range will
//! be from 10 ^ floor(log10(min)) to 10 ^ ceil(log10(max)) so it will probably be clearest to use
//! powers of 10 for the min/max values.
void MxPromptSetLogScaleRange (
	Widget prompt,								//!< Prompt widget
	double min,									//!< Minimum value (must be > 0.0)
	double max									//!< Maximum value (must be > min)
	);

//! Set function to be called when prompt value is changed.
//! This function is obsolete and should be replaced by MxPromptAddCallback()
void MxPromptSetModifyCallback (
	Widget prompt,								//!< Prompt widget
	void(*func)(Widget, void*),			//! Function to call when value is changed
	void *funcdata								//!< Additional data to pass to function
	);

//! Set new range for prompt, subject to limits by data type.
//! This function has no effect on the range displayed (if any).
void MxPromptSetRange (
	Widget prompt,								//!< Prompt widget
	double min,									//!< Minimum allowed value
	double max,									//!< Maximum allowed value
	UINT32 flags								//!< Flags (GET_NoMin, GET_NoMax)
	);

//! Set traversal behavior for prompt.
//! Note that prompts will default to TRAVERSE_NEXT if GET_NoForm is specified on creation
//! otherwise to TRAVERSE_NEXT_TAB_GROUP.
void MxPromptSetTraversal (
	Widget prompt,								//!< Prompt widget
	Widget traverseto,						//!< Widget to traverse to if traversedit is XmTRAVERSE_CURRENT
	XmTraversalDirection traversedir		//!< XmTRAVERSE...
	);

//! Set conversion to be performed for numeric fields.
//! Notes:  This will override the GET_AddOne flag if set.
//! 	This has no effect on LatLon and String prompts (will be retained for LatLonToDouble though).
//! 	This function will NOT alter the stored data, only how it is displayed.
void MxPromptSetUnitConv (
	Widget prompt,								//!< Prompt widget
	const UNITCONV *unitconv						//!< Conversion (from = internal, to = display)
	);

//! Set unit conversion for numeric fields using unit code values.
//! Notes:  This will override the GET_AddOne flag if set.
//! 	This has no effect on LatLon and String prompts (will be retained for LatLonToDouble though).
//! 	This function will NOT alter the stored data, only how it is displayed.
//! 	If GET_ShowUnitLabel was specified when the prompt was created the label will be updated.
void MxPromptSetUnitConvCode (
	Widget prompt,								//!< Prompt widget
	int unittype,								//!< Unit type (UNIT_TYPE_...)
	int storedcode,							//!< Unit code for how the value is stored
	int showncode								//!< Unit code for how the value is displayed to the user
	);

//! Set conversion for numeric prompt.
//! This does not change the value of the prompt
void MxPromptSetUnitConvExt (
	Widget widget,								//!< Prompt widget
	const UNITCONV *unitconv,				//!< Conversion type
	UINT32 flags								//!< Flags
	);

//! Attach UserData to the prompt.
//! Use MxPromptGetUserData() to get the data, not MxGetUserData()
void  MxPromptSetUserData (
	Widget prompt,								//!< Prompt widget
	void *data									//!< UserData to attach
	);

//! Set new value.  Takes a double and updates any type of prompt.
void MxPromptSetValue (
	Widget widget,								//!< Prompt widget
	double wal,									//!< Value to set
	int hasvalue								//!< TRUE if valid value available, FALSE if not
	);

//! Set new pointer to prompt value.
void MxPromptSetValuePtr (
	Widget prompt,								//!< Prompt widget
	void *newptr,								//!< New value pointer to use
	int hasvalue								//!< TRUE if valid value available, FALSE if not
	);

//! Set short value in prompt.
//!
//! @return 0
int MxPromptSetValueINT16 (
	Widget prompt,								//!< Widget to prompt to set value
	short val									//!< Value to set it to
	);

//! Set string value in prompt.
//!
//! @return 0
int MxPromptSetValueString (
	Widget prompt,								//!< Widget to prompt to set value
	const char *val							//!< New string value
	);

//! Set a Unicode string value in prompt (must already be a Unicode prompt).
//!
//! @return 0
int MxPromptSetValueUnicode (
	Widget prompt,								//!< Widget to prompt to set value
	const MIUNICODE *val						//!< New string value
	);

//! Notify prompt widget that "pointed-to" value has changed.
void MxPromptUpdateValue (
	Widget prompt								//!< Prompt widget
	);

//! Update the data "pointed-to" to reflect the contents of the prompt.
//!
//! @return 1 if data modified, False if no change was needed
//! This function updates the location in memory that the prompt points to.  This is done automatically
//! when the prompt loses focus, but sometimes it's necessary to do this while the prompt still has 
//! focus.  For example, clicking on a scroll-bar won't cause the prompt to lose focus, but you might
//! want to make sure the data is updated anyway.
//! If "notify" is true and the data has changed, the prompt's modify callback will be called.
int MxPromptUpdateData (
	Widget prompt,								//!< Prompt widget
	Boolean notify								//!< Call "modify" callback if value changed
	);

//! Replace all items in a Pulldown Menu with a new set of items.
void MxPulldownMenuReplaceItems (
	Widget parent,								//!< Parent widget, should be a CascadeButton
	const MENUITEM *items,					//!< Array of menu items, terminated by NULL item
	void *userdata								//!< User data for items created if not specified in items
	);

//! Redraw widget by sending exposure notification.
void MxRedrawWidget (
	Widget widget							//!< Widget to redraw
	);

//! Setup for tracing solid area in raster using vector elements.
int MxRgnGenSetupRasterTraceAreaV (
	Widget pwidget,
	const RVC::OBJITEM& VectObjItem,
	RGNELEMLIST *ElemList,
	int NumElemTypes,
	void(*DispRegionFunc)(const REGION2D*,void*),	//! Function to display region preview
	void *DispRegionData					//!< Data for region display function
	);

//! Setup for tracing bounded area using vector elements.
int MxRgnGenSetupRasterTraceBoundaryV (
	Widget pwidget,
	const RVC::OBJITEM& VectObjItem,
	RGNELEMLIST *ElemList,
	int NumElemTypes,
	void(*DispRegionFunc)(const REGION2D*,void*),	//! Function to display region preview
	void *DispRegionData					//!< Data for region display function
	);

//! Create simple horizontal or vertical "sash" control in XmForm.
//!
//! @return Widget
//! To set the orientation of the sash, set XmNorientation to XmVERTICAL or XmHORIZONTAL in
//! the argument list.  The default orientatation is vertical.
//! When the user moves the sash, widgets attached to the sash or to which the sash is
//! attached will be resized appropriately.  These widgets must be resizable or the results
//! are unpredictable.
//! Currently does not support ATTACH_OPPOSITE_WIDGET attachments.
//! Currently assumes widgets are attached to the form in the adjustment direction on the
//! sides opposite the sash.  This also means that only one sash per form is allowed.
Widget MxSashCreate (
	Widget pwidget,							//!< Parent widget
	Arg *arglist,								//!< Arguments for attachment and orientation
	int argcount								//!< Number of arguments
	);

//! Save "snapshot" of X drawable into a user-specified raster.
//! @return Error code < 0
//! Saving "Window" drawables is not recommended unless you ensure that the window is not obscured.
int MxSaveSnapshotToRaster (
	Widget pwidget,							//!< Scrollbar widget
	const DISPLAYINFO& dispinfo,			//!< Used to retrieve information about display
	Drawable drawable							//!< Drawable (Window or Pixmap) to save from
	);

//! Return values for ScrollBar resources.
//! This function should be used instead of XmScrollBarGetValues() as that function has an error in Motif 1.2.3.
void MxScrollBarGetValues (
	Widget w,									//!< ScrollBar widget
	int *value,									//!< Returned "value" resource (NULL if not needed)
	int *sliderSize,							//!< Returned "slidersize" resource (NULL if not needed)
	int *increment,							//!< Returned "increment" resource (NULL if not needed)
	int *pageIncrement,						//!< Returned "pageIncrement" resource (NULL if not needed)
	int *minimum,								//!< Returned "minimum" resource (NULL if not needed)
	int *maximum								//!< Returned "maximum" resource (NULL if not needed)
	);

//! Pop in dialog allowing user to select an icon.
//! @return 1 if selected icon changed, 0 if not, < 0 if error
int MxSelectIcon (
	Widget pwidget,							//!< Parent for dialog
	char *dialogname,							//!< Name of dialog, will use "MxSelectIcon" if NULL
	char IconClass[16],						//!< Icon class passed/returned
	char IconName[16],						//!< Icon name passed/returned
	int minsize,								//!< Minimum icon size, resizes smaller icons if SELICON_Resize flag set
	int maxsize,								//!< Maximum icon size, 0 if no maximum
	UINT32 flags								//!< SELICON_... flags
	);

//! Select a rotated rectangle or a polygon, single region
int MxSelectRegion (
	Widget parent,								//!< Parent dialog
	const RVC::OBJITEM& ObjItem,
	RVC::OBJTYPE ObjType,
	REGION2D& region
	);

//! Notify MxMainLoopUntilExitSet() that it should terminate.
//! See MxMainLoopUntilExitSet() for details
void MxSetExitProcess ();

//! Set exitmainloop to 0
void MxSetNoExitProcess ();

//! Set keyboard focus to widget and pointer focus to window containing widget.
void MxSetFocusToWidget (
	Widget widget
	);

//! Set help callback.
inline void MxSetHelpCallback (
	Widget widget,
	const char *helpstr						//!< Help code
	) { XtAddCallback(widget,XmNhelpCallback,MxcbDisplayHelp,const_cast<char*>(helpstr)); }

//! Set "callback_data" field for each entry MENUITEM array to specified value.
void MxSetMenuItemCallbackData (
	MENUITEM *mi,								//!< Menu item to set callback data for
	void *cbdata								//!< Callback data
	);

//! Set "userdata" field for each entry MENUITEM array to specified value.
void MxSetMenuItemUserData (
	MENUITEM *mi,								//!< Menu item to set user data for
	void *udata									//!< User data
	);

//! Add, change, or remove a tagged user data resource for a widget.
//! This routine manages multiple userdata resources on a widget using
//! a tag.  To remove a resource, use MxSetNamedUserData(w, name, NULL)
void MxSetNamedUserData (
	Widget w,									//!< Widget to attach tagged user data to
	const char *name,							//!< String to use as a tag (Must be less than 16 characters
	void *data									//!< Data pointer to attach to widget
	);

//! Set widget sensitivity with test for NULL widget.
void MxSetSensitive (
	Widget w,									//!< Widget to set sensitivity for (NULL ok)
	int sensitive								//!< TRUE or FALSE
	);

//! Set whether or not a Text widget is editable.
//! This sets the "editable, "cursorPositionVisible" and "traversalOn" resources for the Text widget.
void MxSetTextEditable (
	Widget widget,								//!< Text widget
	int editable								//!< TRUE if widget should be editable, FALSE if not
	);

// Set ToolTip from RVC::OBJITEM.
void MxSetToolTipFromObjItem (
	Widget widget,
	const RVC::OBJITEM& ObjItem
	);

//! Setup polygon fitting parameters for region generation.
int MxSetupPolygonFitParms (
	Widget parent,								//!< Parent widget
	const RVC::OBJITEM& SrcGeomObjItem,
	RGNELEMLIST *ElemList,
	int NumElemTypes,							//!< Number of element types
	void (*dispreg)(const REGION2D*, void*),
	void *userdata
	);

//! Setup clustering parameters for region generation.
int MxSetupClusterParms (
	Widget parent,								//!< Parent widget
	const RVC::OBJITEM& SrcGeomObjItem,
	RGNELEMLIST *ElemList,
	int NumElemTypes,							//!< Number of element types
	void (*dispreg)(const REGION2D*, void*),
	void *userdata
	);

//! Setup buffer zone interface.
int MxSetupBufferZoneParms (
	void *form,
	const RVC::OBJITEM& SrcObjItem,
	RGNELEMLIST *ElemList,
	int NumElemTypes,							//!< Number of element types
	void (*dispreg)(const REGION2D*, void*),
	void *userdata
	);

//! Setup range parameters for region generation.
int MxSetupRegionFromRangesParms (
	Widget parent,								//!< Parent widget
	const RVC::OBJITEM& ObjItem,
	void (*dispreg)(const REGION2D*, void*),
	void *userdata
	);

//! Setup raster expression parameters for region generation.
int MxSetupRegionFromRastExpressParms (
	Widget parent,								//!< Parent widget
	const RVC::OBJITEM& ObjItemRaster,
	void (*dispreg)(const REGION2D*, void*),
	void *userdata
	);

//! Setup for region generation using viewshed method.
int MxSetupViewshedParms (
	MDLGPARENT parent,
	const RVC::OBJITEM& VectObjItem,
	UINT8 *setdata,
	INT32 NumEntries,
	void (*dispreg)(const REGION2D*, void*),
	void *userdata
	);

//! Setup for region generation using "basin" method.
int MxSetupBasinParms (
	void *form,
	const RVC::OBJITEM& SrcVectObjItem,
	RGNELEMLIST *ElemList,
	int NumElemTypes,							//!< Number of element types
	void (*dispreg)(const REGION2D*, void*),
	void *userdata
	);

//! Setup for region generation using "texture growth" method.
int MxSetupTextureGrowthParms (
	MDLGPARENT parent,
	const RVC::OBJITEM& VectObjItem,
	RGNELEMLIST *ElemList,
	int NumElemTypes,							//!< Number of element types
	void (*dispreg)(const REGION2D*, void*),
	void *userdata
	);

//! Setup for region generation using "flood fill" method.
int MxSetupFloodParms (
	void *vparent,
	const RVC::OBJITEM& ObjItemVector,
	INT32 ActiveType,
	INT32 ActiveLine,
	void (*dispreg)(const REGION2D*, void*),
	void *userdata
	);

//! Setup for region generation using "raster cells" method.
int MxSetupRasterCellsParms (
	MDLGPARENT parent,
	const RVC::OBJITEM& VectObjItem,
	RGNELEMLIST *ElemList,
	int NumElemTypes,
	void (*dispreg)(const REGION2D*, void*),
	void *userdata
	);

//! Set widget foreground color to specified color.
void MxSetWidgetColor (
	Widget widget,								//!< Widget to set foreground color for
	COLOR const& color						//!< Color value to set
	);

//! Set window containing specified widget to full-screen size.
void MxSetWindowFullScreen (
	Widget widget								//!< Widget contained in window to be sized to full-screen
	);

//! Read and set window position from .ini file, update when changed.
//! @return Error code if < 0 or 0 if no error
int MxSetWindowPosition (
	Widget widget,								//!< Widget to be positioned (ancestor WMShell will actually be used)
	const char *group,						//!< Major heading, (application name)
	const char *field							//!< Field name
	);

//! Set window position from tntmips.ini, update when changed.
//! @return Error code if < 0 or 0 if no error
int MxSetWindowPositionXY (
	Widget widget,								//!< Widget to be positioned
	const char *group,						//!< Major heading, (application name)
	const char *field,						//!< Field name
	int x,
	int y
	);

//! Read and set window size from .ini file, update when changed.
//! @return Error code if < 0 or 0 if no error
//! If the 'group' field is set to NULL, GetAppName() will be used to set it internally
int MxSetWindowSize (
	Widget widget,								//!< Widget to be sized
	const char *group,						//!< Major heading (application name)
	const char *field,						//!< Field name
	int dftwidth,								//!< Default width to use if none in .ini file
	int dftheight								//!< Default height to use
	);

//! Set string in XmLabel widget from Unicode string.
void MxSetXmLabelString (
	Widget text,								//!< Text widget
	const MIUNICODE* const string			//!< Unicode string
	);

//! Show or hide a widget with test for NULL widget.
//! Manages or unmanages the widget to show or hide it.
void MxShowWidget (
	Widget w,									//!< Widget to show or hide (NULL ok)
	bool show = true							//!< 'true' to Manage, 'false' to Unmanage
	);

//! Draw MIUNICODE string on drawing area or other X "Window" using the default X fontlist.
//!
//! The default fontlist is read from tntproc.ini (once at program startup).
//! It can be modified by using Support / Setup / Fonts... in TNTmips
//!
//! The string passed in is assumed to be in "logical" order.  In other words,
//! if the string has right-to-left text, the right-most character comes before
//! the left-most character.  The text will be flipped to visual order as it's
//! rendered.
//!
//! Note:	This function is a replacement for XmStringDraw().  
void MxStringDraw (
	Display *display,							//!< X Display
	Window window,								//!< X Window
	GC gc,										//!< X GC
	INT32 x,										//!< x offset, upper edge of text box
	INT32 y,										//!< y offset, upper edge of text box
	const MIUNICODE* ucstr,					//!< MIUNICODE string to display
	TEXTBASEORIGIN origin,
	MGD::INTERFACETEXTSTYLE style = MGD::INTERFACETEXTSTYLE_Normal
	);

#ifdef USE_XFT
bool MxStringIsUsingAntiAlias ();

void MxStringDrawXft (
	Display *display,                                                       //!< X Display
	Window window,                                                          //!< X Window
	GC gc,                                                                          //!< X GC
	INT32 x,                                                                                //!< x offset, upper edge of text box
	INT32 y,                                                                                //!< y offset, upper edge of text box
	const MIUNICODE* ucstr,                                 //!< MIUNICODE string to display
	TEXTBASEORIGIN origin,
	COLOR color,
	const LRECT2D& ClipRectangle,
	MGD::INTERFACETEXTSTYLE style = MGD::INTERFACETEXTSTYLE_Normal
	);
#endif

//! Draw MIUNICODE string on drawing area or other X "Window" using the default X fontlist.
//!
//! This function is the same as MxStringDraw(), but assumes that the text passed
//! in is already in "visual" order.
//! <b>You almost never want this function.</b>
//! Use MxStringDraw() instead.  If there is no right-to-left text, it doesn't
//! matter which order it's in (visual == logical for most languages).  For
//! Arabic, it has to flip from visual to logical, do glyph substitutions, and
//! flip it back.
//!
//! The default fontlist is read from tntproc.ini (once at program startup).
//! It can be modified by using Support / Setup / Fonts... in TNTmips
//!
//! Note:	This function is a replacement for XmStringDraw().  It used to create
//! an XmString and call XmStringDraw(), but now it just draws it itself.
//!
//! @see MxStringDraw
void MxStringDrawVisual (
	Display *display,							//!< X Display
	Window window,								//!< X Window
	GC gc,										//!< X GC
	INT32 x,										//!< x offset, upper edge of text box
	INT32 y,										//!< y offset, upper edge of text box
	const MIUNICODE* ucstr,					//!< MIUNICODE string to display
	TEXTBASEORIGIN origin,
	MGD::INTERFACETEXTSTYLE style = MGD::INTERFACETEXTSTYLE_Normal
	);

#ifdef USE_XFT
void MxStringDrawVisualXft (
	Display *display,                                                       //!< X Display
	Window window,                                                          //!< X Window
	GC gc,                                                                          //!< X GC
	INT32 x,                                                                                //!< x offset, upper edge of text box
	INT32 y,                                                                                //!< y offset, upper edge of text box
	const MIUNICODE* ucstr,                                 //!< MIUNICODE string to display
	TEXTBASEORIGIN origin,
	COLOR color,
	const LRECT2D& ClipRectangle,
	MGD::INTERFACETEXTSTYLE style = MGD::INTERFACETEXTSTYLE_Normal
	);
#endif

//! Get width, height and ascent for a MIUNICODE string using the default X FontList.
//!
//! The default fontlist is read from tntproc.ini (once at program startup).
//! It can be modified by using Support / Setup / Fonts... in TNTmips
//!
//!	The width returned will include right bearing of the last letter
//!	which may be greater than the "width" of that letter.  (See 
//!	MxStringWidth() for explanation)
//!
//! Note:	This function is a replacement for XmStringExtents() and XmStringBaseline().
//! 
//! Used by MGD::CONTEXT::InterfaceTextGetExtents
void MxStringExtents (
	const MIUNICODE* const ucstr,			//!< MIUNICODE string to process 
	UINT32 *TextWidth,						//!< Width of string, can be NULL
	UINT32 *TextHeight,						//!< Height of string, can be NULL
	UINT32 *TextAscent,						//!< Ascent of first line of string, can be NULL
	bool bHeightForClipping,				//!< If true, height may include glyph parts which normally extend outside the design extent.  If false, it uses the height as specified by the font designer, which is more suitable	for glyph positioning.
	MGD::INTERFACETEXTSTYLE style = MGD::INTERFACETEXTSTYLE_Normal
	);

//! Used by MGD::CONTEXT::InterfaceTextGetMetrics
void MxStringMetrics (
	const MIUNICODE* const ucstr,			//!< MIUNICODE string to process 
	TEXTMETRICS& metrics,
	MGD::INTERFACETEXTSTYLE style = MGD::INTERFACETEXTSTYLE_Normal
	);

//! Return the width of an string using the default X fontlist.
//!
//! The default fontlist is read from tntproc.ini (once at program startup).
//! It can be modified by using Support / Setup / Fonts... in TNTmips
//!
//! By default, this function reports string width based on the advance width of
//! characters only.  This will give an accurate measurement for character
//! placement.  Some glyphs, however may draw to the right of their "width"
//! so as to overlap the next glyph.  To find this width, you need pass true
//! for the bInkMetrics parameter.  This would be more accurate for clipping.
//!
//! @return width in pixels
//!
int MxStringWidth (
	const MIUNICODE* const ucstr,			//!< MIUNICODE string to process
	bool bInkMetrics = false,
	MGD::INTERFACETEXTSTYLE style = MGD::INTERFACETEXTSTYLE_Normal
	);

//! Return the width of an string using the default X fontlist.
//!
//!	This function is the same as MxStringWidth, but the string is assumed
//! 	to already be in visual order.
//! @return width in pixels
//!
int MxStringWidthVisual (
	const MIUNICODE* const ucstr,			//!< MIUNICODE string to process
	bool bInkMetrics = false,
	MGD::INTERFACETEXTSTYLE style = MGD::INTERFACETEXTSTYLE_Normal
	);

//! Create form containing child forms for tabs.
//!
//! @return Unmanaged main TabPages form or NULL if error
//! The format of the MxTABPAGEITEM structure is :
//! typedef void (*TABPAGEFUNC) (void *wPage, int numPage, void* userdata);
//!		Widget wPage - Form widget for a page (item) 	/ passed/
//! 			    NOTE: this widget has individual user data attached to it,
//! 			    as specified in corresponding _MxTABPAGEITEM.
//! 	int    NumPage - page (item) number 		/passed/
//! 	void *userdata - global user data (same as passed to MxTabPagesCreate)
//! typedef struct _MxTABPAGEITEM {
//! 	char *label;
//! 	ULONG flags;
//! 	TABPAGEFUNC  create;
//! 	TABPAGEFUNC  activate;
//! 	TABPAGEFUNC  deactivate;
//! 	TABPAGEFUNC  destroy;
//! 	void *pageUserData;	 this is "page" user data and it is different from TabPages userdata 
//! 	} MxTABPAGEITEM;
//! Available flags:
//! MxTABPAGE_DISABLED	TabPage is initially not "Sensitive"
Widget MxTabPagesCreate (
	Widget pwidget,							//!< Parent widget
	MxTABPAGEITEM *items,					//!< Irems for TabPages, last element must have label of NULL
	void *userdata,							//!< Value to set "userData" resource to for main TabPages form
	Arg *arglist,								//!< Arguments for resource settings
	int argcount								//!< Number of arguments in arglist
	);

//! Change currently selected item in a TabPages.
//!
//! @return previously selected item or less 0, if error
int MxTabPagesSetActiveItem (
	Widget tbwidget,							//!< TabPages widget returned by MxTabPagesCreate
	int itemnum									//!< Item number from original MxTABPAGEITEM array
	);

//! Get currently selected item in a TabPages.
//!
//! @return currently selected item or less 0, if error
int MxTabPagesGetActiveItem (
	Widget tbwidget							//!< TabPages widget returned by MxTabPagesCreate
	);

//! Set sensitivity for a TabPages item.
//!
//! @return TRUE if successful, FALSE if not
int MxTabPagesSetSensitiveItem (
	Widget tbwidget,							//!< TabPages widget returned by MxTabPagesCreate
	int itemnum,								//!< Item number from original MxTABPAGEITEM array
	int											//!< TRUE/FALSE
	);

//! Get UserData attached to the main TabPages form.
//!
//! @return 0 if successful, less 0 if not
int MxTabPagesGetUserData (
	Widget tbwidget,							//!< TabPages widget returned by MxTabPagesCreate
	void **userdata
	);

//! Clear text and tooltip.
void MxTextClearXe (
	Widget xetext								//!< XeText widget
	);

//! Bind XeText widget to text editor.
//!
//! @return 0
int MxTextEditBindXeText (
	void *vtehandle,							//!< Text editor handle
	Widget xetext								//!< XeText widget
	);

//! Check if changes made, if so ask user if want to save and save if user wants to.
//!
//! @return Cancel, error or "no"
int MxTextEditCheckSave (
	void *vtehandle							//!< Text editor handle
	);

//! Create text editor handle.
//!
//! @return 0 or error < 0
int MxTextEditCreateHandle (
	void **vtehandle,							//!< Text editor handle
	UINT32 flags								//!< Flags
	);

//! Create scrolled XeText widget and bing to editor handle.
//!
//! @return XeText widget
Widget MxTextEditCreateXeText (
	Widget pwidget,							//!< Parent widget
	void *vtehandle,							//!< Text editor handle
	Arg *arglist,
	int argcount
	);

//! Get MENUITEM array for "Edit" menu.
//!
//! @return MENUITEM array
const MENUITEM * MxTextEditGetMenuEdit (
	void *vtehandle							//!< Text editor handle
	);

//! Get MENUITEM array for "File" menu.
//!
//! @returns MENUITEM array
const MENUITEM * MxTextEditGetMenuFile (
	void *vtehandl,							//!< Text editor handle
	UINT32 flags								//!< Flags
	);

//! Load file for text editor.
int MxTextEditLoadFile (
	void *vtehandle,							//!< Text editor handle
	MIUNICODE *filename							//!< Name of file to load
	);

//! Set function call when close menu item selected.
void MxTextEditSetCloseFunc (
	void *vtehandle,							//!< Text editor handle
	void (*CloseFunc)(void *),				//! Function call for close
	void *CloseData							//!< Data for function call
	);

//! Set default file extension used when saving.
void MxTextEditSetDftExtn (
	void *vtehandle,							//!< Text editor handle
	char *dftextn								//!< Default file extension to set
	);

//! Set encoding used when saving.
void MxTextEditSetEncoding (
	void *vtehandle,							//!< Text editor handle
	char *encoding								//!< Encoding to set
	);

//! Set XeText widget to show FILEPATH with ToolTip.
void MxTextSetFilePathXe (
	Widget xetext,								//!< XeText widget
	const FILEPATH& filepath				//!< Filepath to show
	);

//! Set XeText widget to show file/object info with ToolTip from RVC::OBJITEM.
void MxTextSetObjItemXe (
	Widget xetext,								//!< XeText widget
	const RVC::OBJITEM& objitem
	);

//! Allow user to define theme map.
//! @return Error code
int MxThemeMapEdit (
	Widget pwidget,							//!< Parent widget
	const RVC::OBJITEM& GeoObjItem,
	ELEMTYPE elemtype,						//!< Element type
	INT16& StyAsTableNum,					//!< Style-assignment table number passed/returned
	const UINT32 styletype = 0xFFFF		//!< STYLETYPE_Empty = 0xFFFF
	);

//! Set _TimeOutInMainLoop to val.
void MxTimeOutInMainLoopSet (
	int val
	);

//! Get window title as Unicode string.
//! @return Title string, MmFree() when done
//! The "update date" will be stripped from the title if necessary.
MIUNICODE *MxTitleGetString (
	Widget widget								//!< Widget (will find shell thereof)
	);

//! Set window title to given Unicode string.
void MxTitleSetString (
	Widget widget,								//!< Widget (will find shell thereof)
	const MISTRING& string					//!< String or TEXTID to use for title
	);

}	//! extern "C"
//! Set window title to given string with option whether to show process ID.
void MxTitleSetString (
	Widget widget,								//!< Widget (will find shell thereof)
	const MISTRING& string,					//!< String or TEXTID to use for title
	bool ShowProcessID						//!< Show process ID in title
	);
extern "C" {

//! Create True type symbol selecting dialog.
//! @return Error code < 0
int MxTTFSymbolSelect (
	Widget widget,								//!< Parent widget
	MISTRING& fontname,						//!< TrueType font name
	UINT16 *GlyphIndex,						//!< Internal Glyph index in TTF file (passed/returned)
	MIUNICODE *UnicodeNum						//!< Corresponded MIUNICODE number of character (NULL if not needed)
	);

//! Get top-level application shell.
//! @return widget
Widget MxTopLevel (
	void
	);

//! Forcefully remobe a server keyboard grab.
//! Removes a server grab on the keyboard.  See MxUngrabPointer() for details.
void MxUngrabKeyboard (
	Display *display							//!< The display to remove the grab from
	);

//! Forcefull remove a server pointer grab.
//! This function removes a server grab on the pointer (mouse).  If you have
//! an event loop which throws out some events, it's a good idea to call this
//! function afterwards.  When a popup menu is created it registers a passive 
//! grab on its parent with Xt.  When you click the right mouse button on 
//! the menu's parent widget, Xt calls XGrabPointer and expects the menu's
//! event handler to ungrab the pointer.  It's important to realize that this
//! is done at a lower level than the event dispatcher -- you can't prevent it.
//! If you throw out the right mouse button event, the grab never gets
//! ungrabbed and you have a mess.
void MxUngrabPointer (
	Display *display							//!< The display to remove the grab from
	);

//! Process X events until specified widget is "unmapped".
//! This is normally done in modal dialogs to process events until the dialog is closed.
void MxWaitForUnmap (
	Widget widget
	);

//! Process X events until specified widget is "unmapped", calling delegate during idle processing.
//! This is normally done in modal dialogs to process events until the dialog is closed.
//! While no events are pending, the delegate will be called repeatedly, incrementing the passed
//! "count" parameter until the delegate returns 'false'.  Once an event becomes pending, the "count"
//! will be reset to 1 the next time the delegate is called.
void MxWaitForUnmapWithIdleDelegate (
	Widget widget,
	DELEGATE_BOOL_UINT32 DelegateOnIdle
	);

//! Create "big list".
//! @return List widget
Widget MxblCreate (
	Widget pwidget,							//!< Parent widget
	int maxsize,
	Arg *formargs,
	int formn
	);

//! Update the list after a change.
void MxblRefresh (
	Widget dform,								//!< Widget returned by MxblCreate()
	INT32 start
	);

//! Change the number of items in the list.
void MxblResize (
	Widget dform,								//!< Widget returned by MxblCreate()
	INT32 numitems,
	INT32 start
	);

//! Set editing mode.
void MxblSetEditMode (
	Widget dform,								//!< Widget returned by MxblCreate()
	UINT32 mode									//!< Mode to set
	);

//! Set "selected" position in list.
void MxblSetEditPosn (
	Widget dform,								//!< Widget returned by MxblCreate()
	INT32 editposn								//!< Position to select
	);

//! Set function to be called to get list entry string.
void MxblSetEntryCallback (
	Widget dform,								//!< Widget returned by MxblCreate()
	void (*EntryCB)(INT32, char*, void*),	//! Function to call
	void *udata
	);

//! Set function to be called when user selects item in list.
//! @return 
void MxblSetHighlightCallback (
	Widget dform,								//!< Widget returned by MxblCreate()
	void (*HighlightCB)(INT32, void*),	//! Function to call
	void *udata
	);

//! Create a widget for viewing HTML.
//! @return HTML widget
Widget MxCreateHTMLWidget(
	Widget parent,								//!< Parent widget
	char *name,
	Arg *arg,
	int n
	);

#define	WaitForUnmap(w) MxWaitForUnmap(w)

#ifndef GENERATING_DOXYGEN_OUTPUT

//! Initialize application and return main application shell.
Widget _mxAppInitialize (
	int *argc,
	char *argv[],
	Arg *arglist,
	int argcount
	);

void _mxFixEncodingConverter ();

char * _mxReadDftFontList (
	void
	);

#endif //!< GENERATING_DOXYGEN_OUTPUT


}	//! extern "C"

/*----------------------------------------------------------------------------*/
/*		Inline functions																			*/
/*----------------------------------------------------------------------------*/

//! Set XmToggleButton state with check for NULL.
inline void MxToggleButtonSetState (
	Widget tb,									//!< ToggleButton (NULL allowed)
	Boolean state,								//!< State (TRUE or FALSE)
	Boolean notify=FALSE						//!< Invoke callbacks, optional, default=FALSE
	) { if (tb) XmToggleButtonSetState(tb,(state!=0),(notify!=0)); }

//------------------------------------------------------------------------------

//!@}

#ifndef DEPRECATE_ALL
//!\addtogroup tooltip ToolTip functions
//!\deprecated Replaced by MGUI::CTRL_TOOLTIP
//!@{

typedef void *TOOLTIPHANDLE;

#define	TOOLTIPFLAG_NoEventHandler		0x00000002	//!<  Caller handles events and time-outs 

//! Add a ToolTip to a Widget.
//! \deprecated Replaced by MGUI::CTRL_TOOLTIP
DEPRECATED ERRVALUE MxToolTipAdd (
	Widget widget,								//!< Widget to add ToolTip to, must have a fontlist
	const MIUNICODE *string,					//!< String to add, NULL to use labelString from widget
	TOOLTIPHANDLE *tthandle = 0,			//!< ToolTip handle returned if needed
	UINT32 flags = 0							//!< 0 or TOOLTIPFLAG_NoEventHandler
	);

//! Check cursor motion to see if tooltip should be hidden
//! \deprecated Replaced by MGUI::CTRL_TOOLTIP
//! 
//! This is only needed by code which does its own timer control to control
//! and calls MxToolTipShow() itself.
//! Pass the x_root and y_root from the MotionNotify callback as coordinates
//! If bAutoHide is true (which it is by default if not specified) and the
//! movement is enough that the tooltip should be hidden, it will it.  If
//! bAutoHide is false, this function won't hide it but will return true
//! if you should hide it.
//! @return true if cursor was (or should be) hidden, false if not.
DEPRECATED bool MxToolTipCheckMotion (
	TOOLTIPHANDLE handle,						//!< ToolTip handle
	int x_root,										//!< X coordinate of cursor (relative to root window)
	int y_root,										//!< Y coordinate of cursor (relative to root window)
	bool bAutoHide = true
	);

//! Check cursor motion to see if tooltip should be hidden
//! \deprecated Replaced by MGUI::CTRL_TOOLTIP
//! 
//! This is only needed by code which does its own timer control to control
//! and calls MxToolTipShow() itself.
//! If bAutoHide is true (which it is by default if not specified) and the
//! movement is enough that the tooltip should be hidden, it will it.  If
//! bAutoHide is false, this function won't hide it but will return true
//! if you should hide it.
//! Obviously, any event other than MotionNotify will return false.
//! @return true if cursor was (or should be) hidden, false if not.
DEPRECATED bool MxToolTipCheckMotion (
	TOOLTIPHANDLE handle,						//!< ToolTip handle
	const XEvent* ev,								//!< Event to check
	bool bAutoHide = true
	);

//! Show tool tip with specified text.
//! \deprecated Replaced by MGUI::CTRL_TOOLTIP
DEPRECATED ERRVALUE MxToolTipHide (
	TOOLTIPHANDLE handle,					//!< ToolTip handle
	UINT32 flags = 0
	);

//! Determine if tooltip is currently popped up.
//! \deprecated Replaced by MGUI::CTRL_TOOLTIP
DEPRECATED bool MxToolTipIsVisible (
	TOOLTIPHANDLE handle						//!< ToolTip handle
	);

//! Show tool tip with specified text.
//! \deprecated Replaced by MGUI::CTRL_TOOLTIP
DEPRECATED ERRVALUE MxToolTipShow (
	TOOLTIPHANDLE handle,					//!< ToolTip handle
	const MIUNICODE *text,						//!< Text to show
	UINT32 flags = 0
	);

//!@}
#endif

//------------------------------------------------------------------------------

//! Class to contain and automatically free XmString.
class XMSTR {
	public:

		//! Default constructor.
		XMSTR (
			): m_xmstr(0) { }

		//! Construct from MISTRING.  Useful for TEXTID assignments
		XMSTR (
			MISTRING const& str
			): m_xmstr(MxCreateXmStringUC(str)) { }

		//! Construct from MISTRING.  Useful for TEXTID assignments
		XMSTR (
			MIUNICODE const* str
			): m_xmstr(MxCreateXmStringUC(str)) { }

		//! Destructor.
		~XMSTR (
			) { Free(); }

		//! Assignment from MISTRING.  Useful for TEXTID assignments.
		XMSTR& operator= (
			MISTRING const& str
			) { Free(); m_xmstr = MxCreateXmStringUC(str); return (*this); }

		//! Assignment from MISTRING.  Useful for TEXTID assignments.
		XMSTR& operator= (
			MIUNICODE const* str
			) { Free(); m_xmstr = MxCreateXmStringUC(str); return (*this); }

		//! Retrieve the XmString.
		XmString GetXmString (
			) const { return (m_xmstr); }

		//! Set from specified widget labelString resource.
		void SetFromWidget (
			Widget widget
			) { Free(); Arg arg; XtSetArg(arg,XmNlabelString,&m_xmstr); XtGetValues(widget,&arg,1); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		XmString m_xmstr;

		void Free (
			) { if (m_xmstr != 0) XmStringFree(m_xmstr); }

		XMSTR (const XMSTR&);
		XMSTR& operator= (const XMSTR&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//------------------------------------------------------------------------------


#endif		// #ifndef INC_MI32_XDEFNS_H
