/**
 * \file rvc/dialogs.h
 * \brief Definitions for RVC-specific dialogs
 *
 * \if NODOC
 * $Id: dialogs.h_v 1.66 2012/07/26 21:00:20 mju Exp $
 *
 * $Log: dialogs.h_v $
 * Revision 1.66  2012/07/26 21:00:20  mju
 * Add rvc::ui::dlgGetTable/Field with mistring names.
 *
 * Revision 1.65  2011/12/09 22:34:08  mju
 * Remove redundant hdrs.
 *
 * Revision 1.64  2011/09/02 13:18:35  mju
 * Exclude macros from dox.
 *
 * Revision 1.63  2011/02/04 16:10:49  mju
 * Remove legacy dlgGeorefMake... defns.
 *
 * Revision 1.62  2010/04/21 16:48:07  mju
 * Add dlgSelectElements.
 *
 * Revision 1.61  2009/11/03 16:05:38  mju
 * Remove dlgMaintProjectFile.
 *
 * Revision 1.60  2009/04/06 15:16:54  mju
 * Move dlgDbFieldSetupPicklist to uidbtable hdr.
 *
 * Revision 1.59  2009/04/01 15:04:13  mju
 * Add dlgDbFieldSetupPickList.
 *
 * Revision 1.58  2008/09/16 21:04:42  mju
 * Remove unused editdescriptorflag values for legacy controls.
 *
 * Revision 1.57  2007/04/18 21:02:58  mju
 * Use textid__none for default titles so don't need textid hdr.
 *
 * Revision 1.56  2006/10/13 20:27:30  mju
 * No longer include dbtable hdr.
 *
 * Revision 1.55  2006/10/13 17:51:27  mju
 * If no_deprecated don't included unneeded hdrs.
 *
 * Revision 1.54  2006/10/13 14:05:36  mju
 * Use getobjflags hdr.
 *
 * Revision 1.53  2006/04/13 16:29:11  dwilliss
 * doxygen problem
 *
 * Revision 1.52  2006/02/01 14:04:38  mju
 * Revert to only having objectname versions.
 *
 * Revision 1.51  2006/01/31 17:55:22  mju
 * Add dlgSelectTable/Field using MISTRING names.
 *
 * Revision 1.50  2006/01/25 16:57:16  scowan
 * Removed unused function prototypes.
 *
 * Revision 1.49  2005/11/16 22:46:55  mju
 * Add dlgGetGeoDatabases.
 *
 * Revision 1.48  2005/08/12 15:44:37  scowan
 * Added more methods.
 *
 * Revision 1.47  2005/08/04 15:03:13  scowan
 * Added another edit object descriptor function.
 *
 * Revision 1.46  2005/08/03 21:47:51  scowan
 * Added anotehr edit object descriptor and removed a edit name desc because
 * nothing called it.
 *
 * Revision 1.45  2005/08/02 22:11:23  scowan
 * Added edit descriptor dialog calls.
 *
 * Revision 1.44  2005/07/27 17:27:24  scowan
 * Added edit object desciptor functions./
 *
 * Revision 1.43  2005/03/10 00:27:29  scowan
 * Removed unused deprecated version of dlgGetTableField.
 *
 * Revision 1.42  2005/01/04 14:53:21  dwilliss
 * Some doxygen comments to try to make it document what to include to get
 * these.
 *
 * Revision 1.41  2004/12/16 14:28:04  mju
 * Remove previously deprecated mfc-only prototype for getobjectset.
 *
 * Revision 1.40  2004/07/13 15:23:47  mju
 * Add dlgGeorefMakeSimple.
 *
 * Revision 1.39  2003/12/16 16:48:31  mju
 * Use mistring for title in EditNameDesc.
 *
 * Revision 1.38  2003/10/31 16:03:10  mju
 * Add dlgMetadataAttach.
 *
 * Revision 1.37  2003/10/30 22:48:53  dwilliss
 * Fixed description of new function.  Had cut/paste description from another
 * function.
 *
 * Revision 1.36  2003/10/30 22:38:02  dwilliss
 * Added flag for selecting just a table.
 * Added proto for function to select just a table
 *
 * Revision 1.34  2003/10/15 21:51:51  mju
 * Use const MISTRING& for all DlgGetObject... fns so don't need TEXTID variants.
 *
 * Revision 1.33  2003/10/03 20:15:16  linux32build!build
 * Don't capitalize enum any more. Genitor needed it, Doxygen doesn't like it.
 *
 * Revision 1.32  2003/09/25 21:56:37  dwilliss
 * doxygen
 *
 * Revision 1.31  2003/09/22 20:25:10  scowan
 * I though I checked this in.
 *
 * Revision 1.30  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.29  2003/09/08 20:25:53  dwilliss
 * Added doxygen tags for function groups
 *
 * Revision 1.28  2003/06/27 14:10:14  scowan
 * Resolved abiguity.
 *
 * Revision 1.27  2003/06/27 14:06:54  scowan
 * Added legacy table field selection function call.
 *
 * Revision 1.26  2003/06/25 20:53:36  scowan
 * Updated to allow select table/field to use mgui.
 *
 * Revision 1.25  2003/06/25 14:44:51  mju
 * Don't prototype DlgGetTableField except for X as not (yet) available for MFC.
 *
 * Revision 1.24  2002/09/09 20:06:03  mju
 * Move getobjfilter class defns to own headers.
 *
 * Revision 1.23  2002/09/05 21:31:26  scowan
 * Change include file name.
 *
 * Revision 1.22  2002/09/03 22:04:49  mju
 * Add GETOBJFILTER_RASTER.
 * Change GETOBJFILTER virtual method naming and make private.
 *
 * Revision 1.21  2002/09/03 15:16:09  mju
 * Change filters to have Clear() and error value support.
 *
 * Revision 1.20  2002/08/30 14:06:17  mju
 * Move SELTABFIELD flags here and put in RVC namespace.
 *
 * Revision 1.19  2002/08/19 15:40:21  mju
 * Remove deprecated DlgGetObject... versions using msg lookup.
 * Add GetObjectSet versions using textid for labels.
 *
 * Revision 1.17  2002/08/12 13:29:19  mju
 * Add DlgTableEditScaleSelection.
 *
 * Revision 1.16  2002/06/06 15:49:49  mju
 * Add TEXTID versions and deprecate promptkey versions.
 *
 * Revision 1.15  2002/05/16 16:32:04  scowan
 * Changed get object filter to use text id's.
 *
 * Revision 1.14  2002/03/25 21:45:26  scowan
 * Added filter to table field selection code.
 *
 * Revision 1.13  2002/02/15 16:37:08  mju
 * Add SELTABFIELD parm to GetTableField().
 *
 * Revision 1.12  2002/02/15 15:30:04  mju
 * Add description comment for GetTableField.
 *
 * Revision 1.11  2002/02/14 18:13:17  mju
 * Add DlgTableEditSelection, DlgGetTableField prototypes.
 *
 * Revision 1.6  2001/07/16 14:11:18  mju
 * Add DlgEditObjectNameDesc.
 *
 * Revision 1.5  2001/06/18 19:47:08  mju
 * Change default for 'minobjs' to 1.
 *
 * Revision 1.4  2001/06/18 16:30:40  mju
 * Add inigroup parameter.
 * Remove numobjs from GetObjectSet().
 *
 * Revision 1.3  2001/06/15 22:11:23  mju
 * Add MaintProjectFile function.
 *
 * Revision 1.1  2001/06/13 20:10:10  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_DIALOGS_H
#define	INC_RVC_DIALOGS_H

#ifndef	INC_RVC_OTYPESET_H
#include <rvc/otypeset.h>
#endif

#ifndef	INC_RVC_GETOBJFLAGS_H
#include <rvc/getobjflags.h>
#endif

#ifndef	INC_RVC_OBJECTNAME_H
#include <rvc/objectname.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

// Legacy inclusion
#if !defined(INC_RVC_GETOBJFILTER_H) && !defined(NO_DEPRECATED)
#include <rvc/getobjfilter.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class FILEPATH;
class UNSIGNEDSET;
namespace RVC {
	class GETOBJFILTER;
	class GETTABLEFIELDFILTER;
	class DESCRIPTOR;
	class OBJITEM;
	class OBJITEMLIST;
	}
#endif


namespace RVC {

//! Flags for RVC::DlgGetTableField.
enum SELTABFIELD {
	SELTABFIELD_Default =			0x0000,	//!< Default selection
	SELTABFIELD_AllowNoTable =		0x0001,	//!< Allow "no table" selection
	SELTABFIELD_AllowNone =			0x0001,	//!< Allow "none" selection
	SELTABFIELD_NoInternal =		0x0008,	//!< Don't allow 'internal' tables to be selected
	SELTABFIELD_TableOnly =		   0x0010,	//!< Only prompt for table, not field
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(SELTABFIELD)
#endif



//! Flags for RVC::DlgEditObjectDescriptor and RVC::DlgEditFileDescriptor
enum EDITDESCRIPTORFLAGS {
	EDITDESCRIPTORFLAG_Default = 					0x0000,
	EDITDESCRIPTORFLAG_NoModifyName =		 	0x0008
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(EDITDESCRIPTORFLAGS)
#endif


//!\addtogroup DialogFunctions	Dialog Functions
//!@{

//! Dialog to allow user to edit RVC object ACL settings
ERRVALUE DlgEditACL (
	MDLGPARENT dlgparent,
	const MISTRING& title,						//!< Title string, can use TEXTID also
	const RVC::OBJITEM& ObjItem,				//!< if creating a new object, then ObjItem.IsNewRequest() == true and parent must be correct
	SIMPLE_ARRAY<UINT8>& buffer
	);
	
//! Dialog to allow user to edit RVC file descriptor values
ERRVALUE DlgEditFileDescriptor (
	MDLGPARENT dlgparent,								//!< Parent for dialog
	RVC::DESCRIPTOR& Descriptor,						//!< Descriptor to edit
	const FILEPATH& ParentDirectory,					//!< Parent directory that this file is created in, used for call to FILEPATH::GetNameSizeLimit()
	const MISTRING& title = TEXTID__None,
	EDITDESCRIPTORFLAGS flags = EDITDESCRIPTORFLAG_Default
	);

//! Dialog to allow user to edit RVC object descriptor values
ERRVALUE DlgEditObjectDescriptor (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	DESCRIPTOR& Descriptor,					//!< Descriptor to edit
	const MISTRING& title = TEXTID__None,
	EDITDESCRIPTORFLAGS flags = EDITDESCRIPTORFLAG_Default
	);

//! Dialog to allow user to edit RVC object descriptor values
ERRVALUE DlgEditObjectDescriptor (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	DESCRIPTOR& Descriptor,					//!< Descriptor to edit
	FastDelegate<bool(RVC::OBJECTNAME&)> delegate,
	const MISTRING& title = TEXTID__None,
	EDITDESCRIPTORFLAGS flags = EDITDESCRIPTORFLAG_Default
	);

//! Dialog to allow user to edit RVC object descriptor values and check for conflicts with existing objects
//! If an existing object is being edited the 'ObjItem' must be attached to it.
//! If a new object is being edited the 'ObjItem' should be setup via OBJITEM::CreateNew().
ERRVALUE DlgEditObjectDescriptor (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	OBJITEM& ObjItem,					//!< OBJITEM containing descriptor to edit
	const MISTRING& title = TEXTID__None,
	EDITDESCRIPTORFLAGS flags = EDITDESCRIPTORFLAG_Default
	);

//! Dialog to allow user to edit RVC object name, description and other user settable values
ERRVALUE DlgEditObjectInfo (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& ObjItem			//!< Item for object to edit
	);
	
//! Dialog to select geodatabase connections.
ERRVALUE DlgGetGeoDatabases (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& promptstr,					//!< Prompt string to show at top of dialog (also allows TEXTID)
	RVC::OBJITEMLIST& objlist,					//!< Object list returned
	RVC::GETOBJFILTER *filter = 0				//!< Filter to determine if object is selectable
	);


//! Dialog to select a single object.
ERRVALUE DlgGetObject (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& promptstr,					//!< Prompt string to show at top of dialog (also allows TEXTID)
	const RVC::OBJTYPESET& objtypes,			//!< Object type(s) allowed for selection
	RVC::OBJITEM& objitem,						//!< Selected object passed/returned
	GETOBJFLAGS flags = GETOBJFLAG_None,	//!< Flags
	RVC::GETOBJFILTER *filter = 0,			//!< Filter to determine if object is selectable
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);


//! Dialog to select a variable number of objects.
ERRVALUE DlgGetObjects (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& promptstr,					//!< Prompt string to show at top of dialog (also allows TEXTID)
	const RVC::OBJTYPESET& objtypes,			//!< Object type(s) allowed for selection
	RVC::OBJITEMLIST& objlist,					//!< Object list returned
	GETOBJFLAGS flags = GETOBJFLAG_None,	//!< Flags
	RVC::GETOBJFILTER *filter = 0,			//!< Filter to determine if object is selectable
	int minobjs = 1,								//!< Minimum number of objects required
	int maxobjs = 0,								//!< Maximum allowed to be selected, 0 if no limit
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);


//! Dialog to select a specific set of objects, TEXTID labels.
ERRVALUE DlgGetObjectSet (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& promptstr,					//!< Prompt string to show at top of dialog (also allows TEXTID)
	const RVC::OBJTYPESET& objtypes,			//!< Object type(s) allowed for selection
	const TEXTID labelids[],					//!< Label IDs for each object
	RVC::OBJITEMLIST& objlist,					//!< Object list passed/returned
	GETOBJFLAGS flags = GETOBJFLAG_None,	//!< Flags
	RVC::GETOBJFILTER *filter = 0,			//!< Filter to determine if object is selectable
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);


//! Dialog to select a specific set of objects, MISTRING labels.
ERRVALUE DlgGetObjectSet (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& promptstr,					//!< Prompt string to show at top of dialog (also allows TEXTID)
	const RVC::OBJTYPESET& objtypes,			//!< Object type(s) allowed for selection
	const MISTRING labelstr[],					//!< Label names for each object
	RVC::OBJITEMLIST& objlist,					//!< Object list passed/returned
	GETOBJFLAGS flags = GETOBJFLAG_None,	//!< Flags
	RVC::GETOBJFILTER *filter = 0,			//!< Filter to determine if object is selectable
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);


//! Dialog to select a specific set of objects, labels derived from object list provided.
ERRVALUE DlgGetObjectSet (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const MISTRING& promptstr,					//!< Prompt string to show at top of dialog (also allows TEXTID)
	const RVC::OBJTYPESET& objtypes,			//!< Object type(s) allowed for selection
	RVC::OBJITEMLIST& objlist,					//!< Object list passed/returned
	GETOBJFLAGS flags = GETOBJFLAG_None,	//!< Flags
	RVC::GETOBJFILTER *filter = 0,			//!< Filter to determine if object is selectable
	const char *inigroup = 0					//!< INI group for default file, will use process if 0
	);


//! Dialog to select subobject.
//! Also automatically allows selection of table if parent is a database and only OBJTYPE_Table specified.
//! If the specified 'objitem' is not a child of 'parentobjitem' then it will be ignored.
ERRVALUE DlgGetSubObject (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const RVC::OBJITEM& parentobjitem,		//!< Parent object
	const MISTRING& promptstr,					//!< Prompt string to show at top of dialog (also allows TEXTID)
	const RVC::OBJTYPESET& objtypes,			//!< Object type(s) allowed for selection
	RVC::OBJITEM& objitem,						//!< Selected object passed/returned
	GETOBJFLAGS flags = GETOBJFLAG_None,	//!< Flags
	RVC::GETOBJFILTER *filter = 0				//!< Filter to determine if object is selectable
	);


//! Dialog to select table from database object.
ERRVALUE DlgGetTable (
	MDLGPARENT dlgparent,									//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,						//!< Database object item reference
	RVC::OBJECTNAME& TableNameShort,						//!< Table name passed/returned
	TEXTID titleid = TEXTID__None,						//!< Dialog title, default is TEXTID_Title_SelectTable
	SELTABFIELD flags = SELTABFIELD_TableOnly,		//!< Flags for allowed selections
	GETTABLEFIELDFILTER *pFilter = 0
	);
	
//! Dialog to select table and field from a database object.
ERRVALUE DlgGetTableField (
	MDLGPARENT dlgparent,									//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,						//!< Database object item reference
	RVC::OBJECTNAME& TableNameShort,						//!< Table name passed/returned
	RVC::OBJECTNAME& FieldNameShort,						//!< Field name passed/returned
	TEXTID titleid = TEXTID__None,						//!< Dialog title, default is TEXTID_Title_SelectTableField
	SELTABFIELD flags = SELTABFIELD_Default,			//!< Flags for allowed selections
	GETTABLEFIELDFILTER *pFilter = 0
	);
	
//! Dialog to prompt user for objects and attach metadata to them.
ERRVALUE DlgMetadataAttach (
	MDLGPARENT dlgparent							//!< Parent for dialog
	);

//! Recover RVC data from corrupt files.
ERRVALUE DlgRecoverProjectFile (
	MDLGPARENT dlgparent,
	bool mainproc									//!< Behave as though main process rather than child dialog
	);

//! Display and select elements using multiple selection tools.
ERRVALUE DlgSelectElements (
	MDLGPARENT dlgparent,
	const RVC::OBJITEM& ObjItem,
	ELEMTYPE ElemType,
	UNSIGNEDSET& ResultSet
	);

//! Dialog for editing "scale selection" table.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditScaleSelection (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,			//!< Database object item reference
	ELEMTYPE elemtype,							//!< Element type
	RVC::OBJECTNAME& tablename					//!< Selection table name passed/returned
	);


//! Dialog for editing "selection" table.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditSelection (
	MDLGPARENT dlgparent,						//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,			//!< Database object item reference
	ELEMTYPE elemtype,							//!< Element type
	RVC::OBJECTNAME& tablename					//!< Selection table name passed/returned
	);

namespace UI {
//! Dialog to select table from database object.
ERRVALUE DlgGetTable (
	MDLGPARENT dlgparent,									//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,						//!< Database object item reference
	MISTRING& TableName,										//!< Table name passed/returned
	TEXTID titleid = TEXTID__None,						//!< Dialog title, default is TEXTID_Title_SelectTable
	SELTABFIELD flags = SELTABFIELD_TableOnly,		//!< Flags for allowed selections
	GETTABLEFIELDFILTER *pFilter = 0
	);
	
//! Dialog to select table and field from a database object.
ERRVALUE DlgGetTableField (
	MDLGPARENT dlgparent,									//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,						//!< Database object item reference
	MISTRING& TableName,										//!< Table name passed/returned
	MISTRING& FieldName,										//!< Field name passed/returned
	TEXTID titleid = TEXTID__None,						//!< Dialog title, default is TEXTID_Title_SelectTableField
	SELTABFIELD flags = SELTABFIELD_Default,			//!< Flags for allowed selections
	GETTABLEFIELDFILTER *pFilter = 0
	);

}	// End namespace UI

//!@}

}	// End of RVC namespace


#endif	// INC_RVC_DIALOGS_H
