/**
 * \file rvc/dispgrp.h
 * \brief interface for the RVC::DISPGROUP class.
 *
 * \if NODOC
 * $Id: dispgrp.h_v 1.8 2006/09/08 19:28:06 scowan Exp $
 *
 * $Log: dispgrp.h_v $
 * Revision 1.8  2006/09/08 19:28:06  scowan
 * Change enum from serialize to its own enum.
 *
 * Revision 1.7  2005/06/30 21:01:52  scowan
 * Added makeparms.
 *
 * Revision 1.6  2003/10/07 17:34:21  scowan
 * Object virtual method changes.
 *
 * Revision 1.5  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.4  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.3  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2001/06/15 17:33:21  scowan
 * Added to RVC namespace.
 *
 * Revision 1.1  2000/11/29 17:09:49  scowan
 * Initial revision
 *
 * \endif
**/

//!
//!	\class RVC::DISPGROUP rvc/dispgrp.h
//!	\brief Manages RVC::OBJTYPE_Display_Group objects.
//!
//!	The RVC::DISPGROUP class manages RVC::OBJTYPE_Display_Group objects.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Display_Group
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Display_Group
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder

#ifndef INC_RVC_DISPGRP_H
#define INC_RVC_DISPGRP_H

#ifndef INC_RVC_SERIALZE_H
	#include <rvc/serialze.h>
#endif

namespace RVC {

class DISPGROUP : public RVC::SERIALIZE {
	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class MAKEPARMS;
		#endif

		//! Default constructor
		DISPGROUP (
			);

		//! Copy Constructor
		DISPGROUP (
			const RVC::DISPGROUP& obj
			);

		//! Destructor
		virtual ~DISPGROUP (
			);

		//! Assignment
		RVC::DISPGROUP& operator= (
			const RVC::DISPGROUP& rhs
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


class DISPGROUP::MAKEPARMS : public SERIALIZE::MAKEPARMS {
	public:
		MAKEPARMS (
			UINT16 Version = 1				 			//!< Version number to assign to new object
			) : SERIALIZE::MAKEPARMS(RVC::OBJTYPE_Display_Group, Version, RVC::OBJTYPE_All, DISPPARMUSAGE_Display)
			{}
	};			

}		//! End of RVC namespace

#endif 	//!< INC_RVC_DISPGRP_H
