/**
 * \file <rvc/oprasterproperties.h>
 * \brief RVC::OP::RASTER_GEOMETRIC_PROPERTIES definition
 *
 * \if NODOC
 * $Id: oprasterproperties.h_v 1.5 2011/03/16 13:21:58 mju Exp $
 *
 * $Log: oprasterproperties.h_v $
 * Revision 1.5  2011/03/16 13:21:58  mju
 * Fix doc comments.
 *
 * Revision 1.1  2007/01/03 16:51:41  drb
 * Initial revision
 *	\endif
**/

#ifndef INC_RVC_OP_RASTER_PROPERTIES_H
#define INC_RVC_OP_RASTER_PROPERTIES_H

#ifndef INC_MI32_RPOVRGN_H
	#include <mi32/rpovrgn.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
	class OBJITEM;
	class DBASE_POINT;
	class DBASE_POLYGON;
#endif // GENERATING_DOXYGEN_OUTPUT

namespace OP {

//! Class used to compute Raster Properties for a geometric object
class RASTER_GEOMETRIC_PROPERTIES {
	public:

		enum TABLEDESCRIPTOR {
			TABLEDESCRIPTOR_UseDefault,
			TABLEDESCRIPTOR_EditDialog,
			TABLEDESCRIPTOR_SetManually
			};

		RASTER_GEOMETRIC_PROPERTIES (
			) : 
			m_ComputeHistogram(false),
			m_IncludeIslands(false),
			m_BoundaryCells(RASTER_REGION_PROP_CALC::BOUNDARYCELLS_IncludeAll),
			m_PopupTableDescriptorDialog(false),
			m_PointTableDescriptorMethod(RVC::OP::RASTER_GEOMETRIC_PROPERTIES::TABLEDESCRIPTOR_UseDefault),
			m_PolygonTableDescriptorMethod(RVC::OP::RASTER_GEOMETRIC_PROPERTIES::TABLEDESCRIPTOR_UseDefault),
			m_HistogramTableDescriptorMethod(RVC::OP::RASTER_GEOMETRIC_PROPERTIES::TABLEDESCRIPTOR_UseDefault)
			{
			}

		~RASTER_GEOMETRIC_PROPERTIES ( ) { }

		//! Compute the raster properties for the rasters using the source vector
		ERRVALUE ComputeRasterProperties ( 
			RVC::OBJITEM& VectObjItem,			//! Source Vector 
			RVC::OBJITEMLIST& RastObjItemList,	//! Source Rasters 
			MISTRING& report					//! Report of computations
			);

		//! Get type of boundary cells to be used
		RASTER_REGION_PROP_CALC::BOUNDARYCELLS GetBoundaryCells ( 
			) {	return m_BoundaryCells; }

		//! Set type of boundary cells to use
		void SetBoundaryCells ( RASTER_REGION_PROP_CALC::BOUNDARYCELLS BoundaryCells )
			{ m_BoundaryCells = BoundaryCells; }
		
		//! Get whether to compute histograms
		bool GetComputeHistograms ( )
			{ return m_ComputeHistogram; }

		//! Set whether to compute histograms
		void SetComputeHistograms ( bool ComputeHistogram ) 
			{ m_ComputeHistogram = ComputeHistogram; }

		//! Get whether to include island polygons
		bool GetIncludeIslands ( ) 
			{ return m_IncludeIslands; }

		//! Set whether to include island polygons
		void SetIncludeIslands ( bool IncludeIslands ) 
			{ m_IncludeIslands = IncludeIslands; }

		//! Set how the point table descriptor is generated
		void SetPointTableDescriptor (
			TABLEDESCRIPTOR method,				//! Method to use
			RVC::DESCRIPTOR* descriptor = 0		//! Used only with _SetManually
			);

		//! Set how the polygon table descriptor is generated
		void SetPolygonTableDescriptor (
			TABLEDESCRIPTOR method,				//! Method to use
			RVC::DESCRIPTOR* descriptor = 0		//! Used only with _SetManually
			);

		//! Set how the histogram table descriptor is generated
		void SetHistogramTableDescriptor (
			TABLEDESCRIPTOR method,				//! Method to use
			RVC::DESCRIPTOR* descriptor = 0		//! Used only with _SetManually
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RASTER_REGION_PROP_CALC::BOUNDARYCELLS m_BoundaryCells;
		bool m_ComputeHistogram;
		bool m_IncludeIslands;
	
		TABLEDESCRIPTOR m_PointTableDescriptorMethod;
		RVC::DESCRIPTOR m_PointTableDescriptor;
		TABLEDESCRIPTOR m_PolygonTableDescriptorMethod;
		RVC::DESCRIPTOR m_PolygonTableDescriptor;
		TABLEDESCRIPTOR m_HistogramTableDescriptorMethod;
		RVC::DESCRIPTOR m_HistogramTableDescriptor;

		bool m_PopupTableDescriptorDialog;

		INT32 MakeStatisticsTableForPoints(RVC::DBASE_POINT& pointdbase, RVC::RASTER& raster);
		INT32 MakeStatisticsTableForPolygons(RVC::DBASE_POLYGON& polygondbase, RVC::RASTER& raster);
		INT32 MakeHistogramTableForPolygons(RVC::DBASE_POLYGON& polygondbase, RVC::RASTER& raster);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

	} // End OP namespace
} // End RVC namespace


#endif 	// INC_RVC_OP_RASTER_PROPERTIES_H

