/**
 * \file rvc/oprastertoregion.h
 * \brief Operation to convert raster to region
 *
 * \if NODOC
 * $Id: oprastertoregion.h_v 1.3 2011/10/13 21:12:05 mju Exp $
 *
 * $Log: oprastertoregion.h_v $
 * Revision 1.3  2011/10/13 21:12:05  mju
 * Remove redundant hdr.
 *
 * Revision 1.2  2011/09/02 12:53:15  mju
 * Dox hdr.
 *
 * Revision 1.1  2007/04/02 17:08:28  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_OPRASTERTOREGION_H
#define	INC_OPRASTERTOREGION_H

#ifndef INC_MI32_DOUBLEAR_H
	#include <mi32/doublear.h>
#endif

#ifndef INC_MI32_RANGE_H			// For DOUBLE_RANGE definition
	#include <mi32/range.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT  
class REGION2D;
#endif // GENERATING_DOXYGEN_OUTPUT

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT  
class OBJITEM;
#endif // GENERATING_DOXYGEN_OUTPUT

namespace OP {

//=====================================================================================================================

//! Converter for performing raster to region conversion
class RASTER_TO_REGION {
	public:

	//! Compute region around selected raster cell values.
	//! The returned region coordinates will be in raster (column, line) coordinates.
	static ERRVALUE ComputeFromValues (
		const OBJITEM& ObjItem,								//!< Raster object to compute region on
		const DOUBLE_ARRAY<double>& values,				//!< Cell values list allocated
		double range, 											//!< Range for cell values (value +/- limit) to be included to region
		REGION2D& region										//!< Region returned in raster (column, line) coordinates
		);
	
	//! Compute region from binary raster.
	//! The returned region coordinates will be in raster (column, line) coordinates.
	static ERRVALUE ComputeFromBinary (
		const OBJITEM& ObjItem,								//!< Raster object to compute region on
		REGION2D& region										//!< Region returned in raster (column, line) coordinates
		);
		
	//! Compute regions from thresholded (by number of ranges) raster.
	static ERRVALUE ComputeFromValueRanges (
		const OBJITEM& ObjItem,								//!< Input raster object
		const DOUBLE_ARRAY<DOUBLE_RANGE>& ranges,		//!< ranges of raster values to be included to regions
		REGION2D& region										//!< Region returned in raster (column, line) coordinates
		);
	};

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_OPRASTERTOREGION_H
