/******************************************************************************
 *
 * \file sml/vectorobject.h
 * \brief SML VECTOROBJECT
 *
 * \if NODOC
 * $Log: vectorobject.h_v $
 * Revision 1.2  2007/05/08 15:46:41  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 21:07:19  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_VECTOROBJECT_H
#define INC_SML_VECTOROBJECT_H

#ifndef INC_SML_OBJECT_H
#include <sml/object.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class VECTOR;
	class VECTORHANDLE;
	class VECTORTOOLKIT;
	}
#endif

namespace SML {

/*****************************************************************************/
/*
*	class SML::VECTOROBJECT
*/
class RVCSYSLIBCLASSEXPORT VECTOROBJECT : public OBJECT {
	public:
		//! Get a pointer to the RVC::VECTOR interface class.
		//! Do NOT free this pointer.  It will be destroyed when the SML::OBJECT is
		//! destroyed.  The interface returned is Attached to the object but may
		//! or may not be open.
		//! If the SML::BOJECT does not refer to any existing real object, this
		//! method will return NULL.
		RVC::VECTOR* GetInterface (
			) const;

		UINT32 GetToolkitFlags (
			) const;

		RVC::VECTORTOOLKIT* GetToolkitHandle (
			);				//!<  used for writing to vector

		//! Set a pointer to the RVC::VECTOR interface class.
		//! After setting the interface, the SML::OBJECT "owns" it and will 
		//! delete it in its own destructor.  Do not under any circumstances
		//! pass a pointer to an local variable that will go out of scope.
		//! Since SML::OBJECT is implemented in rvcsys.dll it does not have
		//! have access to the copy constructor.
		void SetInterface (
			RVC::VECTOR* obj
			);

		void SetToolkitHandle (
			RVC::VECTORTOOLKIT* vtk
			);

		void SetToolkitFlags (
			UINT32 flags
			);

		//! Get the RVC::VECTORHANDLE of the open vector
		//! Note: this will open the vector if necessary and leave it open
		ERRVALUE GetHandle (
			RVC::VECTORHANDLE*& handle
			);

	private:
#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Constructor
		VECTOROBJECT (
			CONTEXT* context
			);

		//! Destructor
		virtual ~VECTOROBJECT (
			);

		virtual int v_OpenLow (
			int mode
			);

		virtual void v_CloseLow (
			);

		virtual RVC::OBJTYPE v_GetObjType (
			) const;

		RVC::VECTORTOOLKIT* m_vtk;
		RVC::VECTORHANDLE* m_handle;
		UINT32 m_CurrentFlags;						//!<  flags for vector toolkit

		friend class CONTEXT;		//!< Only SML::CONTEXT can call our ctor/dtor
		friend class OBJECT;			//!< All our virtual methods are private and called through SML::OBJECT
#endif // GENERATING_DOXYGEN_OUTPUT
	};

} // end of SML namespace

typedef SML::VECTOROBJECT SMLVECTOROBJECT;

#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_VECTOROBJECT_H


