/**
 * \file imagefilteroverridetype.h <spatmod/imagefilteroverridetype.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_OVERRIDETYPE definition.
 *
 * \if NODOC
 * $Id: imagefilteroverridetype.h_v 1.1 2006/11/27 18:21:29 mju Exp $
 *
 * $Log: imagefilteroverridetype.h_v $
 * Revision 1.1  2006/11/27 18:21:29  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTEROVERRIDETYPE_H
#define	INC_SPATMOD_IMAGEFILTEROVERRIDETYPE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to override pixel type of an image.
//! Does not actually change the image data at all, only changes the apparent pixel type.
class FILTER_OVERRIDETYPE : public FILTER {
	public:

		//! Construct with specified target pixel type.
		FILTER_OVERRIDETYPE (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			PIXEL::TYPE PixelType					//!< Override pixel type
			);

		// Destructor.
		virtual ~FILTER_OVERRIDETYPE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PIXEL::TYPE m_PixelType;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_OVERRIDETYPE (const FILTER_OVERRIDETYPE&);
		FILTER_OVERRIDETYPE& operator= (const FILTER_OVERRIDETYPE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTEROVERRIDETYPE_H

