/**
 * \file spatmod/imagesourcetileset.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_BINGMAPS definition.
 *
 * \if NODOC
 * $Id: imagesourcebingmaps.h_v 1.12 2012/02/06 16:22:51 bcolombe Exp $
 *
 * $Log: imagesourcebingmaps.h_v $
 * Revision 1.12  2012/02/06 16:22:51  bcolombe
 * Fix header include
 *
 * Revision 1.11  2011/04/05 15:33:43  bcolombe
 * Add resample method
 *
 * Revision 1.10  2011/04/04 16:06:10  bcolombe
 * Add v_ProcessBottomTile
 *
 * Revision 1.9  2011/03/21 19:28:48  mju
 * Add getCopyrightString.
 *
 * Revision 1.8  2011/02/07 23:18:14  bcolombe
 * Add DrawOverlay
 *
 * Revision 1.7  2011/02/04 21:58:35  bcolombe
 * Add v_GetDataTip
 *
 * Revision 1.6  2011/02/04 20:31:59  bcolombe
 * Add SetupURLs
 *
 * Revision 1.5  2011/02/04 18:30:56  bcolombe
 * Derive from SOURCE_TILESET
 *
 * Revision 1.4  2011/02/02 22:45:08  bcolombe
 * Remove unnecessary header
 *
 * Revision 1.3  2011/01/28 23:20:06  bcolombe
 * Add SetMapStyle
 *
 * Revision 1.2  2011/01/24 20:26:57  bcolombe
 * Changes to make multi-threaded
 *
 * Revision 1.1  2011/01/24 18:09:09  bcolombe
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEBINGMAPS_H
#define	INC_SPATMOD_IMAGESOURCEBINGMAPS_H

#ifndef	INC_SPATMOD_IMAGESOURCETILESET_H
#include <spatmod/imagesourcetileset.h>
#endif

#ifndef INC_SPATMOD_IMAGESAMPLEITERATOR_H
#include <spatmod/imagesampleiterator.h>
#endif

#ifndef INC_SPATMOD_IMAGESTAGEAUTOPTR_H
#include <spatmod/imagestageautoptr.h>
#endif

#ifndef INC_GRE_LAYERDC_H
#include <gre/layerdc.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

class SOURCE_BINGMAPS : public SOURCE_TILESET {
	public:
		
		enum MAPSTYLE {
			MAPSTYLE_Aerial =						0,
			MAPSTYLE_Aerial_v0 =					1,
			MAPSTYLE_Aerial_v1 =					2,
			MAPSTYLE_AerialWithLabels =		3,
			MAPSTYLE_AerialWithLabels_v0 =	4,
			MAPSTYLE_AerialWithLabels_v1 =	5,
			MAPSTYLE_Birdseye =					6,
			MAPSTYLE_Birdseye_v0 =				7,
			MAPSTYLE_Birdseye_v1 =				8,
			MAPSTYLE_BirdseyeWithLabels =		9,
			MAPSTYLE_BirdseyeWithLabels_v0 =	10,
			MAPSTYLE_BirdseyeWithLabels_v1 =	11,
			MAPSTYLE_Road =						12,
			MAPSTYLE_Road_v0 =					13,
			MAPSTYLE_Road_v1 =					14
			};

		//! Constructor.
		SOURCE_BINGMAPS (MAPSTYLE MapStyle);

		// Destructor.
		virtual ~SOURCE_BINGMAPS ();

		ERRVALUE DrawOverlay (const GRE::LAYERDC& layerdc);

		//! Get copyright string.
		const MISTRING& GetCopyrightString () const
			{ return (m_CopyrightString); }

		void SetMapStyle (MAPSTYLE MapStyle);

		void SetResampleMethod (const SPATMOD::IMAGE::RESAMPLEMETHOD& ResampleMethod) {m_ResampleMethod = ResampleMethod;}

	protected:
		virtual ERRVALUE v_GetDatatip (const DPOINT2D& point, MISTRING& string, INT32 ZoomLevelToUse, SOURCE_TILESET::DATATIPFLAG flags);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MAPSTYLE m_MapStyle;
		MISTRING m_CopyrightString;
		MISTRING m_LogoURL;
		SPATMOD::IMAGE::RESAMPLEMETHOD m_ResampleMethod;
		
		void GetMapStyleString (MISTRING& MapStyleStr);
		void GetXMLString (MISTRING& xmlstr);
		void GetXMLString (MISTRING& xmlstr, INT32 ZoomLevelToUse, const LPOINT2D& TileIdx);
		ERRVALUE SetupURLs();

		// SOURCE_TILESET overrides.
		virtual void v_GetCacheName (MISTRING& CacheName);
		virtual ERRVALUE v_GetDatatipObjectNames (const DPOINT2D& point, MISTRING& string, INT32 ZoomLevelToUse, int indent);
		virtual ERRVALUE v_ProcessBottomTile (SPATMOD::IMAGE::STAGE*& source, INT32 ZoomLevelToUse, const LPOINT2D& TileIdx);

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		SOURCE_BINGMAPS (const SOURCE_BINGMAPS&);
		SOURCE_BINGMAPS& operator= (const SOURCE_BINGMAPS&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEGOOGLEMAPS_H



